/***************************************************************************
               buttonwidgetbase.cpp  -  base class for button widgets
                             -------------------
    begin                : Sat Sep 20 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>

#include "utils.h"
#include "../errors.h"
#include "buttonwidgetbase.h"
#include "membermethodhandler.h"
#include "memberfunctionproperty.h"
#include "pagebase.h"
 
using namespace std;

extern const int DEFAULT_FONTSIZE;
extern const eboxy_color DEFAULT_FONT_COLOR;
extern const eboxy_color DEFAULT_SELECTED_FONT_COLOR;
extern const char *TYPESTR_BUTTON;

extern ResourceManagerBase *resourcemanager;

ButtonWidgetBase::ButtonWidgetBase(const char *name, int x, int y, int w, int h)
: WidgetBase(x, y) {
  this->w = w;
  this->h = h;
  // Set defaults
  tabstop = true;
  font = NULL;
  action = "";
  caption = "";
  captionx = 0;
  captiony = 0;
  captiondropx = 2;
  captiondropy = 2;
  normalimage = NULL;
  selectedimage = NULL;
  pressedimage = NULL;
  disabledimage = NULL;
  fontcolor = DEFAULT_FONT_COLOR;
  fontfocuscolor = DEFAULT_FONT_COLOR;
  fontdisabledcolor = DEFAULT_FONT_COLOR;
  fontpressedcolor = DEFAULT_FONT_COLOR;
  fontfocuscolorused = false;
  fontpressedcolorused = false;
  fontdisabledcolorused = false;
  fontsize = DEFAULT_FONTSIZE;
  enabled = true;
  this->type = TYPESTR_BUTTON;

  MemberFunctionProperty<ButtonWidgetBase> *mp;

  mp = new MemberFunctionProperty<ButtonWidgetBase>("caption", this, &ButtonWidgetBase::pget_Caption, &ButtonWidgetBase::pset_Caption, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ButtonWidgetBase>("font", this, &ButtonWidgetBase::pget_Font, &ButtonWidgetBase::pset_Font, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ButtonWidgetBase>("fontsize", this, &ButtonWidgetBase::pget_FontSize, &ButtonWidgetBase::pset_FontSize, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ButtonWidgetBase>("fontcolor", this, &ButtonWidgetBase::pget_FontColor, &ButtonWidgetBase::pset_FontColor, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ButtonWidgetBase>("fontfocuscolor", this, &ButtonWidgetBase::pget_FontFocusColor, &ButtonWidgetBase::pset_FontFocusColor, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ButtonWidgetBase>("fontpressedcolor", this, &ButtonWidgetBase::pget_FontPressedColor, &ButtonWidgetBase::pset_FontPressedColor, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ButtonWidgetBase>("fontdisabledcolor", this, &ButtonWidgetBase::pget_FontDisabledColor, &ButtonWidgetBase::pset_FontDisabledColor, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ButtonWidgetBase>("image", this, &ButtonWidgetBase::pget_Image, &ButtonWidgetBase::pset_Image, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ButtonWidgetBase>("selectedimage", this, &ButtonWidgetBase::pget_SelectedImage, &ButtonWidgetBase::pset_SelectedImage, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ButtonWidgetBase>("pressedimage", this, &ButtonWidgetBase::pget_PressedImage, &ButtonWidgetBase::pset_PressedImage, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ButtonWidgetBase>("disabledimage", this, &ButtonWidgetBase::pget_DisabledImage, &ButtonWidgetBase::pset_DisabledImage, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ButtonWidgetBase>("captionx", this, &ButtonWidgetBase::pget_CaptionX, &ButtonWidgetBase::pset_CaptionX, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ButtonWidgetBase>("captiony", this, &ButtonWidgetBase::pget_CaptionY, &ButtonWidgetBase::pset_CaptionY, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ButtonWidgetBase>("captiondropx", this, &ButtonWidgetBase::pget_CaptionDropX, &ButtonWidgetBase::pset_CaptionDropX, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ButtonWidgetBase>("captiondropy", this, &ButtonWidgetBase::pget_CaptionDropY, &ButtonWidgetBase::pset_CaptionDropY, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ButtonWidgetBase>("enabled", this, &ButtonWidgetBase::pget_Enabled, &ButtonWidgetBase::pset_Enabled, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ButtonWidgetBase>("action", this, &ButtonWidgetBase::pget_Action, &ButtonWidgetBase::pset_Action, false);
  AddProperty(mp);

  MemberMethodHandler<ButtonWidgetBase> *mh = new MemberMethodHandler<ButtonWidgetBase>("click", this, 0, &ButtonWidgetBase::m_Click);
  AddMethod(mh);

  AddEvent("OnClick");
  AddEvent("OnGotFocus");
  AddEvent("OnLostFocus");
  defaultevent = 0;
}

ButtonWidgetBase::~ButtonWidgetBase(void) {
}

bool ButtonWidgetBase::StandardAction(int actiontype) {
  switch(actiontype) {
    case STDACTION_ACCEPT:
      this->SimulateClick();
      return true;
      break;
    case STDACTION_ITEMPREV:
      FindParentPage()->FocusPreviousWidget();
      return true;
      break;
    case STDACTION_ITEMNEXT:
      FindParentPage()->FocusNextWidget();
      return true;
      break;
    default:
      return false;
  }
}

void ButtonWidgetBase::SetCaption(std::string caption) {
  this->caption = caption;
}

/* Property get/set functions */

// -- caption --

Variant ButtonWidgetBase::pget_Caption(void) {
  return anytovariant(caption);
}

int ButtonWidgetBase::pset_Caption(Variant value) {
  SetCaption(value);
  return EBERR_SUCCESS;
}

// -- font --

Variant ButtonWidgetBase::pget_Font(void) {
  return anytovariant(font->GetFilename());
}

int ButtonWidgetBase::pset_Font(Variant value) {
  FontResource *font = resourcemanager->LoadFontResource(value, fontsize);
  if(font) {
    SetFont(font);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// -- fontsize --

Variant ButtonWidgetBase::pget_FontSize(void) {
  return anytovariant(fontsize);
}

int ButtonWidgetBase::pset_FontSize(Variant value) {
  fontsize = value;
  if(font)
    return pset_Font(anytovariant(font->GetFilename()));
  else
    return EBERR_SUCCESS;
}

// -- fontcolor --

Variant ButtonWidgetBase::pget_FontColor(void) {
  return anytovariant(colortostr(fontcolor));
}

int ButtonWidgetBase::pset_FontColor(Variant value) {
  eboxy_color c = strtocolor(value);
  SetFontColor(c);
  return EBERR_SUCCESS;
}

// -- fontfocuscolor --

Variant ButtonWidgetBase::pget_FontFocusColor(void) {
  if(fontfocuscolorused)
    return anytovariant(colortostr(fontfocuscolor));
  else
    return VARNULL;
}

int ButtonWidgetBase::pset_FontFocusColor(Variant value) {
  if(value != VARNULL && value != "") {
    eboxy_color c = strtocolor(value);
    fontfocuscolorused = true;
    SetFontFocusColor(c);
  }
  else {
    fontfocuscolorused = false;
    SetFontFocusColor(DEFAULT_FONT_COLOR);
  }
  return EBERR_SUCCESS;
}

// -- fontpressedcolor --

Variant ButtonWidgetBase::pget_FontPressedColor(void) {
  if(fontpressedcolorused)
    return anytovariant(colortostr(fontpressedcolor));
  else
    return VARNULL;
}

int ButtonWidgetBase::pset_FontPressedColor(Variant value) {
  if(value != VARNULL && value != "") {
    eboxy_color c = strtocolor(value);
    fontpressedcolorused = true;
    SetFontPressedColor(c);
  }
  else {
    fontpressedcolorused = false;
    SetFontPressedColor(DEFAULT_FONT_COLOR);
  }
  return EBERR_SUCCESS;
}

// -- fontdisabledcolor --

Variant ButtonWidgetBase::pget_FontDisabledColor(void) {
  if(fontdisabledcolorused)
    return anytovariant(colortostr(fontdisabledcolor));
  else
    return VARNULL;
}

int ButtonWidgetBase::pset_FontDisabledColor(Variant value) {
  if(value != VARNULL && value != "") {
    eboxy_color c = strtocolor(value);
    fontdisabledcolorused = true;
    SetFontDisabledColor(c);
  }
  else {
    fontdisabledcolorused = false;
    SetFontDisabledColor(DEFAULT_FONT_COLOR);
  }
  return EBERR_SUCCESS;
}

// -- image --

Variant ButtonWidgetBase::pget_Image(void) {
  if( normalimage )
    return anytovariant(normalimage->GetFilename());
  else
    return VARNULL;
}

int ButtonWidgetBase::pset_Image(Variant value) {
  ImageResource *image = resourcemanager->LoadImageResource(value, false);
  if(image) {
    SetNormalImage(image);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// -- selectedimage --

Variant ButtonWidgetBase::pget_SelectedImage(void) {
  if( selectedimage )
    return anytovariant(selectedimage->GetFilename());
  else
    return VARNULL;
}

int ButtonWidgetBase::pset_SelectedImage(Variant value) {
  ImageResource *image = resourcemanager->LoadImageResource(value, false);
  if(image || value == VARNULL || value == "") {
    SetSelectedImage(image);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// -- pressedimage --

Variant ButtonWidgetBase::pget_PressedImage(void) {
  if( pressedimage )
    return anytovariant(pressedimage->GetFilename());
  else
    return VARNULL;
}

int ButtonWidgetBase::pset_PressedImage(Variant value) {
  ImageResource *image = resourcemanager->LoadImageResource(value, false);
  if(image || value == VARNULL || value == "") {
    SetPressedImage(image);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// -- disabledimage --

Variant ButtonWidgetBase::pget_DisabledImage(void) {
  if( disabledimage )
    return anytovariant(disabledimage->GetFilename());
  else
    return VARNULL;
}

int ButtonWidgetBase::pset_DisabledImage(Variant value) {
  ImageResource *image = resourcemanager->LoadImageResource(value, false);
  if(image || value == VARNULL || value == "") {
    SetDisabledImage(image);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// -- captionx --

Variant ButtonWidgetBase::pget_CaptionX(void) {
  return anytovariant(captionx);
}

int ButtonWidgetBase::pset_CaptionX(Variant value) {
  SetCaptionX(value);
  return EBERR_SUCCESS;
}

// -- captiony --

Variant ButtonWidgetBase::pget_CaptionY(void) {
  return anytovariant(captiony);
}

int ButtonWidgetBase::pset_CaptionY(Variant value) {
  SetCaptionY(value);
  return EBERR_SUCCESS;
}

// -- captiondropx --

Variant ButtonWidgetBase::pget_CaptionDropX(void) {
  return anytovariant(captiondropx);
}

int ButtonWidgetBase::pset_CaptionDropX(Variant value) {
  SetCaptionDropX(value);
  return EBERR_SUCCESS;
}

// -- captiondropy --

Variant ButtonWidgetBase::pget_CaptionDropY(void) {
  return anytovariant(captiondropy);
}

int ButtonWidgetBase::pset_CaptionDropY(Variant value) {
  SetCaptionDropY(value);
  return EBERR_SUCCESS;
}

// -- enabled --

Variant ButtonWidgetBase::pget_Enabled(void) {
  return anytovariant(enabled);
}

int ButtonWidgetBase::pset_Enabled(Variant value) {
  SetEnabled(value);
  return EBERR_SUCCESS;
}

// -- action --

Variant ButtonWidgetBase::pget_Action(void) {
  return anytovariant(action);
}

int ButtonWidgetBase::pset_Action(Variant value) {
  SetAction(value);
  return EBERR_SUCCESS;
}

// Method functions

Variant ButtonWidgetBase::m_Click(int numargs, Variant args[]) {
  if(enabled)
    SimulateClick();
  return VARNULL;
}
