/***************************************************************************
               buttonwidgetbase.h  -  base class for button widgets
                             -------------------
    begin                : Sat Sep 20 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INCLUDED__BUTTONWIDGETBASE_H
#define INCLUDED__BUTTONWIDGETBASE_H

#include "utils.h"
#include "widgetbase.h"
#include "actionobject.h"
#include "resourcemanagerbase.h"
#include "variant.h"

class ButtonWidgetBase: public virtual WidgetBase, public ActionObject {
  public:
    ButtonWidgetBase(const char *name, int x, int y, int w, int h);
    virtual ~ButtonWidgetBase(void);
    virtual void SetNormalImage(ImageResource *image) = 0;
    virtual void SetSelectedImage(ImageResource *image) = 0;
    virtual void SetPressedImage(ImageResource *image) = 0;
    virtual void SetDisabledImage(ImageResource *image) = 0;
    virtual void SetFont(FontResource *font) = 0;
    virtual void SetFontColor(const eboxy_color &c) = 0;
    virtual void SetFontFocusColor(const eboxy_color &c) = 0;
    virtual void SetFontDisabledColor(const eboxy_color &c) = 0;
    virtual void SetFontPressedColor(const eboxy_color &c) = 0;
    virtual void SetCaption(std::string caption);
    virtual void SetCaptionX(int value) = 0;
    virtual void SetCaptionY(int value) = 0;
    virtual void SetCaptionDropX(int value) = 0;
    virtual void SetCaptionDropY(int value) = 0;
    virtual void SetAction(string actionname) = 0;
    virtual void SetEnabled(bool enabled) = 0;
    virtual void SetWidth(int value) = 0;
    virtual void SetHeight(int value) = 0;
    virtual void SimulateClick(void) = 0;
    virtual bool StandardAction(int actiontype);

  protected:
    std::string caption;
    std::string action;
    FontResource *font;
    int fontsize;
    eboxy_color fontcolor;
    eboxy_color fontfocuscolor;
    bool fontfocuscolorused;
    eboxy_color fontdisabledcolor;
    bool fontdisabledcolorused;
    eboxy_color fontpressedcolor;
    bool fontpressedcolorused;
    unsigned int activecolors;
    ImageResource *normalimage;
    ImageResource *selectedimage;
    ImageResource *pressedimage;
    ImageResource *disabledimage;
    int captionx;
    int captiony;
    int captiondropx;
    int captiondropy;
    bool enabled;

    Variant pget_Caption(void);
    int pset_Caption(Variant value);
    Variant pget_Font(void);
    int pset_Font(Variant value);
    Variant pget_FontSize(void);
    int pset_FontSize(Variant value);
    Variant pget_FontColor(void);
    int pset_FontColor(Variant value);
    Variant pget_FontFocusColor(void);
    int pset_FontFocusColor(Variant value);
    Variant pget_FontPressedColor(void);
    int pset_FontPressedColor(Variant value);
    Variant pget_FontDisabledColor(void);
    int pset_FontDisabledColor(Variant value);
    Variant pget_Image(void);
    int pset_Image(Variant value);
    Variant pget_SelectedImage(void);
    int pset_SelectedImage(Variant value);
    Variant pget_PressedImage(void);
    int pset_PressedImage(Variant value);
    Variant pget_DisabledImage(void);
    int pset_DisabledImage(Variant value);
    Variant pget_CaptionX(void);
    int pset_CaptionX(Variant value);
    Variant pget_CaptionY(void);
    int pset_CaptionY(Variant value);
    Variant pget_CaptionDropX(void);
    int pset_CaptionDropX(Variant value);
    Variant pget_CaptionDropY(void);
    int pset_CaptionDropY(Variant value);
    Variant pget_Enabled(void);
    int pset_Enabled(Variant value);
    Variant pget_Action(void);
    int pset_Action(Variant value);
    Variant m_Click(int numargs, Variant args[]);
};


#endif
