/***************************************************************************
                          eventhandler.cpp  -  eboxy event handler
                             -------------------
    begin                : Sun Sep 8 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <cstring>
#include <cstdlib>

#include <pthread.h>

#include "eventhandler.h"

using namespace std;

// Prototype for thread function
void *fire_event_thread(void *arg);

EventHandler::EventHandler(const char *name, Object *source, Object *sink) {
  this->name = strdup(name);
  this->source = source;
  this->sink = sink;
  this->threaded = true;
}

EventHandler::~EventHandler(void) {
  free(name);
}

const char *EventHandler::GetName(void) {
  return name;
}

int EventHandler::Fire(bool allowthread) {
  if(threaded && allowthread) {
    pthread_attr_t thread_attr;
    int res;

    // Need to start the thread up in the detached state, or we run out of thread IDs fast
    // I consider this to be a BUG in LinuxThreads, let's hope it gets fixed in 2.6/3.0
    res = pthread_attr_init(&thread_attr);
    if(res) {
      cerr << "Failed to create thread attribute (" << res << ")" << endl;
      return -1;
    }
    res = pthread_attr_setdetachstate(&thread_attr, PTHREAD_CREATE_DETACHED);
    if(res) {
      cerr << "Failed to set thread detached state (" << res << ")" << endl;
      return -1;
    }
    // Now create the thread
    res = pthread_create(&event_thread, &thread_attr, fire_event_thread, (void *)this);
    if(res) {
      cerr << "Failed to create event thread (" << res << ")" << endl;
      return -1;
    }
    // Done with the attribute, release it
    pthread_attr_destroy(&thread_attr);
    return 0;
  }
  else {
    // Unthreaded
    return InternalFire();
  }
}

Object *EventHandler::GetSource(void) {
  return source;
}

Object *EventHandler::GetSink(void) {
  return sink;
}

int EventHandler::InternalFire(void) {
  // implemented in inherited objects
  return 0;
}

void EventHandler::SetThreaded(bool threaded) {
  this->threaded = threaded;
}

bool EventHandler::IsThreaded(void) {
  return this->threaded;
}

// Thread function
void *fire_event_thread(void *arg) {
  void *result = (void *)((EventHandler *)arg)->Fire(false);
  pthread_exit(result);
}

