/***************************************************************************
           externalmethodhandler.cpp  -  method handler for plugins
                             -------------------
    begin                : Thu May 22 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>

#include "externalmethodhandler.h"

using namespace std;

ExternalMethodHandler::ExternalMethodHandler(const char *name, Object *owner, int numargs, methodfn methodfuncex)
: MethodHandler(name, owner, numargs) {
  this->methodfuncex = methodfuncex;
}

Variant ExternalMethodHandler::Call(int numargs, Variant args[]) {
  if(numargs == this->numargs) {
    char *cargs[numargs];
    for(int j=0;j<numargs;j++) {
      string str = args[j];
      cargs[j] = strdup(str.c_str());
    }
    char *retstr = methodfuncex(owner->GetName(), numargs, (const char **)cargs);
    for(int j=0;j<numargs;j++) {
      free(cargs[j]);
    }
    Variant retvar;
    if(retstr) {
      retvar = anytovariant(retstr);
      free(retstr);
    }
    else
      retvar = VARNULL;
    return retvar;
  }
  else {
    cerr << "Wrong number of arguments passed to method " << name << " of object " << owner->GetName();
    cerr << " (" << this->numargs << " expected, " << numargs <<  " supplied)" << endl;
  }
  return VARNULL;
}
