/***************************************************************************
               labelwidgetbase.cpp  -  base class for label widgets
                             -------------------
    begin                : Sat Nov 15 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "utils.h"

#include "resourcemanagerbase.h"
#include "memberfunctionproperty.h"
#include "labelwidgetbase.h"
#include "../errors.h"

using namespace std;

extern const int DEFAULT_FONTSIZE;
extern const eboxy_color DEFAULT_FONT_COLOR;
extern const char *TYPESTR_LABEL;

extern ResourceManagerBase *resourcemanager;

LabelWidgetBase::LabelWidgetBase(const char *name, int x, int y, int w, int h, std::string text, FontResource *font)
: WidgetBase(x, y) {
  this->w = w;
  this->h = h;
  this->text = text;
  this->font = font;
  this->fontsize = DEFAULT_FONTSIZE;
  this->fontcolor = DEFAULT_FONT_COLOR;
  this->wordwrap = false;
  this->autosize = false;
  this->textalign = TEXTALIGN_LEFT;
  this->type = TYPESTR_LABEL;

  MemberFunctionProperty<LabelWidgetBase> *mp;
  mp = new MemberFunctionProperty<LabelWidgetBase>("caption", this, &LabelWidgetBase::pget_Caption, &LabelWidgetBase::pset_Caption, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<LabelWidgetBase>("font", this, &LabelWidgetBase::pget_Font, &LabelWidgetBase::pset_Font, true);
  AddProperty(mp);
  mp = new MemberFunctionProperty<LabelWidgetBase>("fontsize", this, &LabelWidgetBase::pget_FontSize, &LabelWidgetBase::pset_FontSize, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<LabelWidgetBase>("fontcolor", this, &LabelWidgetBase::pget_FontColor, &LabelWidgetBase::pset_FontColor, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<LabelWidgetBase>("textalign", this, &LabelWidgetBase::pget_TextAlign, &LabelWidgetBase::pset_TextAlign, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<LabelWidgetBase>("wordwrap", this, &LabelWidgetBase::pget_WordWrap, &LabelWidgetBase::pset_WordWrap, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<LabelWidgetBase>("autosize", this, &LabelWidgetBase::pget_AutoSize, &LabelWidgetBase::pset_AutoSize, false);
  AddProperty(mp);

  defaultproperty = FindProperty("caption");
}

LabelWidgetBase::~LabelWidgetBase(void) {

}

void LabelWidgetBase::SetText(std::string text) {
  this->text = text;
}

std::string LabelWidgetBase::GetText(void) {
  return text;
}

void LabelWidgetBase::SetFont(FontResource *font) {
  this->font = font;
}

void LabelWidgetBase::SetFontColor(eboxy_color c) {
  this->fontcolor = c;
}

void LabelWidgetBase::SetWordWrap(bool wordwrap) {
  this->wordwrap = wordwrap;
}

void LabelWidgetBase::SetAutoSize(bool autosize) {
  this->autosize = autosize;
}

void LabelWidgetBase::SetAlign(enum_textalign textalign) {
  this->textalign = textalign;
}


/* Property get/set functions */

// -- caption --

Variant LabelWidgetBase::pget_Caption(void) {
  return anytovariant(text);
}

int LabelWidgetBase::pset_Caption(Variant value) {
  SetText(value);
  return EBERR_SUCCESS;
}

// -- font --

Variant LabelWidgetBase::pget_Font(void) {
  return anytovariant(font->GetFilename());
}

int LabelWidgetBase::pset_Font(Variant value) {
  FontResource *font = resourcemanager->LoadFontResource(value, fontsize);
  if(font) {
    SetFont(font);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// -- fontsize --

Variant LabelWidgetBase::pget_FontSize(void) {
  return anytovariant(fontsize);
}

int LabelWidgetBase::pset_FontSize(Variant value) {
  fontsize = value;
  if(font)
    return pset_Font(anytovariant(font->GetFilename()));
  else
    return EBERR_SUCCESS;
}

// -- fontcolor --

Variant LabelWidgetBase::pget_FontColor(void) {
  return anytovariant(colortostr(fontcolor));
}

int LabelWidgetBase::pset_FontColor(Variant value) {
  SetFontColor(strtocolor(value));
  return EBERR_SUCCESS;
}

// -- textalign --

Variant LabelWidgetBase::pget_TextAlign(void) {
  return anytovariant(textaligninttostr(textalign));
}

int LabelWidgetBase::pset_TextAlign(Variant value) {
  SetAlign(textalignstrtoint(value));
  return EBERR_SUCCESS;
}

// -- autosize --

Variant LabelWidgetBase::pget_AutoSize(void) {
  return anytovariant(autosize);
}

int LabelWidgetBase::pset_AutoSize(Variant value) {
  SetAutoSize(value);
  return EBERR_SUCCESS;
}

// -- wordwrap --

Variant LabelWidgetBase::pget_WordWrap(void) {
  return anytovariant(wordwrap);
}

int LabelWidgetBase::pset_WordWrap(Variant value) {
  SetWordWrap(value);
  return EBERR_SUCCESS;
}


enum_textalign textalignstrtoint(const string &str) {
  if(lowercase(str) == "center" || lowercase(str) == "centre" || lowercase(str) == "middle" ==0)
    return TEXTALIGN_CENTER;
  else if(lowercase(str) == "right")
    return TEXTALIGN_RIGHT;
  else // left
    return TEXTALIGN_LEFT;
}

string textaligninttostr(int textalign) {
  switch(textalign) {
    case TEXTALIGN_RIGHT:
      return "right";
    case TEXTALIGN_CENTER:
      return "center";
    default:
      return "left";
  }
}

