/***************************************************************************
             listboxwidgetbase.cpp  -  listbox widget base class
                             -------------------
    begin                : Mon Nov 17 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "utils.h"
#include "resourcemanagerbase.h"
#include "memberfunctionproperty.h"
#include "membermethodhandler.h"
#include "../errors.h"
 
#include "listboxwidgetbase.h"
 
using namespace std;

extern const char *TYPESTR_LISTBOX;
extern const int DEFAULT_FONTSIZE;
extern const eboxy_color DEFAULT_SELECTED_FONT_COLOR;
extern ResourceManagerBase *resourcemanager;

ListBoxWidgetBase::ListBoxWidgetBase(const char *name, int x, int y, int w, int h, FontResource *font, eboxy_color fontcolor)
: WidgetBase(x, y) {
  this->w = w;
  this->h = h;
  this->font = font;
  this->fontsize = DEFAULT_FONTSIZE;
  this->fontcolor = fontcolor;

  // Set defaults
  tabstop = true;
  backgroundimage = NULL;
  backgroundfocusimage = NULL;
  backgroundstyle = IMAGESTYLE_NORMAL;
  selitembackgroundimage = NULL;
  selitembackgroundfocusimage = NULL;
  selitembackgroundstyle = IMAGESTYLE_NORMAL;
  scrollbarbackgroundimage = NULL;
  scrollbarbackgroundstyle = IMAGESTYLE_NORMAL;
  scrollknobimage = NULL;
  selitemfontcolor = DEFAULT_SELECTED_FONT_COLOR;
  scrollbarwidth = 16;
  borderx = 0;
  bordery = 0;
  itemborderx = 0;
  itembordery = 0;
  itemheight = -1; // automatic (text height)
  selectedindex = 0;
  scrollbarmode = AUTOPROPERTY_AUTO;
  this->type = TYPESTR_LISTBOX;

  MemberFunctionProperty<ListBoxWidgetBase> *mp;
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("font", this, &ListBoxWidgetBase::pget_Font, &ListBoxWidgetBase::pset_Font, true);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("fontsize", this, &ListBoxWidgetBase::pget_FontSize, &ListBoxWidgetBase::pset_FontSize, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("fontcolor", this, &ListBoxWidgetBase::pget_FontColor, &ListBoxWidgetBase::pset_FontColor, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("background", this, &ListBoxWidgetBase::pget_Background, &ListBoxWidgetBase::pset_Background, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("backgroundfocus", this, &ListBoxWidgetBase::pget_BackgroundFocus, &ListBoxWidgetBase::pset_BackgroundFocus, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("backgroundstyle", this, &ListBoxWidgetBase::pget_BackgroundStyle, &ListBoxWidgetBase::pset_BackgroundStyle, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("selectedindex", this, &ListBoxWidgetBase::pget_SelectedIndex, &ListBoxWidgetBase::pset_SelectedIndex, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("selectedtext", this, &ListBoxWidgetBase::pget_SelectedText, NULL, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("count", this, &ListBoxWidgetBase::pget_Count, NULL, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("selitembackground", this, &ListBoxWidgetBase::pget_SelItemBackground, &ListBoxWidgetBase::pset_SelItemBackground, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("selitembackgroundfocus", this, &ListBoxWidgetBase::pget_SelItemBackgroundFocus, &ListBoxWidgetBase::pset_SelItemBackgroundFocus, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("selitembackgroundstyle", this, &ListBoxWidgetBase::pget_SelItemBackgroundStyle, &ListBoxWidgetBase::pset_SelItemBackgroundStyle, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("selitemfontcolor", this, &ListBoxWidgetBase::pget_SelItemFontColor, &ListBoxWidgetBase::pset_SelItemFontColor, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("scrollbarbackground", this, &ListBoxWidgetBase::pget_ScrollBarBackground, &ListBoxWidgetBase::pset_ScrollBarBackground, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("scrollbarbackgroundstyle", this, &ListBoxWidgetBase::pget_ScrollBarBackgroundStyle, &ListBoxWidgetBase::pset_ScrollBarBackgroundStyle, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("scrollbarknob", this, &ListBoxWidgetBase::pget_ScrollBarKnob, &ListBoxWidgetBase::pset_ScrollBarKnob, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("scrollbarwidth", this, &ListBoxWidgetBase::pget_ScrollBarWidth, &ListBoxWidgetBase::pset_ScrollBarWidth, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("scrollbar", this, &ListBoxWidgetBase::pget_ScrollBar, &ListBoxWidgetBase::pset_ScrollBar, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("borderx", this, &ListBoxWidgetBase::pget_BorderX, &ListBoxWidgetBase::pset_BorderX, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("bordery", this, &ListBoxWidgetBase::pget_BorderY, &ListBoxWidgetBase::pset_BorderY, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("itemborderx", this, &ListBoxWidgetBase::pget_ItemBorderX, &ListBoxWidgetBase::pset_ItemBorderX, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("itembordery", this, &ListBoxWidgetBase::pget_ItemBorderY, &ListBoxWidgetBase::pset_ItemBorderY, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ListBoxWidgetBase>("itemheight", this, &ListBoxWidgetBase::pget_ItemHeight, &ListBoxWidgetBase::pset_ItemHeight, false);
  AddProperty(mp);

  MemberMethodHandler<ListBoxWidgetBase> *mh;
  mh = new MemberMethodHandler<ListBoxWidgetBase>("additem", this, 1, &ListBoxWidgetBase::m_AddItem);
  AddMethod(mh);
  mh = new MemberMethodHandler<ListBoxWidgetBase>("removeitem", this, 1, &ListBoxWidgetBase::m_RemoveItem);
  AddMethod(mh);
  mh = new MemberMethodHandler<ListBoxWidgetBase>("removelastitem", this, 0, &ListBoxWidgetBase::m_RemoveLastItem);
  AddMethod(mh);
  mh = new MemberMethodHandler<ListBoxWidgetBase>("clear", this, 0, &ListBoxWidgetBase::m_Clear);
  AddMethod(mh);
  mh = new MemberMethodHandler<ListBoxWidgetBase>("setitemtext", this, 2, &ListBoxWidgetBase::m_SetItemText);
  AddMethod(mh);

  AddEvent("OnSelectionChange");
  AddEvent("OnEmpty");
  AddEvent("OnChoose");
  AddEvent("OnGotFocus");
  AddEvent("OnLostFocus");
}

ListBoxWidgetBase::~ListBoxWidgetBase(void) {
  while(listitems.size() > 0)
    listitems.pop_back();
}

bool ListBoxWidgetBase::StandardAction(int actiontype) {
  switch(actiontype) {
    case STDACTION_ACCEPT:
      ChooseSelection();
      return true;
      break;
    case STDACTION_ITEMPREV:
      SelectPrevious(1);
      return true;
      break;
    case STDACTION_ITEMNEXT:
      SelectNext(1);
      return true;
      break;
    case STDACTION_ITEMPAGEPREV:
      SelectPreviousPage();
      return true;
      break;
    case STDACTION_ITEMPAGENEXT:
      SelectNextPage();
      return true;
      break;
    case STDACTION_ITEMFIRST:
      SetSelectedIndex(0);
      return true;
      break;
    case STDACTION_ITEMLAST:
      SetSelectedIndex(GetItemCount()-1);
      return true;
      break;
    default:
      return false;
  }
}



/* Property get/set functions */

// -- font --

Variant ListBoxWidgetBase::pget_Font(void) {
  return anytovariant(font->GetFilename());
}

int ListBoxWidgetBase::pset_Font(Variant value) {
  FontResource *font = resourcemanager->LoadFontResource(value, fontsize);
  if(font) {
    SetFont(font);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// -- fontsize --

Variant ListBoxWidgetBase::pget_FontSize(void) {
  return anytovariant(fontsize);
}

int ListBoxWidgetBase::pset_FontSize(Variant value) {
  fontsize = value;
  if(font)
    return pset_Font(anytovariant(font->GetFilename()));
  else
    return EBERR_SUCCESS;
}

// -- fontcolor --

Variant ListBoxWidgetBase::pget_FontColor(void) {
  return anytovariant(colortostr(fontcolor));
}

int ListBoxWidgetBase::pset_FontColor(Variant value) {
  SetFontColor(strtocolor(value));
  return EBERR_SUCCESS;
}

// -- background --

Variant ListBoxWidgetBase::pget_Background(void) {
  if( backgroundimage )
    return anytovariant(backgroundimage->GetFilename());
  else
    return VARNULL;
}

int ListBoxWidgetBase::pset_Background(Variant value) {
  ImageResource *image = resourcemanager->LoadImageResource(value, false);
  if(image) {
    SetBackgroundImage(image);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// -- backgroundfocus --

Variant ListBoxWidgetBase::pget_BackgroundFocus(void) {
  if( backgroundfocusimage )
    return anytovariant(backgroundfocusimage->GetFilename());
  else
    return VARNULL;
}

int ListBoxWidgetBase::pset_BackgroundFocus(Variant value) {
  ImageResource *image = resourcemanager->LoadImageResource(value, false);
  if(image) {
    SetBackgroundFocusImage(image);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// -- backgroundstyle --

Variant ListBoxWidgetBase::pget_BackgroundStyle(void) {
  return anytovariant(styleinttostr(backgroundstyle));
}

int ListBoxWidgetBase::pset_BackgroundStyle(Variant value) {
  SetBackgroundStyle(stylestrtoint(value));
  return EBERR_SUCCESS;
}

// -- selectedindex --

Variant ListBoxWidgetBase::pget_SelectedIndex(void) {
  return anytovariant(GetSelectedIndex());
}

int ListBoxWidgetBase::pset_SelectedIndex(Variant value) {
  int idx = value;
  if(idx > listitems.size() - 1 || idx < 0) {
    return EBERR_BAD_PROPERTY_VALUE;
  }
  else {
    SetSelectedIndex(value);
    return EBERR_SUCCESS;
  }
}

// -- selectedtext --

Variant ListBoxWidgetBase::pget_SelectedText(void) {
  if(listitems.size() > 0)
    return anytovariant(listitems[selectedindex]);
  else
    return VARNULL;
}

// -- count --

Variant ListBoxWidgetBase::pget_Count(void) {
  return anytovariant((int)(listitems.size()));
}

// -- selitembackground --

Variant ListBoxWidgetBase::pget_SelItemBackground(void) {
  if( selitembackgroundimage )
    return anytovariant(selitembackgroundimage->GetFilename());
  else
    return VARNULL;
}

int ListBoxWidgetBase::pset_SelItemBackground(Variant value) {
  ImageResource *image = resourcemanager->LoadImageResource(value, false);
  if(image) {
    SetSelectedItemImage(image);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// -- selitembackgroundfocus --

Variant ListBoxWidgetBase::pget_SelItemBackgroundFocus(void) {
  if( selitembackgroundfocusimage )
    return anytovariant(selitembackgroundfocusimage->GetFilename());
  else
    return VARNULL;
}

int ListBoxWidgetBase::pset_SelItemBackgroundFocus(Variant value) {
  ImageResource *image = resourcemanager->LoadImageResource(value, false);
  if(image) {
    SetSelectedItemFocusImage(image);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// -- selitembackgroundstyle --

Variant ListBoxWidgetBase::pget_SelItemBackgroundStyle(void) {
  return anytovariant(styleinttostr(selitembackgroundstyle));
}

int ListBoxWidgetBase::pset_SelItemBackgroundStyle(Variant value) {
  SetSelectedItemStyle(stylestrtoint(value));
  return EBERR_SUCCESS;
}

// -- selitemfontcolor --

Variant ListBoxWidgetBase::pget_SelItemFontColor(void) {
  return anytovariant(colortostr(selitemfontcolor));
}

int ListBoxWidgetBase::pset_SelItemFontColor(Variant value) {
  SetSelectedItemFontColor(strtocolor(value));
  return EBERR_SUCCESS;
}

// -- scrollbarbackground --

Variant ListBoxWidgetBase::pget_ScrollBarBackground(void) {
  if( scrollbarbackgroundimage )
    return anytovariant(scrollbarbackgroundimage->GetFilename());
  else
    return VARNULL;
}

int ListBoxWidgetBase::pset_ScrollBarBackground(Variant value) {
  ImageResource *image = resourcemanager->LoadImageResource(value, false);
  if(image) {
    SetScrollBarBackgroundImage(image);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// -- scrollbarbackgroundstyle --

Variant ListBoxWidgetBase::pget_ScrollBarBackgroundStyle(void) {
  return anytovariant(styleinttostr(scrollbarbackgroundstyle));
}

int ListBoxWidgetBase::pset_ScrollBarBackgroundStyle(Variant value) {
  SetScrollBarBackgroundStyle(stylestrtoint(value));
  return EBERR_SUCCESS;
}

// -- scrollbarknob --

Variant ListBoxWidgetBase::pget_ScrollBarKnob(void) {
  if( scrollknobimage )
    return anytovariant(scrollknobimage->GetFilename());
  else
    return VARNULL;
}

int ListBoxWidgetBase::pset_ScrollBarKnob(Variant value) {
  ImageResource *image = resourcemanager->LoadImageResource(value, false);
  if(image) {
    SetScrollBarKnobImage(image);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// -- scrollbarwidth --

Variant ListBoxWidgetBase::pget_ScrollBarWidth(void) {
  return anytovariant(scrollbarwidth);
}

int ListBoxWidgetBase::pset_ScrollBarWidth(Variant value) {
  SetScrollBarWidth(value);
  return EBERR_SUCCESS;
}

// -- scrollbar --

Variant ListBoxWidgetBase::pget_ScrollBar(void) {
  return anytovariant(autointtostr(scrollbarmode));
}

int ListBoxWidgetBase::pset_ScrollBar(Variant value) {
  SetScrollBarMode(autostrtoint(value));
  return EBERR_SUCCESS;
}

// -- borderx --

Variant ListBoxWidgetBase::pget_BorderX(void) {
  return anytovariant(borderx);
}

int ListBoxWidgetBase::pset_BorderX(Variant value) {
  SetBorderX(value);
  return EBERR_SUCCESS;
}

// -- bordery --

Variant ListBoxWidgetBase::pget_BorderY(void) {
  return anytovariant(bordery);
}

int ListBoxWidgetBase::pset_BorderY(Variant value) {
  SetBorderY(value);
  return EBERR_SUCCESS;
}

// -- itemborderx --

Variant ListBoxWidgetBase::pget_ItemBorderX(void) {
  return anytovariant(itemborderx);
}

int ListBoxWidgetBase::pset_ItemBorderX(Variant value) {
  SetItemBorderX(value);
  return EBERR_SUCCESS;
}

// -- itembordery --

Variant ListBoxWidgetBase::pget_ItemBorderY(void) {
  return anytovariant(itembordery);
}

int ListBoxWidgetBase::pset_ItemBorderY(Variant value) {
  SetItemBorderY(value);
  return EBERR_SUCCESS;
}

// -- itemheight --

Variant ListBoxWidgetBase::pget_ItemHeight(void) {
  return anytovariant(itemheight);
}

int ListBoxWidgetBase::pset_ItemHeight(Variant value) {
  SetItemHeight(value);
  return EBERR_SUCCESS;
}

/* Method functions */

Variant ListBoxWidgetBase::m_AddItem(int numargs, Variant args[]) {
  AddItem(args[0]);
  return VARNULL;
}

Variant ListBoxWidgetBase::m_RemoveItem(int numargs, Variant args[]) {
  RemoveItem(args[0]);
  return VARNULL;
}

Variant ListBoxWidgetBase::m_RemoveLastItem(int numargs, Variant args[]) {
  RemoveItem(listitems.size()-1);
  return VARNULL;
}

Variant ListBoxWidgetBase::m_Clear(int numargs, Variant args[]) {
  Clear();
  return VARNULL;
}

Variant ListBoxWidgetBase::m_SetItemText(int numargs, Variant args[]) {
  SetItemText(args[0], args[1]);
  return VARNULL;
}
