/***************************************************************************
             listboxwidgetbase.h  -  listbox widget base class
                             -------------------
    begin                : Mon Nov 17 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INCLUDED__LISTBOXWIDGETBASE_H
#define INCLUDED__LISTBOXWIDGETBASE_H

#include "widgetbase.h"
#include "resource.h"

class ListBoxWidgetBase: virtual public WidgetBase {
  public:
    ListBoxWidgetBase(const char *name, int x, int y, int w, int h, FontResource *font, eboxy_color fontcolor);
    ~ListBoxWidgetBase(void);
    virtual void SetFont(FontResource *font) = 0;
    virtual void SetFontColor(eboxy_color c) = 0;
    virtual void SetSelectedItemFontColor(eboxy_color c) = 0;
    virtual void SetBackgroundImage(ImageResource *image) = 0;
    virtual void SetBackgroundFocusImage(ImageResource *image) = 0;
    virtual void SetBackgroundStyle(int style) = 0;
    virtual void SetSelectedItemImage(ImageResource *image) = 0;
    virtual void SetSelectedItemFocusImage(ImageResource *image) = 0;
    virtual void SetSelectedItemStyle(int style) = 0;
    virtual void SetScrollBarMode(int scrollbarmode) = 0;
    virtual void SetScrollBarBackgroundImage(ImageResource *image) = 0;
    virtual void SetScrollBarBackgroundStyle(int style) = 0;
    virtual void SetScrollBarKnobImage(ImageResource *image) = 0;
    virtual void SetScrollBarWidth(int width) = 0;
    virtual void SetBorderX(int pixels) = 0;
    virtual void SetBorderY(int pixels) = 0;
    virtual void SetItemBorderX(int pixels) = 0;
    virtual void SetItemBorderY(int pixels) = 0;
    virtual void SetItemHeight(int pixels) = 0;
    virtual void SetSelectedIndex(int index) = 0;
    virtual int GetSelectedIndex(void) = 0;
    virtual int GetItemCount(void) = 0;
    virtual void SelectNext(int count) = 0;
    virtual void SelectNextPage(void) = 0;
    virtual void SelectPrevious(int count) = 0;
    virtual void SelectPreviousPage(void) = 0;
    virtual void AddItem(std::string text) = 0;
    virtual void RemoveItem(int index) = 0;
    virtual void Clear(void) = 0;
    virtual void SetItemText(int index, std::string text) = 0;
    virtual void ChooseSelection(void) = 0;
    virtual bool StandardAction(int actiontype);
  protected:
    std::vector<std::string> listitems;
    FontResource *font;
    int fontsize;
    eboxy_color fontcolor;
    eboxy_color selitemfontcolor;
    ImageResource *backgroundimage;
    ImageResource *backgroundfocusimage;
    int backgroundstyle;
    ImageResource *selitembackgroundimage;
    ImageResource *selitembackgroundfocusimage;
    int selitembackgroundstyle;
    int scrollbarmode;
    ImageResource *scrollbarbackgroundimage;
    int scrollbarbackgroundstyle;
    ImageResource *scrollknobimage;
    int scrollbarwidth;
    int borderx;
    int bordery;
    int itemborderx;
    int itembordery;
    int itemheight;
    int selectedindex;

    Variant pget_Font(void);
    int pset_Font(Variant value);
    Variant pget_FontSize(void);
    int pset_FontSize(Variant value);
    Variant pget_FontColor(void);
    int pset_FontColor(Variant value);
    Variant pget_Background(void);
    int pset_Background(Variant value);
    Variant pget_BackgroundFocus(void);
    int pset_BackgroundFocus(Variant value);
    Variant pget_BackgroundStyle(void);
    int pset_BackgroundStyle(Variant value);
    Variant pget_SelectedIndex(void);
    int pset_SelectedIndex(Variant value);
    Variant pget_SelectedText(void);
    Variant pget_Count(void);
    Variant pget_SelItemBackground(void);
    int pset_SelItemBackground(Variant value);
    Variant pget_SelItemBackgroundFocus(void);
    int pset_SelItemBackgroundFocus(Variant value);
    Variant pget_SelItemBackgroundStyle(void);
    int pset_SelItemBackgroundStyle(Variant value);
    Variant pget_SelItemFontColor(void);
    int pset_SelItemFontColor(Variant value);
    Variant pget_ScrollBarBackground(void);
    int pset_ScrollBarBackground(Variant value);
    Variant pget_ScrollBarBackgroundStyle(void);
    int pset_ScrollBarBackgroundStyle(Variant value);
    Variant pget_ScrollBarKnob(void);
    int pset_ScrollBarKnob(Variant value);
    Variant pget_ScrollBarWidth(void);
    int pset_ScrollBarWidth(Variant value);
    Variant pget_ScrollBar(void);
    int pset_ScrollBar(Variant value);
    Variant pget_BorderX(void);
    int pset_BorderX(Variant value);
    Variant pget_BorderY(void);
    int pset_BorderY(Variant value);
    Variant pget_ItemBorderX(void);
    int pset_ItemBorderX(Variant value);
    Variant pget_ItemBorderY(void);
    int pset_ItemBorderY(Variant value);
    Variant pget_ItemHeight(void);
    int pset_ItemHeight(Variant value);

    Variant m_AddItem(int numargs, Variant args[]);
    Variant m_RemoveItem(int numargs, Variant args[]);
    Variant m_RemoveLastItem(int numargs, Variant args[]);
    Variant m_Clear(int numargs, Variant args[]);
    Variant m_SetItemText(int numargs, Variant args[]);
};

#endif
