/***************************************************************************
      memberfunctionproperty.h - property object that uses member
                                 getter/setter functions
                             -------------------
    begin                : Wed Nov 22 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INCLUDED__MEMBERFUNCTIONPROPERTY_H
#define INCLUDED__MEMBERFUNCTIONPROPERTY_H

#include "customproperty.h"
#include "variant.h"

template <class T> class MemberFunctionProperty: public CustomProperty {
  typedef int propsethandler(Variant value);
  typedef Variant propgethandler(void);
  typedef propsethandler (T::*propsetmethod);
  typedef propgethandler (T::*propgetmethod);

  public:
    MemberFunctionProperty<T>(const char *name, T *object, propgetmethod getter, propsetmethod setter, bool required)
    : CustomProperty(name) {
      this->object = object;
      this->getter = getter;
      this->setter = setter;
      this->required = required;
    };
    int SetValue(Variant value) {
      if(setter)
        return (object->*setter)(value);
      else {
        cerr << "Property " << name << " is read-only" << endl;
        return -1;
      }
    };
    Variant GetValue(void) {
      if(getter)
        return (object->*getter)();
      else {
        cerr << "Property " << name << " is write-only" << endl;
        return VARNULL;
      }
    };
    virtual bool IsReadOnly(void) {
      return (setter == NULL);
    };
  private:
    propgetmethod getter;
    propsetmethod setter;
    T *object;
};

#endif
