/***************************************************************************
                 objcontainer.cpp  -  object container abstract class
                             -------------------
    begin                : Fri Dec 27 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <vector>

#include "objcontainer.h"

using namespace std;

ObjectContainer::~ObjectContainer(void) {
  // Tidy up contained objects
  ClearObjects();
}

void ObjectContainer::AddObject(DynamicObject *object) {
  containedobjects.push_back(object);
  object->SetOwner(this);
  object->SetContainer(this);
}

void ObjectContainer::AddUnownedObject(DynamicObject *object) {
  // This object is managed elsewhere - we only want a reference to it
  // So we don't take ownership (and hence we don't try to free it later)
  containedobjects.push_back(object);
  object->SetContainer(this);
}

DynamicObject *ObjectContainer::FindObject(const char *name, const char *type) {
  for(int i=0; i<containedobjects.size(); i++) {
    if(containedobjects[i]->IsFindable() && !strcasecmp(containedobjects[i]->GetName(), name) && (type == NULL || (!strcasecmp(containedobjects[i]->GetType(), type))))
      return containedobjects[i];
  }
  // No object found
  return NULL;
}

void ObjectContainer::ClearObjects(void) {
  while(containedobjects.size() > 0) {
    DynamicObject *object = containedobjects.back();
    containedobjects.pop_back();
    object->SetContainer(NULL);
    if(object->GetOwner() == this)
      delete object;
  }
}

void ObjectContainer::RemoveObject(DynamicObject *object) {
  for(int i=0; i<containedobjects.size(); i++) {
    if(containedobjects[i] == object) {
      containedobjects.erase(containedobjects.begin() + i);
      object->SetContainer(NULL);
      if(object->GetOwner() == this)
        delete object;
      break;
    }
  }
}

void ObjectContainer::RemoveObjectsOwnedBy(void *ownerref) {
  for(int i=0; i<containedobjects.size(); i++) {
    if(containedobjects[i]->GetOwner() == ownerref) {
      containedobjects.erase(containedobjects.begin() + i);
    }
  }
}

void ObjectContainer::Notify(Object *obj) {
  // FIXME why are we using Object if it's an DynamicObject ?!
  DynamicObject *dobj = dynamic_cast<DynamicObject *>(obj);
  RemoveObject(dobj);
}

void ObjectContainer::RemoveOwnedObject(void *object) {
  // FIXME is this really necessary?
  RemoveObject((DynamicObject *)object);
}
