/***************************************************************************
                  pagebase.h  -  base class for eboxy pages
                             -------------------
    begin                : Mon Sep 22 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INCLUDED__PAGEBASE_H
#define INCLUDED__PAGEBASE_H

#include "widgetbase.h"
#include "objcontainer.h"
#include "resource.h"
#include "actionhandler.h"

class PageBase: public virtual WidgetBase, public ObjectContainer {
  public:
    PageBase(const char *name, int x, int y, int w, int h);
    virtual ~PageBase(void);
    virtual void AddWidget(WidgetBase *widget) = 0;
    virtual void RemoveWidget(WidgetBase *widget) = 0;
    virtual void SetBackground(ImageResource *image);
    void FocusNextWidget(void);
    void FocusPreviousWidget(void);
    virtual void UpdateFocus(void) = 0;
    WidgetBase *GetFocusedWidget(void);
    void SetFocus(WidgetBase *widget);
    DynamicObject *FindObject(const char *name, const char *type);
    WidgetBase *FindWidget(const char *name, const char *type);
    void DoOnShow(void);
    void AddActionHandler(ActionHandler *handler);
    void RemoveActionHandler(ActionHandler *handler);
  protected:
    ImageResource *background;
    std::vector<WidgetBase *> widgets;
    int focusedindex;
    std::vector<ActionHandler *> actionhandlers;

    void DoInstantiate(void);
    void DoUninstantiate(void);

    Variant pget_WidgetCount(void);
    Variant pget_Background(void);
    int pset_Background(Variant value);

    Variant m_AddWidget(int numargs, Variant args[]);
    Variant m_RemoveWidget(int numargs, Variant args[]);
    Variant m_GetWidget(int numargs, Variant args[]);
};

#endif
