/***************************************************************************
         picturewidgetbase.cpp  -  base class for static picture widget
                             -------------------
    begin                : Mon Nov 17 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "resourcemanagerbase.h"
#include "memberfunctionproperty.h"
#include "membermethodhandler.h"
#include "../errors.h"

#include "picturewidgetbase.h"

using namespace std;

extern const char *TYPESTR_PICTURE;
extern ResourceManagerBase *resourcemanager;

PictureWidgetBase::PictureWidgetBase(const char *name, int x, int y, ImageResource *image)
: WidgetBase(x, y) {
  this->image = image;
  this->type = TYPESTR_PICTURE;

  MemberFunctionProperty<PictureWidgetBase> *mp;
  mp = new MemberFunctionProperty<PictureWidgetBase>("image", this, &PictureWidgetBase::pget_Image, &PictureWidgetBase::pset_Image, true);
  AddProperty(mp);

  MemberMethodHandler<PictureWidgetBase> *mh;
  mh = new MemberMethodHandler<PictureWidgetBase>("reloadimage", this, 0, &PictureWidgetBase::m_ReloadImage);
  AddMethod(mh);
}

void PictureWidgetBase::ReloadImage(void) {
  if(this->image) {
    ImageResource *newimage = resourcemanager->LoadImageResource(this->image->GetFilename(), false);
    if(newimage)
      SetImage(newimage);
  }
}

// Property set/get functions

// -- image

Variant PictureWidgetBase::pget_Image(void) {
  if(image)
    return anytovariant(image->GetFilename());
  else
    return VARNULL;
}

int PictureWidgetBase::pset_Image(Variant value) {
  ImageResource *image = resourcemanager->LoadImageResource(value, false);
  if(image) {
    SetImage(image);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// Method functions

Variant PictureWidgetBase::m_ReloadImage(int numargs, Variant args[]) {
  ReloadImage();
  return VARNULL;
}

