/***************************************************************************
         resourcemanagerbase.cpp  -  base class for resource managers
                             -------------------
    begin                : Sun Nov 30 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "resourcemanagerbase.h"

#include <iostream>
#include <errno.h>
#include "stringlist.h"
#include "utils.h"
 
using namespace std;

ResourceManagerBase::ResourceManagerBase(void) {
  imagepath = "";
  fontpath = "";
}

ResourceManagerBase::~ResourceManagerBase(void) {
}

ImageResource *ResourceManagerBase::LoadImageResource(std::string filename, bool force) {
  string fullpath = SearchPath(imagepath, filename);

  if(fullpath == "" || !checkFileExists(fullpath.c_str()))
    return NULL;

  return InternalLoadImageResource(fullpath, force);
}

FontResource *ResourceManagerBase::LoadFontResource(std::string filename, int fontsize) {
  string fullpath = SearchPath(fontpath, filename);
  
  if(fullpath == "" || !checkFileExists(fullpath.c_str()))
    return NULL;

  return InternalLoadFontResource(fullpath, fontsize);
}

std::string ResourceManagerBase::SearchPath(std::string &path, std::string &filename) {
  if(filename == "")
    return "";

  if(filename.find("/") != string::npos || path == "")
    return expandPath(filename);

  StringList pathlist;
  pathlist.SetDelimitedText(path.c_str(), ':');
  pathlist.Insert(".", 0);

  for(int i=0; i<pathlist.size(); i++) {
    // Make up test path
    string testfilename = pathlist[i];
    if(testfilename[testfilename.length()] != '/')
      testfilename += "/";
    testfilename += filename;
    // Check if the file exists in this path
    int rc = statfile(testfilename.c_str());
    if(rc == 0)
      return testfilename;
    else if(rc != ENOENT) {
      // File exists but isn't accessible
      checkFileExists(testfilename.c_str());
      return "";
    }
  }
  // Not found
  cerr << "Could not find file " << filename << " in path " << path << endl;
  return "";
}

void ResourceManagerBase::SetImageSearchPath(std::string path) {
  imagepath = path;
}

void ResourceManagerBase::SetFontSearchPath(std::string path) {
  fontpath = path;
}
