/***************************************************************************
                          stringlist.cpp  -  generic string list class
                             -------------------
    begin                : Sat May 10 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <cstring>
#include <iostream>
#include <vector>

#include "stringlist.h"

using namespace std;

// FIXME: this should be changed to use C++ strings instead of char *

// FIXME: find a better place for this
char *makeNameValuePair(const char *name, const char *value);
  
StringList::StringList(void) {
  //
}

StringList::~StringList(void) {
  Clear();
}

const char *StringList::GetItem(int index) {
  if(index >= 0 && index < strings.size())
    return strings[index];
  else {
    cerr << "StringList::GetItem - index " << index << " out of range" << endl;
    return NULL;
  }
}

void StringList::SetItem(int index, const char *str) {
  if(index >= 0 && index < strings.size()) {
    free(strings[index]);
    strings[index] = strdup(str);
  }
  else
    cerr << "StringList::SetItem - index " << index << " out of range" << endl;
}

void StringList::Add(const char *str) {
  if(str) {
    char *storedstr = strdup(str);
    strings.push_back(storedstr);
  }
  else
    cerr << "StringList::Add - cannot store null strings!" << endl;
}

void StringList::Insert(const char *str, int index) {
  if(str) {
    if(index >= 0 && index <= strings.size()) {
      char *storedstr = strdup(str);
      strings.insert(strings.begin() + index, storedstr);
    }
    else
      cerr << "StringList::Insert - index " << index << " out of range" << endl;
  }
  else
    cerr << "StringList::Insert - cannot store null strings!" << endl;
}

void StringList::Delete(int index) {
  if(index >= 0 && index < strings.size()) {
    char *storedstr = strings[index];
    strings.erase(strings.begin() + index);
    free(storedstr);
  }
  else
    cerr << "StringList::Delete - index " << index << " out of range" << endl;
}

char *StringList::GetName(int index) {
  if(index >= 0 && index < strings.size()) {
    char *chr = strchr(strings[index], '=');
    if(chr) {
      char *name = (char *)malloc((chr - strings[index]) + 1);
      strncpy(name, strings[index], chr - strings[index]);
      return name;
    }
    else {
      cerr << "StringList::GetName - item at index " << index << " is not a name-value pair" << endl;
      return NULL;
    }
  }
  else {
    cerr << "StringList::GetName - index " << index << " out of range" << endl;
    return NULL;
  }
}

const char *StringList::GetValue(int index) {
  if(index >= 0 && index < strings.size()) {
    char *chr = strchr(strings[index], '=');
    if(chr)
      return(chr+1);
    else {
      cerr << "StringList::GetName - item at index " << index << " is not a name-value pair" << endl;
      return NULL;
    }
  }
  else {
    cerr << "StringList::GetName - index " << index << " out of range" << endl;
    return NULL;
  }
}

const char *StringList::GetValue(const char *name) {
  // FIXME
}

void StringList::SetValue(int index, const char *value) {
  // FIXME
}

void StringList::SetValue(const char *name, const char *value) {
  int index;
  
  if(name==NULL) {
    cerr << "StringList::SetValue - name cannot be null!" << endl;
    return;
  }
  
  index = IndexOfName(name);
  if(index > 0) {
    if(strlen(strings[index]) < (strlen(name) + strlen(value) + 1)) {
      strcpy(strings[index], name);
      strcat(strings[index], "=");
      strcat(strings[index], value);
    }
    else {
      // Have to make a new string
      free(strings[index]);
      strings[index] = makeNameValuePair(name, value);
    }
  }
  else {
    // Add new name=value pair
    strings.push_back(makeNameValuePair(name, value));
  }
}

int StringList::IndexOf(const char *item) {
  for(int i=0; i<strings.size(); i++) {
    if(strcmp(item, strings[i]) == 0)
      return i;
  }
  return -1;
}

int StringList::IndexOfName(const char *name) {
  int length = strlen(name);
  
  for(int i=0; i<strings.size(); i++) {
    if(strncmp(name, strings[i], length) == 0) {
      if(strings[i][length] == '=')
        return i;
    }
  }
  return -1;
}

int StringList::size(void) {
  return strings.size();
}

const char *StringList::operator[](int i) {
  return GetItem(i);
}

char *makeNameValuePair(const char *name, const char *value) {
  char *item;
  int length;

  length = strlen(name) + 2;
  if(value)
    length += strlen(value);
  item = (char *)malloc(length);

  strcpy(item, name);
  strcat(item, "=");
  strcat(item, value);

  return item;
}

void StringList::Clear(void) {
  while(strings.size() > 0) {
    char *str = strings.back();
    if(str)
      free(str);
    strings.pop_back();
  }
}

void StringList::SetDelimitedText(const char *delimitedtext, char delimiter) {
  string buffer = "";

  Clear();
  for(int i=0; i<strlen(delimitedtext); i++) {
    if(delimitedtext[i] != delimiter)
      buffer += delimitedtext[i];
    else {
      Add(buffer.c_str());
      buffer = "";
    }
  }
  if(buffer != "")
    Add(buffer.c_str());
}
