/***************************************************************************
        textfieldwidgetbase.h  -  base class for textfield widgets
                             -------------------
    begin                : Fri Oct 3 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INCLUDED__TEXTFIELDWIDGETBASE_H
#define INCLUDED__TEXTFIELDWIDGETBASE_H

#include <sys/time.h>

#include "utils.h"
#include "widgetbase.h"
#include "resource.h"
#include "variant.h"

#define TEXTINPUT_DELAY_TIME 1000000

enum TextFieldInsertMode {
  INSMODE_INSERT,
  INSMODE_OVERWRITE,
  INSMODE_BACKOVER
};

class TextFieldWidgetBase: public virtual WidgetBase {
  public:
    TextFieldWidgetBase(const char *name, int x, int y, int w, int h, FontResource *font, eboxy_color fontcolor, int maxlength);
    virtual ~TextFieldWidgetBase(void);
    virtual bool SendChar(char c, TextFieldInsertMode insmode) = 0;
    virtual void Backspace(void) = 0;
    virtual void DeleteCurrChar(void) = 0;
    virtual void SetBackgroundImage(ImageResource *image) = 0;
    virtual void SetBackgroundFocusImage(ImageResource *image) = 0;
    virtual void SetBackgroundStyle(int style) = 0;
    virtual void SetFont(FontResource *font) = 0;
    virtual void SetFontColor(eboxy_color c) = 0;
    virtual void SetBorderX(int pixels) = 0;
    virtual void SetBorderY(int pixels) = 0;
    virtual void SetValidChars(std::string validchars) = 0;
    virtual int GetCursorPos(void) = 0;
    virtual void SetCursorPos(int pos) = 0;
    virtual void SetText(std::string str);
    virtual void InputDigit(int chr);
    virtual std::string GetText(void);
    virtual bool StandardAction(int actiontype);
  protected:
    std::string text;
    std::string validchars;
    int maxlength;
    ImageResource *background;
    ImageResource *backgroundfocus;
    int backgroundstyle;
    FontResource *font;
    int fontsize;
    eboxy_color fontcolor;
    int borderx;
    int bordery;
    int inpdigit;
    const char *inpdigitptr;
    timeval input_time;

    Variant pget_Text(void);
    int pset_Text(Variant value);
    Variant pget_Font(void);
    int pset_Font(Variant value);
    Variant pget_FontSize(void);
    int pset_FontSize(Variant value);
    Variant pget_FontColor(void);
    int pset_FontColor(Variant value);
    Variant pget_Background(void);
    int pset_Background(Variant value);
    Variant pget_BackgroundFocus(void);
    int pset_BackgroundFocus(Variant value);
    Variant pget_BackgroundStyle(void);
    int pset_BackgroundStyle(Variant value);
    Variant pget_MaxLength(void);
    int pset_MaxLength(Variant value);
    Variant pget_BorderX(void);
    int pset_BorderX(Variant value);
    Variant pget_BorderY(void);
    int pset_BorderY(Variant value);
    Variant pget_CursorPos(void);
    int pset_CursorPos(Variant value);
    Variant pget_ValidChars(void);
    int pset_ValidChars(Variant value);
};

timeval getSystemTime(void);
bool compareTimes(timeval *t1, timeval *t2);
      
#endif
