/***************************************************************************
                          timerbase.cpp  -  base class for timers
                             -------------------
    begin                : Fri Nov 7 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "utils.h"
#include "memberfunctionproperty.h"
#include "membermethodhandler.h"

#include "timerbase.h"

using namespace std;

extern const char *TYPESTR_TIMER;

TimerObjectBase::TimerObjectBase(const char *name, int interval, bool autostart)
: Object(name) {
  this->interval = interval;
  this->enabled = false;
  this->autostart = autostart;
  this->type = TYPESTR_TIMER;

  MemberFunctionProperty<TimerObjectBase> *mp;
  mp = new MemberFunctionProperty<TimerObjectBase>("enabled", this, &TimerObjectBase::pget_Enabled, &TimerObjectBase::pset_Enabled, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<TimerObjectBase>("interval", this, &TimerObjectBase::pget_Interval, &TimerObjectBase::pset_Interval, false);
  AddProperty(mp);

  MemberMethodHandler<TimerObjectBase> *mh;
  mh = new MemberMethodHandler<TimerObjectBase>("start", this, 0, &TimerObjectBase::m_Start);
  AddMethod(mh);
  mh = new MemberMethodHandler<TimerObjectBase>("stop", this, 0, &TimerObjectBase::m_Stop);
  AddMethod(mh);

  AddEvent("OnTimer");
  defaultevent = 0;
}

TimerObjectBase::~TimerObjectBase(void) {
  // Make sure timer isn't still going!
  SetEnabled(false);
}

void TimerObjectBase::SetInterval(int interval) {
  if(interval != this->interval) {
    this->interval = interval;
    if(this->enabled) {
      // Restart timer so that change takes effect
      SetEnabled(false);
      SetEnabled(true);
    }
  }
}

void TimerObjectBase::SetEnabled(bool enable) {
  // Override in parent classes to init/uninit real timer
  this->enabled = enable;
}

void TimerObjectBase::Fire(void) {
  FireEvent("OnTimer");
}

void TimerObjectBase::AutoStart(void) {
  // This is used to handle timers that start off enabled.
  // We do it this way because we don't want to actually start
  // the timer until we're ready.
  if(autostart)
    SetEnabled(true);
}

int TimerObjectBase::DoInitialise(void) {
  return 0;
}

void TimerObjectBase::DoUninitialise(void) {
  // Make sure timer is disabled (stopped)
  SetEnabled(false);
}


/* Property get/set functions */

// -- interval --

Variant TimerObjectBase::pget_Interval(void) {
  return anytovariant(interval);
}

int TimerObjectBase::pset_Interval(Variant value) {
  SetInterval(value);
  return 0;
}

// -- enabled --

Variant TimerObjectBase::pget_Enabled(void) {
  return anytovariant(enabled);
}

int TimerObjectBase::pset_Enabled(Variant value) {
  SetEnabled(value);
  return 0;
}

/* Method functions */

Variant TimerObjectBase::m_Start(int numargs, Variant args[]) {
  SetEnabled(true);
  return VARNULL;
}

Variant TimerObjectBase::m_Stop(int numargs, Variant args[]) {
  SetEnabled(false);
  return VARNULL;
}

