/***************************************************************************
         togglebuttonwidgetbase.cpp  -  togglebutton widget base class
                             -------------------
    begin                : Mon Nov 17 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "actionmanager.h"
#include "resourcemanagerbase.h"
#include "variant.h"
#include "utils.h"
#include "../errors.h"
#include "memberfunctionproperty.h"
#include "pagebase.h"

#include "togglebuttonwidgetbase.h"
 
using namespace std;

extern const char *TYPESTR_TOGGLEBUTTON;
extern ResourceManagerBase *resourcemanager;

ToggleButtonWidgetBase::ToggleButtonWidgetBase(const char *name, int x, int y, int w, int h)
: WidgetBase(x, y) {
  this->w = w;
  this->h = h;
  // Set defaults
  tabstop = true;
  action = "";
  normalimage = NULL;
  normalfocusimage = NULL;
  checkedimage = NULL;
  checkedfocusimage = NULL;
//X  disabledimage = NULL;
  enabled = true;
  checked = false;
  this->type = TYPESTR_TOGGLEBUTTON;

  MemberFunctionProperty<ToggleButtonWidgetBase> *mp;
  mp = new MemberFunctionProperty<ToggleButtonWidgetBase>("normalimage", this, &ToggleButtonWidgetBase::pget_NormalImage, &ToggleButtonWidgetBase::pset_NormalImage, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ToggleButtonWidgetBase>("normalfocusimage", this, &ToggleButtonWidgetBase::pget_NormalFocusImage, &ToggleButtonWidgetBase::pset_NormalFocusImage, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ToggleButtonWidgetBase>("checkedimage", this, &ToggleButtonWidgetBase::pget_CheckedImage, &ToggleButtonWidgetBase::pset_CheckedImage, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ToggleButtonWidgetBase>("checkedfocusimage", this, &ToggleButtonWidgetBase::pget_CheckedFocusImage, &ToggleButtonWidgetBase::pset_CheckedFocusImage, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ToggleButtonWidgetBase>("enabled", this, &ToggleButtonWidgetBase::pget_Enabled, &ToggleButtonWidgetBase::pset_Enabled, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ToggleButtonWidgetBase>("checked", this, &ToggleButtonWidgetBase::pget_Checked, &ToggleButtonWidgetBase::pset_Checked, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<ToggleButtonWidgetBase>("action", this, &ToggleButtonWidgetBase::pget_Action, &ToggleButtonWidgetBase::pset_Action, false);
  AddProperty(mp);

  AddEvent("OnClick");
  AddEvent("OnGotFocus");
  AddEvent("OnLostFocus");
}

ToggleButtonWidgetBase::~ToggleButtonWidgetBase(void) {
  //
}

void ToggleButtonWidgetBase::SetAction(std::string actionname) {
  this->action = GetUserAction(actionname);
}

void ToggleButtonWidgetBase::SetEnabled(bool enabled) {
  this->enabled = enabled;
}

bool ToggleButtonWidgetBase::GetChecked(void) {
  return checked;
}

void ToggleButtonWidgetBase::SetChecked(bool checked) {
  this->checked = checked;
}

void ToggleButtonWidgetBase::Execute(void) {
  SetChecked(!GetChecked());
}

bool ToggleButtonWidgetBase::StandardAction(int actiontype) {
  switch(actiontype) {
    case STDACTION_ACCEPT:
      SetChecked(!GetChecked());
      return true;
      break;
    case STDACTION_ITEMPREV:
      FindParentPage()->FocusPreviousWidget();
      return true;
      break;
    case STDACTION_ITEMNEXT:
      FindParentPage()->FocusNextWidget();
      return true;
      break;
    default:
      return false;
  }
}

// Properties

// -- normalimage

Variant ToggleButtonWidgetBase::pget_NormalImage(void) {
  if( normalimage )
    return anytovariant(normalimage->GetFilename());
  else
    return VARNULL;
}

int ToggleButtonWidgetBase::pset_NormalImage(Variant value) {
  ImageResource *image = resourcemanager->LoadImageResource(value, false);
  if(image) {
    SetNormalImage(image);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// -- normalfocusimage

Variant ToggleButtonWidgetBase::pget_NormalFocusImage(void) {
  if( normalfocusimage )
    return anytovariant(normalfocusimage->GetFilename());
  else
    return VARNULL;
}

int ToggleButtonWidgetBase::pset_NormalFocusImage(Variant value) {
  ImageResource *image = resourcemanager->LoadImageResource(value, false);
  if(image) {
    SetNormalFocusImage(image);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// -- checkedimage

Variant ToggleButtonWidgetBase::pget_CheckedImage(void) {
  if( checkedimage )
    return anytovariant(checkedimage->GetFilename());
  else
    return VARNULL;
}

int ToggleButtonWidgetBase::pset_CheckedImage(Variant value) {
  ImageResource *image = resourcemanager->LoadImageResource(value, false);
  if(image) {
    SetCheckedImage(image);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// -- checkedfocusimage

Variant ToggleButtonWidgetBase::pget_CheckedFocusImage(void) {
  if( checkedfocusimage )
    return anytovariant(checkedfocusimage->GetFilename());
  else
    return VARNULL;
}

int ToggleButtonWidgetBase::pset_CheckedFocusImage(Variant value) {
  ImageResource *image = resourcemanager->LoadImageResource(value, false);
  if(image) {
    SetCheckedFocusImage(image);
    return EBERR_SUCCESS;
  }
  else
    return EBERR_FILE_ERROR;
}

// -- enabled --

Variant ToggleButtonWidgetBase::pget_Enabled(void) {
  return anytovariant(enabled);
}

int ToggleButtonWidgetBase::pset_Enabled(Variant value) {
  SetEnabled(value);
  return EBERR_SUCCESS;
}

// -- checked --

Variant ToggleButtonWidgetBase::pget_Checked(void) {
  return anytovariant(GetChecked());
}

int ToggleButtonWidgetBase::pset_Checked(Variant value) {
  SetChecked(value);
  return EBERR_SUCCESS;
}

// -- action --

Variant ToggleButtonWidgetBase::pget_Action(void) {
  return anytovariant(action);
}

int ToggleButtonWidgetBase::pset_Action(Variant value) {
  SetAction(value);
  return EBERR_SUCCESS;
}
