/***************************************************************************
                 variant.cpp  -  simple implementation of a variant
                             -------------------
    begin                : Wed Nov 12 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <sstream>
 
#include "variant.h"

// Prototypes
bool stringtobool(std::string &str);
   
using namespace std;

Variant::Variant(void) {
  internaltype = VARTYPE_UNINITIALISED;
  ClearVars();
}

Variant::~Variant(void) {
  // nothing to do here
}

bool Variant::IsEmpty(void) {
  return empty;
}

void Variant::Clear(void) {
  ClearVars();
}

void Variant::ClearVars(void) {
  intvalue = 0;
  stringvalue = "";
  // Note: if more types are added, clear them here
  empty = true;
  // Do not add code here to do anything other than clearing the internal variables,
  // because this is also called from the constructor
}

int Variant::GetType(void) {
  return internaltype;
}

void Variant::SetType(int type) {
  internaltype = type;
}

Variant Variant::operator= (string value) {
  if(internaltype == VARTYPE_UNINITIALISED)
    SetType(VARTYPE_STRING);
  
  switch(internaltype) {
    case VARTYPE_STRING:
      stringvalue = value;
      break;
    case VARTYPE_INT:
      intvalue = atoi(value.c_str());
      break;
    case VARTYPE_BOOL:
      if(stringtobool(value))
        intvalue = true;
      else
        intvalue = false;
    default:
      cerr << "Variant::operator=: Invalid type conversion!" << endl;
  }
  
  return *this;
}
  
Variant Variant::operator= (const char *value) {
  string str;
  str = value;
  return operator=(str);
}

Variant Variant::operator= (void *value) {
  SetType(VARTYPE_OBJREF);
  intvalue = (int)value;
  return *this;
}

Variant Variant::operator= (int value) {
  if(internaltype == VARTYPE_UNINITIALISED)
    SetType(VARTYPE_INT);

  switch(internaltype) {
    case VARTYPE_STRING:
      {
        ostringstream strm;
        strm << value;
        stringvalue = strm.str();
      }
      break;
    case VARTYPE_INT:
      intvalue = value;
      break;
    case VARTYPE_BOOL:
      if(value)
        intvalue = true;
      else
        intvalue = false;
      break;
    default:
      cerr << "Variant::operator=: Invalid type conversion!" << endl;
  }

  return *this;
}

Variant Variant::operator= (bool value) {
  if(internaltype == VARTYPE_UNINITIALISED)
    SetType(VARTYPE_BOOL);

  switch(internaltype) {
    case VARTYPE_STRING:
      if(value)
        stringvalue = "true";
      else
        stringvalue = "false";
      break;
    case VARTYPE_BOOL:
    case VARTYPE_INT:
      intvalue = value;
      break;
    default:
      cerr << "Variant::operator=: Invalid type conversion!" << endl;
  }

  return *this;
}

Variant::operator std::string() {
  switch(internaltype) {
    case VARTYPE_STRING:
      // Do nothing
      break;
    case VARTYPE_INT:
      {
        ostringstream strm;
        strm << intvalue;
        stringvalue = strm.str();
      }
      break;
    case VARTYPE_BOOL:
      if(intvalue)
        stringvalue = "true";
      else
        stringvalue = "false";
      break;
    case VARTYPE_UNINITIALISED:
      stringvalue = "";
      break;
    default:
      cerr << "Variant::operator string: Invalid type conversion!" << endl;
  }
  return stringvalue;
}

Variant::operator int() {
  switch(internaltype) {
    case VARTYPE_STRING:
      intvalue = atoi(stringvalue.c_str());
      break;
    case VARTYPE_BOOL:
    case VARTYPE_INT:
      // Do nothing
      break;
    case VARTYPE_UNINITIALISED:
      intvalue = 0;
      break;
    default:
      cerr << "Variant::operator int: Invalid type conversion!" << endl;
  }
  return intvalue;
}

Variant::operator bool() {
  switch(internaltype) {
    case VARTYPE_STRING:
      if(stringtobool(stringvalue))
        intvalue = true;
      else
        intvalue = false;
      break;
    case VARTYPE_BOOL:
    case VARTYPE_INT:
      // Do nothing
      break;
    case VARTYPE_UNINITIALISED:
      intvalue = false;
      break;
    default:
      cerr << "Variant::operator bool: Invalid type conversion!" << endl;
  }
  return intvalue;
}

Variant::operator void *() {
  if(internaltype == VARTYPE_OBJREF)
    return (void *)intvalue;
  else
    return NULL;
}

bool Variant::operator== (Variant value) {
  switch(internaltype) {
    case VARTYPE_OBJREF:
      {
        int intval = value;
        if(value.GetType() == VARTYPE_OBJREF)
          return intvalue == intval;
        else
          return false;
      }
    case VARTYPE_STRING:
      {
        string strval = value;
        return (stringvalue == strval);
      }
    case VARTYPE_BOOL:
      {
      bool boolval = value;
      return (intvalue == boolval);
      }
    case VARTYPE_INT:
      {
      int intval = value;
      return (intvalue == intval);
      }
    case VARTYPE_UNINITIALISED:
      return (value.GetType() == VARTYPE_UNINITIALISED);
    default:
      return false;
  }
}

bool Variant::operator== (std::string value) {
  switch(internaltype) {
    case VARTYPE_STRING:
      return (stringvalue == value);
    default:
      return false;
  }  
}

bool Variant::operator== (const char *value) {
  string str;
  str = value;
  return operator==(str);
}

bool Variant::operator== (int value) {
  switch(internaltype) {
    case VARTYPE_INT:
      return (intvalue == value);
    default:
      return false;
  }
}

bool Variant::operator== (bool value) {
  switch(internaltype) {
    case VARTYPE_BOOL:
      return (intvalue == value);
    default:
      return false;
  }
}

ostream &operator<<(ostream &stream, Variant v) {
  stream << std::string(v);
  return stream;
}

const char *Variant::c_str(void) {
  string str = *this;
  return stringvalue.c_str();
}

Variant Variant::operator+ (Variant value) {
  Variant ret;
  if(value.GetType() == VARTYPE_STRING || this->GetType() == VARTYPE_STRING)
    ret = ((string)*this + (string)value);
  else
    ret = ((int)*this + (int)value);
  return ret;
}

// Local functions

bool stringtobool(std::string &str) {
  const char *cstr = str.c_str();
  return (strcasecmp(cstr, "true") == 0 || strcasecmp(cstr, "on") == 0 || strcasecmp(cstr, "yes") == 0 || atoi(cstr) != 0);
}
