/***************************************************************************
                 variant.h  -  simple implementation of a variant
                             -------------------
    begin                : Wed Nov 12 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INCLUDED__VARIANT_H
#define INCLUDED__VARIANT_H

#include <string>
#include <iostream>

enum {
  VARTYPE_UNINITIALISED,
  VARTYPE_INT,
  VARTYPE_BOOL,
  VARTYPE_STRING,
  VARTYPE_OBJREF
};

class Variant {
  protected:
    int internaltype;
    int intvalue;
    std::string stringvalue;
    bool empty;

    void ClearVars(void);
  public:
    Variant(void);
    virtual ~Variant(void);
  
    bool IsEmpty(void);
    void Clear(void);
    int GetType(void);
    void SetType(int type);

    Variant operator+ (Variant value);
    Variant operator= (void *value);
    Variant operator= (std::string value);
    Variant operator= (const char *value);
    Variant operator= (int value);
    Variant operator= (bool value);
    bool operator== (Variant value);
    bool operator== (std::string value);
    bool operator== (const char *value);
    bool operator== (int value);
    bool operator== (bool value);
    template <class T> bool operator!= (T value) {
      return !(operator==(value));
    }
    operator std::string();
    operator int();
    operator bool();
    const char *c_str(void);
    operator void *();

    friend std::ostream &operator<<(std::ostream &stream, Variant v);
};

template <class T> Variant anytovariant(T anything) {
  Variant v;
  v = anything;
  return v;
};

template <class T> Variant *newVariant(T anything) {
  Variant *vptr = new Variant;
  *vptr = anything;
  return vptr;
};

const Variant VARNULL;


#endif
