/***************************************************************************
                          widgetbase.h  -  base class for widgets
                             -------------------
    begin                : Sun Sep 21 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INCLUDED__WIDGETBASE_H
#define INCLUDED__WIDGETBASE_H

#include "customobject.h"

class PageBase;

// Standard actions - operations that can be performed on most widgets
#define STDACTION_ACCEPT        0
#define STDACTION_ITEMPREV      1
#define STDACTION_ITEMNEXT      2
#define STDACTION_ITEMPAGEPREV  3
#define STDACTION_ITEMPAGENEXT  4
#define STDACTION_ITEMFIRST     5
#define STDACTION_ITEMLAST      6

enum WIDGET_ORIGIN {
  ORIGIN_SYSTEM,
  ORIGIN_FILE,
  ORIGIN_DYNAMIC
};

class WidgetBase: public CustomObject {
  public:
    WidgetBase(void);
    WidgetBase(int x, int y);
    virtual ~WidgetBase(void);
    // Note: Instantiate() is not defined here because it returns whatever type the
    // real widget is, and we won't know anything about that type
    virtual void Uninstantiate(void) = 0;
    virtual void SetParent(WidgetBase *p);
    virtual bool GetTabStop(void);
    virtual void SetTabStop(bool value);
    virtual int GetX(void);
    virtual int GetY(void);
    virtual void SetX(int value);
    virtual void SetY(int value);
    virtual void SetPosition(int x, int y);
    virtual int GetWidth(void);
    virtual int GetHeight(void);
    virtual void SetWidth(int value);
    virtual void SetHeight(int value);
    virtual void GotFocus(void);
    virtual void LostFocus(void);
    virtual bool IsGUIFocused(void) = 0;
    virtual bool IsEnabled(void) = 0;
    virtual int Clone(WidgetBase *newwidget);
    virtual bool StandardAction(int actiontype);
    virtual bool IsInstantiated(void) = 0;
    virtual void SetVisible(bool visible) = 0;
    virtual bool GetVisible(void);
    void SetOrigin(int origin);
    int GetOrigin(void);
    PageBase *FindParentPage(void);
  protected:
    int x;
    int y;
    int w;
    int h;
    bool tabstop;
    WidgetBase *parent;
    bool focused;
    bool visible;
    int origin;

    Variant pget_X(void);
    int pset_X(Variant value);
    Variant pget_Y(void);
    int pset_Y(Variant value);
    Variant pget_Width(void);
    int pset_Width(Variant value);
    Variant pget_Height(void);
    int pset_Height(Variant value);
    Variant pget_Visible(void);
    int pset_Visible(Variant value);

    Variant m_SetFocus(int numargs, Variant args[]);
};

#endif
