/***************************************************************************
                          buttonwidget.cpp - eboxy button widget
                             -------------------
    begin                : Sat Aug 3 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <cstring>
#include <cstdlib>
#include "SDL_gui.h"

#include "../../basewidgets/utils.h"
#include "gui_extensions.h"
#include "../../basewidgets/actionmanager.h"
#include "../../errors.h"

#include "buttonwidget.h"
#include "../../basewidgets/pagebase.h"

using namespace std;

extern ActionManager *actionmanager;
extern const char *TYPESTR_PAGE;

ButtonWidget::ButtonWidget(const char *name, int x, int y, int w, int h)
: ButtonWidgetBase(name, x, y, w, h), WidgetBase(x, y), Object(name) {
  captionlabel = NULL;
}

ButtonWidget::~ButtonWidget(void) {
  if(realwidget)
    Uninstantiate();
}

GUI_Widget *ButtonWidget::Instantiate(void) {
  if(w<0) w=0;
  if(h<0) h=0;

  if(normalimage == NULL && font == NULL) {
    cerr << "ButtonWidget::Instantiate - button must have a normalimage and/or a font" << endl;
    return NULL;
  }
      
  GUI_ExtButton *button = new GUI_ExtButton(name, x, y, w, h);

  SetupCaptionLabel(button);

  if(normalimage) {
    button->SetNormalImage((GUI_Surface *)(normalimage->GetHandle()));
    button->SetHighlightImage((GUI_Surface *)(normalimage->GetHandle()));  // this is not a typo - this helps with mouse support
  }
  if(pressedimage)
    button->SetPressedImage((GUI_Surface *)(pressedimage->GetHandle()));
  if(disabledimage)
    button->SetDisabledImage((GUI_Surface *)(disabledimage->GetHandle()));

  if(!enabled)
    button->SetFlags(WIDGET_DISABLED);
  else
    button->ClearFlags(WIDGET_DISABLED);

  GUI_EventHandler<ButtonWidget> *cb = new GUI_EventHandler<ButtonWidget>(this, &ButtonWidget::OnClick);
  button->SetClick(cb);
	cb->DecRef();

  cb = new GUI_EventHandler<ButtonWidget>(this, &ButtonWidget::OnUpDown);
  button->SetUpDownCallback(cb);
	cb->DecRef();

  if(action.length() > 0) {
    int rc = actionmanager->RegisterLocalAction(action, this);
    if(rc == EBERR_ACTION_INVALID) {
      cerr << "Warning: action \"" << action << "\" is not a valid user action" << endl;
    }
    else if(rc == EBERR_ACTION_ASSIGNED) {
      cerr << "Warning: action \"" << action << "\" is assigned to more than one widget in this context" << endl;
    }
  }

  realwidget = button;
  return button;
}

void ButtonWidget::Uninstantiate(void) {
  if(realwidget) {
    if(captionlabel) {
      captionlabel->DecRef();
      captionlabel = NULL;
    }
    focused = false;

    if(action.length())
      actionmanager->UnregisterLocalAction(action, this);
  }
  ebWidget::Uninstantiate();
}

void ButtonWidget::SetNormalImage(ImageResource *image) {
  if(image) {
    this->normalimage = image;
    if(realwidget)
      ((GUI_ExtButton *)realwidget)->SetNormalImage((GUI_Surface *)(normalimage->GetHandle()));
    if(this->w==-1)
      SetWidth(-1);
    if(this->h==-1)
      SetHeight(-1);
  }
}

void ButtonWidget::SetSelectedImage(ImageResource *image) {
  if(image) {
    this->selectedimage = image;
  }
}

void ButtonWidget::SetPressedImage(ImageResource *image) {
  if(image) {
    this->pressedimage = image;
    if(realwidget)
      ((GUI_ExtButton *)realwidget)->SetPressedImage((GUI_Surface *)(pressedimage->GetHandle()));
  }
}

void ButtonWidget::SetDisabledImage(ImageResource *image) {
  if(image) {
    this->disabledimage = image;
    if(realwidget)
      ((GUI_ExtButton *)realwidget)->SetDisabledImage((GUI_Surface *)(disabledimage->GetHandle()));
  }
}

void ButtonWidget::SetCaption(string caption) {
  ButtonWidgetBase::SetCaption(caption);
  if(realwidget) {
    if(!captionlabel)
      SetupCaptionLabel(realwidget);

    if(captionlabel) {
      captionlabel->SetText(caption.c_str());
      realwidget->MarkChanged();
    }
  }
}

void ButtonWidget::SetCaptionX(int value) {
  this->captionx = value;
  if(realwidget && captionlabel) {
    SDL_Rect r = captionlabel->GetArea();
    captionlabel->SetPosition(value, r.y);
  }
}

void ButtonWidget::SetCaptionY(int value) {
  this->captiony = value;
  if(realwidget && captionlabel) {
    SDL_Rect r = captionlabel->GetArea();
    captionlabel->SetPosition(r.x, value);
  }
}

void ButtonWidget::SetCaptionDropX(int value) {
  this->captiondropx = value;
  if(realwidget)
    ((GUI_ExtButton *)realwidget)->SetCaptionDropX(value);
}

void ButtonWidget::SetCaptionDropY(int value) {
  this->captiondropy = value;
  if(realwidget)
    ((GUI_ExtButton *)realwidget)->SetCaptionDropY(value);
}

void ButtonWidget::SetFont(FontResource *font) {
  this->font = font;
  if(realwidget) {
    if(!captionlabel)
      SetupCaptionLabel(realwidget);

    if(captionlabel) {
      captionlabel->SetFont((GUI_Font *)(this->font->GetHandle()));
      captionlabel->SetText(caption.c_str()); // fix for bug in SDL_GUI
      realwidget->MarkChanged();
    }
  }
}

void ButtonWidget::SetFontColor(const eboxy_color &c) {
  this->fontcolor = c;
  if(realwidget && captionlabel)
    UpdateLabelColor(realwidget);
}

void ButtonWidget::SetFontFocusColor(const eboxy_color &c) {
  this->fontfocuscolor = c;
  if(realwidget && captionlabel)
    UpdateLabelColor(realwidget);
}

void ButtonWidget::SetFontDisabledColor(const eboxy_color &c) {
  this->fontdisabledcolor = c;
  if(realwidget && captionlabel)
    UpdateLabelColor(realwidget);
}

void ButtonWidget::SetFontPressedColor(const eboxy_color &c) {
  this->fontpressedcolor = c;
  if(realwidget && captionlabel)
    UpdateLabelColor(realwidget);
}

void ButtonWidget::SetAction(string actionname) {
  this->action = GetUserAction(actionname);
}

void ButtonWidget::SetEnabled(bool enabled) {
  this->enabled = enabled;
  if(realwidget) {
    if(!enabled) {
      ((GUI_ExtButton *)realwidget)->SetFlags(WIDGET_DISABLED);
      if(focused)
        FindParentPage()->UpdateFocus();
    }
    else
      ((GUI_ExtButton *)realwidget)->ClearFlags(WIDGET_DISABLED);
    if(captionlabel)
      UpdateLabelColor(realwidget);
  }
}

void ButtonWidget::SetWidth(int value) {
  if(value == -1 && normalimage != NULL)
    value = ((GUI_Surface *)(normalimage->GetHandle()))->GetWidth();
  WidgetBase::SetWidth(value);
} 

void ButtonWidget::SetHeight(int value) {
  if(value == -1 && normalimage != NULL)
    value = ((GUI_Surface *)(normalimage->GetHandle()))->GetHeight();
  WidgetBase::SetHeight(value);
}

void ButtonWidget::OnClick(GUI_Object *sender) {
  this->FireEvent("OnClick");
}

void ButtonWidget::OnUpDown(GUI_Object *sender) {
  if(captionlabel)
    UpdateLabelColor(realwidget);
}

void ButtonWidget::SimulateClick(void) {
  if(realwidget) {
    if(!focused)
      FindParentPage()->SetFocus(this);
    ((GUI_ExtButton *)realwidget)->SimulateClick();
  }
  else
    this->FireEvent("OnClick");
}

void ButtonWidget::GotFocus(void) {
  if(!focused) {
    this->FireEvent("OnGotFocus");
    ebWidget::GotFocus();
    WidgetBase::GotFocus();
    if(selectedimage) {
      ((GUI_Button *)realwidget)->SetNormalImage(((GUI_Surface *)(selectedimage->GetHandle())));
      ((GUI_Button *)realwidget)->SetHighlightImage(((GUI_Surface *)(selectedimage->GetHandle())));
    }
    else if(normalimage) {
      ((GUI_Button *)realwidget)->SetNormalImage(((GUI_Surface *)(normalimage->GetHandle())));
      ((GUI_Button *)realwidget)->SetHighlightImage(((GUI_Surface *)(normalimage->GetHandle())));
    }
    else {
      ((GUI_Button *)realwidget)->SetNormalImage(NULL);
      ((GUI_Button *)realwidget)->SetHighlightImage(NULL);
    }
    if(captionlabel)
      UpdateLabelColor(realwidget);
  }
}

void ButtonWidget::LostFocus(void) {
  if(focused) {
    this->FireEvent("OnLostFocus");
    ebWidget::LostFocus();
    WidgetBase::LostFocus();
    if(normalimage) {
      ((GUI_Button *)realwidget)->SetNormalImage(((GUI_Surface *)(normalimage->GetHandle())));
      ((GUI_Button *)realwidget)->SetHighlightImage(((GUI_Surface *)(normalimage->GetHandle())));
    }
    else {
      ((GUI_Button *)realwidget)->SetNormalImage(NULL);
      ((GUI_Button *)realwidget)->SetHighlightImage(NULL);
    }
    if(captionlabel)
      UpdateLabelColor(realwidget);
  }
}

void ButtonWidget::Execute(void) {
  SimulateClick();
}

void ButtonWidget::UpdateLabelColor(GUI_Widget *widget) {
  eboxy_color c;

  if(enabled) {
    if(((GUI_ExtButton *)widget)->IsDown() && fontpressedcolorused)
      c = fontpressedcolor;
    else if((widget->GetFlags() & WIDGET_INSIDE) != 0 && fontfocuscolorused)
      c = fontfocuscolor;
    else
      c = fontcolor;
  }
  else {
    if(fontdisabledcolorused)
      c = fontdisabledcolor;
    else
      c = fontcolor;
  }
  ((GUI_Label *)captionlabel)->SetTextColor(c.r, c.g, c.b);
  ((GUI_Label *)captionlabel)->SetText(caption.c_str()); // fix for bug in SDL_GUI labelwidget
  widget->MarkChanged();
}

void ButtonWidget::SetupCaptionLabel(GUI_Widget *rwidget) {
  if (font != NULL && caption.length() > 0) {
    GUI_ExtButton *button = (GUI_ExtButton *)rwidget;

    // Using GUI_ExtLabel instead of GUI_Label solely because GUI_Label causes a crash if you
    // set the caption to an empty string (!)
		GUI_Label *label = new GUI_ExtLabel("lButtonLabel", captionx, captiony, w, h, (GUI_Font *)(font->GetHandle()), caption.c_str());
		button->SetCaption(label);
		button->SetCaptionDropX(captiondropx);
		button->SetCaptionDropY(captiondropy);
    captionlabel = label;  // we don't decref() on the label here because we keep a reference
    UpdateLabelColor(button);
	}
}

bool ButtonWidget::IsInstantiated(void) {
  return (realwidget != NULL);
}

bool ButtonWidget::IsGUIFocused(void) {
  return ebWidget::IsGUIFocused();
}

bool ButtonWidget::IsEnabled(void) {
  return ebWidget::IsEnabled();
}
