/***************************************************************************
                         gui_extbutton.cpp  -  extended SDL_gui button class
                             -------------------
    begin                : Tue Aug 27 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <assert.h>
#include <cstring>
#include <cstdlib>

#include "SDL_gui.h"
#include "gui_extensions.h"
#include "../../basewidgets/utils.h"

GUI_ExtButton::GUI_ExtButton(const char *aname, int x, int y, int w, int h)
: GUI_Button(aname, x, y, w, h) {
  // Set defaults
  captiondropx = 2;
  captiondropy = 2;

  disabled->DecRef();
  disabled = NULL;
  normal->DecRef();
  normal = NULL;
  highlight->DecRef();
  highlight = NULL;
  pressed->DecRef();
  pressed = NULL;
  updown_callback = NULL;
  down = false;
}

GUI_ExtButton::~GUI_ExtButton(void) {
  // The following prevents the destructor of GUI_Button segfaulting if we're not using some images
  if(!disabled)
  	disabled =  new GUI_Surface("disabled",  SDL_SWSURFACE, 1, 1, 32, 0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000);
  if(!normal)
  	normal =    new GUI_Surface("normal",    SDL_SWSURFACE, 1, 1, 32, 0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000);
  if(!highlight)
  	highlight = new GUI_Surface("highlight", SDL_SWSURFACE, 1, 1, 32, 0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000);
  if(!pressed)
  	pressed =   new GUI_Surface("pressed",   SDL_SWSURFACE, 1, 1, 32, 0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000);
	if (updown_callback)
    updown_callback->DecRef();
}

void GUI_ExtButton::SetCaptionDropX(int amount) {
  captiondropx = amount;
}

void GUI_ExtButton::SetCaptionDropY(int amount) {
  captiondropy = amount;
}

void GUI_ExtButton::Update(int force)
{
	if (parent==0)
		return;

	if (force)
	{
		GUI_Surface *surface = GetCurrentImage(); // gets the appropriate image for the button state (see Button.cc)
	
		if (flags & WIDGET_TRANSPARENT)
			parent->Erase(&area);
		if (surface)
      DrawClipped(surface, parent, area, area.x, area.y);
	}			

	if (caption)
    caption->DoUpdate(force);
}

void GUI_ExtButton::SetUpDownCallback(GUI_Callback *callback)
{
	GUI_ObjectKeep((GUI_Object **) &updown_callback, callback);
}

int GUI_ExtButton::Event(const SDL_Event *event, int xoffset, int yoffset) {
  int result = 0;

	if(event->type == SDL_MOUSEBUTTONUP) {
	  if(caption != NULL && (GetFlags() & WIDGET_PRESSED) != 0 && down) {
      SDL_Rect r = caption->GetArea();
      caption->SetPosition(r.x - captiondropx, r.y - captiondropy);
      MarkChanged();
      down = false;
      if(updown_callback)
        updown_callback->Call(this);
    }
	}
	else if(event->type == SDL_MOUSEMOTION) {
    if(caption != NULL && (GetFlags() & WIDGET_INSIDE) != 0 && (GetFlags() & WIDGET_PRESSED) != 0) {
			int x = event->motion.x - xoffset;
			int y = event->motion.y - yoffset;
			if (pointinrect(x, y, area) && !down) {
        SDL_Rect r = caption->GetArea();
        caption->SetPosition(r.x + captiondropx, r.y + captiondropy);
        MarkChanged();
        down = true;
        if(updown_callback)
          updown_callback->Call(this);
      }
			else if (!pointinrect(x, y, area) && down) {
        SDL_Rect r = caption->GetArea();
        caption->SetPosition(r.x - captiondropx, r.y - captiondropy);
        MarkChanged();
        down = false;
        if(updown_callback)
          updown_callback->Call(this);
      }
    }
  }

  result = GUI_Drawable::Event(event, xoffset, yoffset);
	
  if(event->type == SDL_MOUSEBUTTONDOWN) {
    if(caption != NULL && (GetFlags() & WIDGET_PRESSED) != 0 && !down) {
      SDL_Rect r = caption->GetArea();
      caption->SetPosition(r.x + captiondropx, r.y + captiondropy);
      MarkChanged();
      down = true;
      if(updown_callback)
        updown_callback->Call(this);
    }
  }

  return result;
}

bool GUI_ExtButton::IsDown(void) {
  return down;
}

void GUI_ExtButton::SimulateClick(void) {
  SDL_Rect r;
  if(caption) {
    // Move caption down
    r = caption->GetArea();
    caption->SetPosition(r.x + captiondropx, r.y + captiondropy);
    MarkChanged();
  }
  // Make it look pressed
  SetFlags(WIDGET_PRESSED | WIDGET_INSIDE);
  // Update the screen. Technically this is not thread-safe, but it seems to work fine.
  GUI_Lock();
 	GUI_GetScreen()->DoUpdate(1);
 	GUI_Unlock();
  // Delay for a bit so you can see the effect
  SDL_Delay(75);
  // Make it look normal
  ClearFlags(WIDGET_PRESSED);
  if(caption) {
    // Move caption up
    r = caption->GetArea();
    caption->SetPosition(r.x - captiondropx, r.y - captiondropy);
    MarkChanged();
  }
  // Now, click it!
  Clicked(0,0);
}

GUI_Surface *GUI_ExtButton::GetCurrentImage(void) {
  // This modified version of the original won't return null images (unless normal is null)
	if((flags & WIDGET_DISABLED) && (disabled != NULL))
		return disabled;
	
	if(flags & WIDGET_INSIDE)
  {
		if((flags & WIDGET_PRESSED) && (pressed != NULL))
			return pressed;
    if(highlight != NULL)
  		return highlight;
	}
	return normal;
}
