/***************************************************************************
                gui_extensions.h  -  eboxy SDL_gui extensions
                           -------------------
    begin                : Thu Aug 22 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <vector>

#include "SDL_gui.h"
#include "../../basewidgets/textfieldwidgetbase.h"

#define DOUBLECLICK_TIME 300   // maximum time in ms between clicks to make a double-click

class GUI_ExtLabel : public GUI_Label
{
	protected:
    char *textstring;
    bool autosize;
    bool wordwrap;
    void DrawSurface(GUI_Surface *surf, SDL_Rect *sr, SDL_Rect *dr);
	public:
		GUI_ExtLabel(const char *aname, int x, int y, int w, int h, GUI_Font *afont, const char *s);
		virtual ~GUI_ExtLabel(void);

		void SetFont(GUI_Font *font);
		void SetTextColor(int r, int g, int b);
		void SetText(const char *s);
    void SetAutoSize(bool autosize);
    void SetWordWrap(bool wordwrap);

		virtual void Update(int force);
};

class GUI_ExtButton : public GUI_Button
{
  protected:
    int captiondropx;
    int captiondropy;
    GUI_Callback *updown_callback; // button presses/unpresses
    bool down;
	public:
		GUI_ExtButton(const char *aname, int x, int y, int w, int h);
    virtual ~GUI_ExtButton(void);
    void SetCaptionDropX(int amount);
    void SetCaptionDropY(int amount);
		void SetUpDownCallback(GUI_Callback *callback);
    void Update(int force);
  	virtual int Event(const SDL_Event *event, int xoffset, int yoffset);
    bool IsDown(void);
    void SimulateClick(void);
    GUI_Surface *GetCurrentImage(void);
};

class GUI_ExtScrollBar : public GUI_ScrollBar
{
  private:
    int maximum;
    int minimum;
    int value;
    int background_style;
    bool reversed;
    GUI_Surface *knob_focus;
    void RecalcPosition(void);
    void RecalcValue(void);
  public:
    GUI_ExtScrollBar(const char *aname, int x, int y, int w, int h);
    virtual ~GUI_ExtScrollBar(void);
		void SetPosition(int x, int y);
    void SetWidth(int w);
    void SetHeight(int h);
    void SetValue(int value);
    int GetValue(void);
    void SetPageStep(int step);
    void SetReversed(bool reversed);
    int GetMaximum(void);
    void SetMaximum(int maximum);
    int GetMinimum(void);
    void SetMinimum(int minimum);
    void Update(int force);
    int Event(const SDL_Event *event, int xoffset, int yoffset);
    void SetBackgroundStyle(int style);
    int GetBackgroundStyle(void);
    void SetKnobFocusImage(GUI_Surface *image);
};

class GUI_ListBox : public GUI_Widget
{
	protected:
		GUI_Font *font;
		SDL_Color textcolor;
		SDL_Color selitemtextcolor;
    bool tracking;
    bool scrolling;
    std::vector<char *> listitems;
    int itemheight;
    int useritemheight;
    int topindex;
    int visibleitems;
    int selectedindex;
    int borderx;
    int bordery;
    int itemborderx;
    int itembordery;
    int scrollbarmode;
    bool scrollbarvisible;
    GUI_Surface *background_image;
    GUI_Surface *background_focus_image;
    int background_style;
    GUI_Surface *selected_item_image;
    GUI_Surface *selected_item_focus_image;
    int selected_item_style;
    GUI_ExtScrollBar *scrollbar;
    GUI_Callback *selection_callback; // user moves selection
    GUI_Callback *choose_callback;    // user presses Enter or equivalent
    Uint32 lastclicktime;

    void RecalcItemHeight(void);
    void OnScrollBarMove(GUI_Object *sender);
    void SetupScrollBar(void);
    void CheckScrollBarVisible(void);
	public:
		GUI_ListBox(const char *aname, int x, int y, int w, int h, GUI_Font *afont, int size);
		virtual ~GUI_ListBox(void);

    void SetFont(GUI_Font *font);
		void SetTextColor(int r, int g, int b);
		void SetSelectedItemTextColor(int r, int g, int b);
    void SetBackgroundImage(GUI_Surface *surface);
    void SetBackgroundFocusImage(GUI_Surface *surface);
    void SetBackgroundStyle(int style);
    void SetSelectedItemImage(GUI_Surface *surface);
    void SetSelectedItemFocusImage(GUI_Surface *surface);
    void SetSelectedItemStyle(int style);
    void SetScrollBarMode(int scrollbarmode);
    void SetScrollBarKnobImage(GUI_Surface *surface);
    void SetScrollBarBackgroundImage(GUI_Surface *surface);
    void SetScrollBarBackgroundStyle(int style);
    void SetScrollBarWidth(int width);

    void SetBorderX(int pixels);
    void SetBorderY(int pixels);
    void SetItemBorderX(int pixels);
    void SetItemBorderY(int pixels);
    void SetItemHeight(int pixels);

    void AddItem(const char *text);
    void RemoveItem(int index);
    void Clear(void);
    void SetItemText(int index, const char *text);

    void SetSelectedIndex(int index);
    void InternalSetSelectedIndex(int index, bool forceevent);
    int GetSelectedIndex(void);
    int GetVisibleItems(void);
    void SetTopIndex(int index);
    void SelectNext(int count);
    void SelectPrevious(int count);
    void SelectFirst(void);
    void SelectLast(void);

    int Event(const SDL_Event *event, int xoffset, int yoffset);
    int ItemAtPoint(int x, int y);

		void SetSelectionChangeCallback(GUI_Callback *callback);
		void SetChooseCallback(GUI_Callback *callback);

    virtual void Update(int force);
};

class GUI_TextField: public GUI_Widget
{
	protected:
		GUI_Font *font;
		SDL_Color textcolor;
		GUI_Surface *background_image;
		GUI_Surface *background_focus_image;
		int background_style;
    int borderx;
    int bordery;
		GUI_Callback *changed_callback;
		int align;
		size_t buffer_size;
		size_t buffer_index;
		char *buffer;
    int cursorpos;
    int startoffset;
    char *validchars;
	public:
		GUI_TextField(const char *name, int x, int y, int w, int h, GUI_Font *font, int maxlength);
		~GUI_TextField(void);

		void SetFont(GUI_Font *afont);
		void SetTextColor(int r, int g, int b);
		void SetText(const char *text);
		const char *GetText(void);
		void SetNormalImage(GUI_Surface *surface);
		void SetFocusImage(GUI_Surface *surface);
    void SetBackgroundStyle(int style);
		void SetChangedCallback(GUI_Callback *callback);
    int GetCursorPos(void);
    void SetCursorPos(int pos);
    void SetBorderX(int pixels);
    void SetBorderY(int pixels);
    const char *GetValidChars(void);
    void SetValidChars(const char *validchars);

		virtual void Update(int force);
		virtual void Clicked(int x, int y);
		virtual int Event(const SDL_Event *event, int xoffset, int yoffset);
    bool SendChar(char c, TextFieldInsertMode insmode);
    void Backspace(void);
    void DeleteCurrChar(void);
};


// Functions (in gui_utils.cpp)
void DrawClipped(GUI_Surface *source, GUI_Drawable *dest, SDL_Rect destarea, int destx, int desty);
unsigned int colortouint(SDL_Color c);
bool pointinrect(int x, int y, SDL_Rect a);
bool rectsintersect(SDL_Rect l, SDL_Rect r);
