/***************************************************************************
       gui_resourcemanager.cpp  -  manager for SDL_gui-based resources
                             -------------------
    begin                : Mon Oct 6 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
 
#include "SDL_gui.h"
#include "gui_resourcemanager.h"
#include "gui_resource.h"
#include "../../basewidgets/utils.h"

using namespace std;

ResourceManager::~ResourceManager(void) {
  UnloadAllResources();
}

ImageResource *ResourceManager::InternalLoadImageResource(string filename, bool force) {
  ImageResource *image = NULL;

  if(filename == "")
    return NULL; // hack to avoid errors

  string fullpath = expandPath(filename);

  for(unsigned int i=0; i<images.size(); i++) {
    if(fullpath == images[i]->GetFilename()) {
      // Image already loaded
      if(!force)
        return images[i];
      else
        image = images[i];
    }
  }

  if(force) {
    if(image == NULL) {
      cerr << "Image " << fullpath << " is not already loaded" << endl;
      return NULL;
    }
  }
  else 
    image = new SDL_guiImageResource(fullpath);

  GUI_Surface *imgsurface = ((GUI_Surface *)image->Load());
  if (imgsurface == NULL) {
    cerr << "Failed to load image " << fullpath << " (file possibly corrupt?)" << endl;
    if(!force)
      delete image;
    return NULL;
  }

  if(!force)
    images.push_back(image);

  return image;
}

FontResource *ResourceManager::InternalLoadFontResource(string filename, int fontsize) {
  FontResource *font = NULL;

  string fullpath = expandPath(filename);

  for(unsigned int i=0; i<fonts.size(); i++) {
    if(fullpath == fonts[i]->GetFilename() && fonts[i]->GetFontSize() == fontsize) {
      // Font already loaded
      return fonts[i];
    }
  }

  font = new SDL_guiFontResource(fullpath, fontsize);

  GUI_Font *realfont = ((GUI_Font *)font->Load());
  if (realfont == NULL) {
    cerr << "Failed to load font " << fullpath << " (file possibly corrupt?)" << endl;
    delete font;
    return NULL;
  }

  fonts.push_back(font);
	return font;
}

void ResourceManager::UnloadAllResources(void) {
  // Clear out images
  while(images.size() > 0) {
    ImageResource *image = images.back();
    image->Unload();
    images.pop_back();
    delete image;
  }
  // Clear out fonts
  while(fonts.size() > 0) {
    FontResource *font = fonts.back();
    font->Unload();
    fonts.pop_back();
    delete font;
  }
}
