/***************************************************************************
                 ifsdl_gui.h  -  SDL_gui interface plugin for eboxy
                             -------------------
    begin                : Sun Jul 13 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INCLUDED__IFSDL_GUI_H
#define INCLUDED__IFSDL_GUI_H

#include <vector>

#include "SDL.h"
#include "SDL_gui.h"

#include "../../basewidgets/interfaceplugin.h"
#include "gui_resourcemanager.h"

#include "page.h"
#include "widget.h"

class SDL_guiInterface: public InterfacePlugin {
  protected:
    int opts;
    bool nokeyrepeat;
    bool hidecursor;
    bool signalquit;
    GUI_Screen *screen;
    GUI_CardStack *cards;
    std::vector<ebPage *> pages;
    ebPage *currentPage;
    ebPage *loadedPage;
    ResourceManager resourcemanager;
    SDL_Joystick *joystick;

    void SendKeyEvent(SDL_Event e);
  public:
    SDL_guiInterface(void);
    virtual ~SDL_guiInterface(void);
    int Initialise(void);
    void BeginEventLoop(void (*loopfunc)(void), bool(*interfaceeventfunc)(InterfaceEvent&));
    void Quit(void);
    void Uninitialise(void);
    void ClearPages(bool clearall);
    Object *CreateObject(const char *type, const char *name);
    void DeleteObject(Object *object);
    int SwitchPage(PageBase *page);
    ResourceManagerBase *GetResourceManager(void);
    PageBase *GetCurrentPage(void);
    PageBase *GetLoadedPage(void);
    PageBase *FindPage(const char *name);
    int GetPageCount(void);
    PageBase *GetPage(unsigned int index);
    DynamicObject *FindExtPageObject(const char *name);
    bool HasFocus(void);
    int GetScreenWidth(void);
    int GetScreenHeight(void);
};

// SDL key to eboxy key code map
const struct {
  eboxykey key;
  SDLKey sdlkey;
} keymap[] = {
  {EKEY_a,              SDLK_a},
  {EKEY_b,              SDLK_b},
  {EKEY_c,              SDLK_c},
  {EKEY_d,              SDLK_d},
  {EKEY_e,              SDLK_e},
  {EKEY_f,              SDLK_f},
  {EKEY_g,              SDLK_g},
  {EKEY_h,              SDLK_h},
  {EKEY_i,              SDLK_i},
  {EKEY_j,              SDLK_j},
  {EKEY_k,              SDLK_k},
  {EKEY_l,              SDLK_l},
  {EKEY_m,              SDLK_m},
  {EKEY_n,              SDLK_n},
  {EKEY_o,              SDLK_o},
  {EKEY_p,              SDLK_p},
  {EKEY_q,              SDLK_q},
  {EKEY_r,              SDLK_r},
  {EKEY_s,              SDLK_s},
  {EKEY_t,              SDLK_t},
  {EKEY_u,              SDLK_u},
  {EKEY_v,              SDLK_v},
  {EKEY_w,              SDLK_w},
  {EKEY_x,              SDLK_x},
  {EKEY_y,              SDLK_y},
  {EKEY_z,              SDLK_z},
  {EKEY_0,              SDLK_0},
  {EKEY_1,              SDLK_1},
  {EKEY_2,              SDLK_2},
  {EKEY_3,              SDLK_3},
  {EKEY_4,              SDLK_4},
  {EKEY_5,              SDLK_5},
  {EKEY_6,              SDLK_6},
  {EKEY_7,              SDLK_7},
  {EKEY_8,              SDLK_8},
  {EKEY_9,              SDLK_9},
  {EKEY_BACKSPACE,      SDLK_BACKSPACE},
  {EKEY_TAB,            SDLK_TAB},
  {EKEY_RETURN,         SDLK_RETURN},
  {EKEY_PAUSE,          SDLK_PAUSE},
  {EKEY_ESCAPE,         SDLK_ESCAPE},
  {EKEY_SPACE,          SDLK_SPACE},
  {EKEY_QUOTE,          SDLK_QUOTE},
  {EKEY_COMMA,          SDLK_COMMA},
  {EKEY_MINUS,          SDLK_MINUS},
  {EKEY_DOT,            SDLK_PERIOD},
  {EKEY_SLASH,          SDLK_SLASH},
  {EKEY_SEMICOLON,      SDLK_SEMICOLON},
  {EKEY_EQUALS,         SDLK_EQUALS},
  {EKEY_LEFTBRACKET,    SDLK_LEFTBRACKET},
  {EKEY_BACKSLASH,      SDLK_BACKSLASH},
  {EKEY_RIGHTBRACKET,   SDLK_RIGHTBRACKET},
  {EKEY_BACKQUOTE,      SDLK_BACKQUOTE},
  {EKEY_F1,             SDLK_F1},
  {EKEY_F2,             SDLK_F2},
  {EKEY_F3,             SDLK_F3},
  {EKEY_F4,             SDLK_F4},
  {EKEY_F5,             SDLK_F5},
  {EKEY_F6,             SDLK_F6},
  {EKEY_F7,             SDLK_F7},
  {EKEY_F8,             SDLK_F8},
  {EKEY_F9,             SDLK_F9},
  {EKEY_F10,            SDLK_F10},
  {EKEY_F11,            SDLK_F11},
  {EKEY_F12,            SDLK_F12},
  {EKEY_F13,            SDLK_F13},
  {EKEY_F14,            SDLK_F14},
  {EKEY_F15,            SDLK_F15},
  {EKEY_KP0,            SDLK_KP0},
  {EKEY_KP1,            SDLK_KP1},
  {EKEY_KP2,            SDLK_KP2},
  {EKEY_KP3,            SDLK_KP3},
  {EKEY_KP4,            SDLK_KP4},
  {EKEY_KP5,            SDLK_KP5},
  {EKEY_KP6,            SDLK_KP6},
  {EKEY_KP7,            SDLK_KP7},
  {EKEY_KP8,            SDLK_KP8},
  {EKEY_KP9,            SDLK_KP9},
  {EKEY_KPSTAR,         SDLK_KP_MULTIPLY},
  {EKEY_KPSLASH,        SDLK_KP_DIVIDE},
  {EKEY_KPDOT,          SDLK_KP_PERIOD},
  {EKEY_KPMINUS,        SDLK_KP_MINUS},
  {EKEY_KPPLUS,         SDLK_KP_PLUS},
  {EKEY_KPENTER,        SDLK_KP_ENTER},
  {EKEY_KPEQUALS,       SDLK_KP_EQUALS},
  {EKEY_UP,             SDLK_UP},
  {EKEY_DOWN,           SDLK_DOWN},
  {EKEY_RIGHT,          SDLK_RIGHT},
  {EKEY_LEFT,           SDLK_LEFT},
  {EKEY_DELETE,         SDLK_DELETE},
  {EKEY_INSERT,         SDLK_INSERT},
  {EKEY_HOME,           SDLK_HOME},
  {EKEY_END,            SDLK_END},
  {EKEY_PAGEUP,         SDLK_PAGEUP},
  {EKEY_PAGEDOWN,       SDLK_PAGEDOWN},
  {EKEY_LSHIFT,         SDLK_LSHIFT},
  {EKEY_RSHIFT,         SDLK_RSHIFT},
  {EKEY_LALT,           SDLK_LALT},
  {EKEY_RALT,           SDLK_RALT},
  {EKEY_LCTRL,          SDLK_LCTRL},
  {EKEY_RCTRL,          SDLK_RCTRL},
  {EKEY_LSUPER,         SDLK_LSUPER},
  {EKEY_RSUPER,         SDLK_RSUPER},
  {EKEY_MENU,           SDLK_MENU},
  {EKEY_PRINTSCREEN,    SDLK_PRINT},
  {EKEY_NUMLOCK,        SDLK_NUMLOCK},
  {EKEY_CAPSLOCK,       SDLK_CAPSLOCK},
  {EKEY_SCROLLLOCK,     SDLK_SCROLLOCK},
  {EKEY_RMETA,          SDLK_RMETA},
  {EKEY_LMETA,          SDLK_LMETA},
  {EKEY_MODE,           SDLK_MODE},
  {EKEY_HELP,           SDLK_HELP},
  {EKEY_SYSREQ,         SDLK_SYSREQ},
  {EKEY_BREAK,          SDLK_BREAK},
  {EKEY_POWER,          SDLK_POWER},
  {EKEY_CLEAR,          SDLK_CLEAR},
  {EKEY_EXCLAIM,        SDLK_EXCLAIM},
  {EKEY_QUOTEDBL,       SDLK_QUOTEDBL},
  {EKEY_HASH,           SDLK_HASH},
  {EKEY_DOLLAR,         SDLK_DOLLAR},
  {EKEY_AMPERSAND,      SDLK_AMPERSAND},
  {EKEY_LEFTPAREN,      SDLK_LEFTPAREN},
  {EKEY_RIGHTPAREN,     SDLK_RIGHTPAREN},
  {EKEY_ASTERISK,       SDLK_ASTERISK},
  {EKEY_PLUS,           SDLK_PLUS},
  {EKEY_LESS,           SDLK_LESS},
  {EKEY_GREATER,        SDLK_GREATER},
  {EKEY_QUESTION,       SDLK_QUESTION},
  {EKEY_AT,             SDLK_AT},
  {EKEY_CARET,          SDLK_CARET},
  {EKEY_UNDERSCORE,     SDLK_UNDERSCORE},
  {EKEY_NONE, SDLK_BACKSPACE}  // end of list
};

// SDL modifier key to eboxy code map
const struct {
  SDLMod sdlmod;
  int modcode;
} modmap[] = {
  {KMOD_LSHIFT, MODKEY_LSHIFT},
  {KMOD_RSHIFT, MODKEY_RSHIFT},
  {KMOD_LCTRL,  MODKEY_LCTRL},
  {KMOD_RCTRL,  MODKEY_RCTRL},
  {KMOD_LALT,   MODKEY_LALT},
  {KMOD_RALT,   MODKEY_RALT},
  {KMOD_CAPS,   MODKEY_CAPSLOCK},
  {KMOD_NUM,    MODKEY_NUMLOCK},
  {KMOD_NONE,   MODKEY_NONE}
};

int transmod(SDLMod sdlmod);
eboxykey transkey(SDLKey sdlkey);

#endif
