/***************************************************************************
                 labelwidget.cpp  -  eboxy static label widget
                             -------------------
    begin                : Sat Aug 3 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "SDL_gui.h"
#include "gui_extensions.h"

#include "../../basewidgets/utils.h"

#include "labelwidget.h"

using namespace std;

LabelWidget::LabelWidget(const char *name, int x, int y, int w, int h, std::string text, FontResource *font)
: LabelWidgetBase(name, x, y, w, h, text, font), WidgetBase(x, y), Object(name) {

}

LabelWidget::~LabelWidget(void) {

}

GUI_Widget *LabelWidget::Instantiate(void) {
  GUI_ExtLabel *label = new GUI_ExtLabel(name, x, y, w, h, (GUI_Font *)(font->GetHandle()), text.c_str());
  label->SetTextColor((int)fontcolor.r, (int)fontcolor.g, (int)fontcolor.b);
  label->SetWordWrap(wordwrap);
  label->SetAutoSize(autosize);
  label->SetAlign(textaligntoalign(textalign));
  label->SetFlags(WIDGET_TRANSPARENT);
  realwidget = label;
  realwidget->MarkChanged(); // fixes bug with SDL_gui (doesn't draw if changed then added)
  return label;
}

void LabelWidget::SetText(std::string text) {
  LabelWidgetBase::SetText(text);
  if(realwidget) {
    GUI_Lock();
    SDL_Rect a = GetArea();
    realwidget->GetParent()->Erase(&a);
    ((GUI_ExtLabel *)realwidget)->SetText(this->text.c_str());
    this->InvalidateArea();
    GUI_Unlock();
  }
}

void LabelWidget::SetFont(FontResource *font) {
  LabelWidgetBase::SetFont(font);
  if(realwidget)
    ((GUI_ExtLabel *)realwidget)->SetFont((GUI_Font *)(this->font->GetHandle()));
}

void LabelWidget::SetFontColor(eboxy_color c) {
  LabelWidgetBase::SetFontColor(c);
  if(realwidget) {
    ((GUI_ExtLabel *)realwidget)->SetTextColor(c.r, c.g, c.b);
    realwidget->MarkChanged();
  }
}

void LabelWidget::SetWordWrap(bool wordwrap) {
  LabelWidgetBase::SetWordWrap(wordwrap);
  if(realwidget)
    ((GUI_ExtLabel *)realwidget)->SetWordWrap(wordwrap);
}

void LabelWidget::SetAutoSize(bool autosize) {
  LabelWidgetBase::SetAutoSize(autosize);
  if(realwidget)
    ((GUI_ExtLabel *)realwidget)->SetAutoSize(this->autosize);
}

void LabelWidget::SetAlign(enum_textalign textalign) {
  LabelWidgetBase::SetAlign(textalign);
  if(realwidget)
    ((GUI_ExtLabel *)realwidget)->SetAlign(textaligntoalign(this->textalign));
}

bool LabelWidget::IsInstantiated(void) {
  return (realwidget != NULL);
}

bool LabelWidget::IsGUIFocused(void) {
  return ebWidget::IsGUIFocused();
}

bool LabelWidget::IsEnabled(void) {
  return ebWidget::IsEnabled();
}


int textaligntoalign(enum_textalign textalign) {
  switch(textalign) {
    case TEXTALIGN_CENTER:
      return WIDGET_VERT_TOP | WIDGET_HORIZ_CENTER;
    case TEXTALIGN_RIGHT:
      return WIDGET_VERT_TOP | WIDGET_HORIZ_RIGHT;
    default:
      return WIDGET_VERT_TOP | WIDGET_HORIZ_LEFT;
  }
}
