/***************************************************************************
                          listboxwidget.cpp  -  eboxy listbox widget
                             -------------------
    begin                : Thu Nov 7 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include "SDL_gui.h"

#include "../../basewidgets/utils.h"
#include "../../errors.h"
#include "gui_extensions.h"

#include "listboxwidget.h"

using namespace std;

ListBoxWidget::ListBoxWidget(const char *name, int x, int y, int w, int h, FontResource *font, eboxy_color fontcolor)
: ListBoxWidgetBase(name, x, y, w, h, font, fontcolor), WidgetBase(x, y), Object(name) {

}

GUI_Widget *ListBoxWidget::Instantiate(void) {
  if(w<0) w=0;
  if(h<0) h=0;

  GUI_ListBox *listbox = new GUI_ListBox(name, x, y, w, h, (GUI_Font *)(font->GetHandle()), font->GetFontSize());
  listbox->SetTextColor((int)fontcolor.r, (int)fontcolor.g, (int)fontcolor.b);
  listbox->SetSelectedItemTextColor((int)selitemfontcolor.r, (int)selitemfontcolor.g, (int)selitemfontcolor.b);
  listbox->SetScrollBarWidth(scrollbarwidth);
  listbox->SetBorderX(borderx);
  listbox->SetBorderY(bordery);
  listbox->SetItemBorderX(itemborderx);
  listbox->SetItemBorderY(itembordery);
  listbox->SetItemHeight(itemheight);

  if(backgroundimage)
    listbox->SetBackgroundImage((GUI_Surface *)(backgroundimage->GetHandle()));
  if(backgroundfocusimage)
    listbox->SetBackgroundFocusImage((GUI_Surface *)(backgroundfocusimage->GetHandle()));
  listbox->SetBackgroundStyle(backgroundstyle);
  listbox->SetScrollBarMode(scrollbarmode);
  if(selitembackgroundimage)
    listbox->SetSelectedItemImage((GUI_Surface *)(selitembackgroundimage->GetHandle()));
  if(selitembackgroundfocusimage)
    listbox->SetSelectedItemFocusImage((GUI_Surface *)(selitembackgroundfocusimage->GetHandle()));
  listbox->SetSelectedItemStyle(selitembackgroundstyle);
  if(scrollbarbackgroundimage)
    listbox->SetScrollBarBackgroundImage((GUI_Surface *)(scrollbarbackgroundimage->GetHandle()));
  listbox->SetScrollBarBackgroundStyle(scrollbarbackgroundstyle);
  if(scrollknobimage)
    listbox->SetScrollBarKnobImage((GUI_Surface *)(scrollknobimage->GetHandle()));

  for(unsigned int i=0; i<listitems.size(); i++) {
    listbox->AddItem(listitems[i].c_str());
  }

  listbox->SetSelectedIndex(selectedindex);

  GUI_EventHandler<ListBoxWidget> *choosecb = new GUI_EventHandler<ListBoxWidget>(this, &ListBoxWidget::OnChoose);
  listbox->SetChooseCallback(choosecb);
	choosecb->DecRef();

  GUI_EventHandler<ListBoxWidget> *selcb = new GUI_EventHandler<ListBoxWidget>(this, &ListBoxWidget::OnSelectionChange);
  listbox->SetSelectionChangeCallback(selcb);
	selcb->DecRef();

  realwidget = listbox;
  return listbox;
}

int ListBoxWidget::Clone(ebWidget *newwidget) {
  int rc = ebWidget::Clone(newwidget);
  if(rc == EBERR_SUCCESS) {
    // Copy list items across
    for(unsigned int i=0; i<listitems.size(); i++) {
      ((ListBoxWidget *)newwidget)->AddItem(listitems[i]);
    }
  }
  return rc;
}

void ListBoxWidget::SetBackgroundImage(ImageResource *image) {
  this->backgroundimage = image;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetBackgroundImage((GUI_Surface *)(image->GetHandle()));
}

void ListBoxWidget::SetBackgroundFocusImage(ImageResource *image) {
  this->backgroundfocusimage = image;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetBackgroundFocusImage((GUI_Surface *)(image->GetHandle()));
}

void ListBoxWidget::SetBackgroundStyle(int style) {
  if(style == IMAGESTYLE_NORMAL || style == IMAGESTYLE_TILED) {
    backgroundstyle = style;
    if(realwidget)
      ((GUI_ListBox *)realwidget)->SetBackgroundStyle(style);
  }
}

void ListBoxWidget::SetSelectedItemImage(ImageResource *image) {
  this->selitembackgroundimage = image;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetSelectedItemImage((GUI_Surface *)(image->GetHandle()));
}

void ListBoxWidget::SetSelectedItemFocusImage(ImageResource *image) {
  this->selitembackgroundfocusimage = image;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetSelectedItemFocusImage((GUI_Surface *)(image->GetHandle()));
}

void ListBoxWidget::SetScrollBarMode(int scrollbarmode) {
  this->scrollbarmode = scrollbarmode;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetScrollBarMode(scrollbarmode);
}

void ListBoxWidget::SetSelectedItemStyle(int style) {
  if(style == IMAGESTYLE_NORMAL || style == IMAGESTYLE_TILED) {
    selitembackgroundstyle = style;
    if(realwidget)
      ((GUI_ListBox *)realwidget)->SetSelectedItemStyle(style);
  }
}

void ListBoxWidget::SetScrollBarBackgroundImage(ImageResource *image) {
  this->scrollbarbackgroundimage = image;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetScrollBarBackgroundImage((GUI_Surface *)(image->GetHandle()));
}

void ListBoxWidget::SetScrollBarBackgroundStyle(int style) {
  if(style == IMAGESTYLE_NORMAL || style == IMAGESTYLE_TILED) {
    scrollbarbackgroundstyle = style;
    if(realwidget)
      ((GUI_ListBox *)realwidget)->SetScrollBarBackgroundStyle(style);
  }
}

void ListBoxWidget::SetScrollBarKnobImage(ImageResource *image) {
  this->scrollknobimage = image;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetScrollBarKnobImage((GUI_Surface *)(image->GetHandle()));
}

void ListBoxWidget::SetFont(FontResource *font) {
  this->font = font;
  if(realwidget) {
    ((GUI_ListBox *)realwidget)->SetFont((GUI_Font *)(font->GetHandle()));
    realwidget->MarkChanged();
  }
}

void ListBoxWidget::SetFontColor(eboxy_color c) {
  this->fontcolor = c;
  if(realwidget) {
    ((GUI_ListBox *)realwidget)->SetTextColor(c.r, c.g, c.b);
    realwidget->MarkChanged();
  }
}

void ListBoxWidget::SetSelectedItemFontColor(eboxy_color c) {
  this->selitemfontcolor = c;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetSelectedItemTextColor(c.r, c.g, c.b);
}

void ListBoxWidget::SetScrollBarWidth(int width) {
  this->scrollbarwidth = width;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetScrollBarWidth(width);
}

void ListBoxWidget::SetBorderX(int pixels) {
  this->borderx = pixels;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetBorderX(pixels);
}

void ListBoxWidget::SetBorderY(int pixels) {
  this->bordery = pixels;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetBorderY(pixels);
}

void ListBoxWidget::SetItemBorderX(int pixels) {
  this->itemborderx = pixels;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetItemBorderX(pixels);
}

void ListBoxWidget::SetItemBorderY(int pixels) {
  this->itembordery = pixels;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetItemBorderY(pixels);
}

void ListBoxWidget::SetItemHeight(int pixels) {
  this->itemheight = pixels;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetItemHeight(pixels);
}

void ListBoxWidget::SetSelectedIndex(int index) {
  if(index >= 0 && index < listitems.size()) {
    selectedindex = index;
    if(realwidget)
      ((GUI_ListBox *)realwidget)->SetSelectedIndex(index);
  }
}

void ListBoxWidget::SelectNext(int count) {
  if(selectedindex < listitems.size() - 1 && listitems.size() > 0) {
    // Select the next item
    if(selectedindex + count > listitems.size() - 1)
      SetSelectedIndex(listitems.size() - 1);
    else
      SetSelectedIndex(selectedindex + count);
  }
}

void ListBoxWidget::SelectNextPage(void) {
  SelectNext(((GUI_ListBox *)realwidget)->GetVisibleItems());
}

void ListBoxWidget::SelectPrevious(int count) {
  if(selectedindex > 0) {
    // Select the previous item
    if(selectedindex - count < 0)
      SetSelectedIndex(0);
    else
      SetSelectedIndex(selectedindex - count);
  }
}

void ListBoxWidget::SelectPreviousPage(void) {
  SelectPrevious(((GUI_ListBox *)realwidget)->GetVisibleItems());
}

int ListBoxWidget::GetSelectedIndex(void) {
  return selectedindex;
}

int ListBoxWidget::GetItemCount(void) {
  return listitems.size();
}

void ListBoxWidget::AddItem(std::string text) {
  listitems.push_back(text);
  if(realwidget)
    ((GUI_ListBox *)realwidget)->AddItem(text.c_str());
}

void ListBoxWidget::RemoveItem(int index) {
  if(index >= 0 && index < listitems.size()) {
    vector<string>::iterator iter = listitems.begin() + index;
    listitems.erase(iter);
    if(realwidget)
      ((GUI_ListBox *)realwidget)->RemoveItem(index);
    if(listitems.size() == 0)
      FireEvent("OnEmpty");
  }
  else
    cerr << "RemoveItem: Listbox " << name << " has no item with index " << index << endl;
}

void ListBoxWidget::Clear(void) {
  int numitems = listitems.size();

  while(listitems.size() > 0)
    listitems.pop_back();

  if(realwidget)
    ((GUI_ListBox *)realwidget)->Clear();

  if(numitems>0)
    FireEvent("OnEmpty");
}

void ListBoxWidget::SetItemText(int index, std::string text) {
  if(index >= 0 && index < listitems.size()) {
    listitems[index] = text;
    if(realwidget)
      ((GUI_ListBox *)realwidget)->SetItemText(index, text.c_str());
  }
  else
    cerr << "SetItemText: Listbox " << name << " has no item with index " << index << endl;
}

void ListBoxWidget::ChooseSelection(void) {
  if(selectedindex >= 0 && listitems.size() > 0)
    FireEvent("OnChoose");
}

void ListBoxWidget::OnChoose(GUI_Object *sender) {
  // Fired from the real listbox widget
  ChooseSelection();
}

void ListBoxWidget::OnSelectionChange(GUI_Object *sender) {
  // Fired from the real listbox widget
  if(realwidget)
    selectedindex = ((GUI_ListBox *)realwidget)->GetSelectedIndex();
  FireEvent("OnSelectionChange");
}

void ListBoxWidget::GotFocus(void) {
  if(!focused) {
    this->FireEvent("OnGotFocus");
    ebWidget::GotFocus();
    ((GUI_ListBox *)realwidget)->MarkChanged();
  }
}

void ListBoxWidget::LostFocus(void) {
  if(focused) {
    this->FireEvent("OnLostFocus");
    ebWidget::LostFocus();
    ((GUI_ListBox *)realwidget)->MarkChanged();
  }
}

bool ListBoxWidget::IsInstantiated(void) {
  return (realwidget != NULL);
}

bool ListBoxWidget::IsGUIFocused(void) {
  return ebWidget::IsGUIFocused();
}

bool ListBoxWidget::IsEnabled(void) {
  return ebWidget::IsEnabled();
}
