/***************************************************************************
                          page.cpp  -  eboxy page (not really a widget)
                             -------------------
    begin                : Sat Aug 3 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>

#include "gui_extensions.h"
#include "../../basewidgets/utils.h"
#include "../../basewidgets/initialisebase.h"

#include "page.h"

using namespace std;

extern const char *TYPESTR_PAGE;
extern const char *TYPESTR_TIMER;
extern const char *TYPESTR_PLUGIN;

ebPage::ebPage(const char *name, int x, int y, int w, int h)
: PageBase(name, x, y, w, h), WidgetBase(x, y), Object(name) {
  //
}

ebPage::~ebPage(void) {
  if(realwidget)
    Uninstantiate();
}

GUI_Widget *ebPage::Instantiate(void) {
  // Initialise plugins
  for(unsigned int i=0;i<containedobjects.size();i++) {
    DynamicObject *object = containedobjects[i];

    // FIXME: need to handle errors here!
    if(!strcmp(object->GetType(), TYPESTR_PLUGIN)) {
      (dynamic_cast<InitialiseBase *>(object))->DoInitialise();
    }
  }

  this->FireEventWait("OnLoad");

  GUI_Panel *panel = new GUI_Panel(GetName(), x, y, w, h);

  if(background)
    panel->SetBackground((GUI_Surface *)(background->GetHandle()));

  for(unsigned int i=0;i<widgets.size();i++) {
    // create the widget and add it to the page
    ebWidget *widget = dynamic_cast<ebWidget *>(widgets[i]);
    if(widget->GetVisible())
      panel->AddWidget(widget->Instantiate());
  }

  DoInstantiate();
  
  realwidget = panel;

  // We can't fire the OnShow event yet because CurrentPage has not been changed
  // (this is done in OnShow() below, which is called by switchPage)
  return panel;
}

void ebPage::Uninstantiate(void) {
  if(realwidget) {
    // Unfocus currently focused widget (FIXME - need lock?)
    if(focusedindex > -1)
      widgets[focusedindex]->LostFocus();
    
    this->FireEventWait("OnUnload");
    DoUninstantiate();
    
    // Remove widgets from the page
    for(unsigned int i=0;i<widgets.size();i++) {
      WidgetBase *widget = widgets[i];
      ebWidget *sgwidget = dynamic_cast<ebWidget *>(widget);

      if(widgets[i]->IsInstantiated())
        realwidget->RemoveWidget(sgwidget->GetInstance());
      widget->Uninstantiate();
    }
    // Deal with any owned objects
    for(unsigned int i=0;i<containedobjects.size();i++) {
      Object *object = containedobjects[i];
      if(!strcmp(object->GetType(), TYPESTR_TIMER)) {
        (dynamic_cast<InitialiseBase *>(object))->DoUninitialise();
      }
      else if(!strcmp(object->GetType(), TYPESTR_PLUGIN)) {
        (dynamic_cast<InitialiseBase *>(object))->DoUninitialise();
      }
    }
    ebWidget::Uninstantiate();
  }
}

void ebPage::UpdateFocus(void) {
  int newfocus = -1;

  if(widgets.size() == 0) {
    // No widgets
    focusedindex = -1;
    return;
  }

  if(realwidget->GetFlags() & WIDGET_PRESSED)  // makes sure we don't get silly stuff happening when the mouse is down
    return;

  // Make sure there is a focused widget
  if(focusedindex < 0 || !((widgets[focusedindex]->GetTabStop()) && widgets[focusedindex]->IsEnabled()))
    FocusNextWidget();
  else {
    for(unsigned int i=0; i<widgets.size(); i++) {
      if(widgets[i]->IsGUIFocused() && ((widgets[i]->GetTabStop()) && widgets[i]->IsEnabled()) ) {
        newfocus = i;
        break;
      }
    }
    if(newfocus < 0) {
      // SDL_gui - mouse has taken away focus
      widgets[focusedindex]->GotFocus();
    }
    else if(focusedindex != newfocus) {
      widgets[focusedindex]->LostFocus();
      focusedindex = newfocus;
      widgets[focusedindex]->GotFocus();
    }
  }
}

void ebPage::SetBackground(ImageResource *image) {
  PageBase::SetBackground(image);
  if(realwidget) {
    if(background)
      ((GUI_Panel *)realwidget)->SetBackground((GUI_Surface *)(background->GetHandle()));
    else
      ((GUI_Panel *)realwidget)->SetBackground(NULL);
    realwidget->MarkChanged();
  }
}

void ebPage::AddWidget(WidgetBase *widget) {
  for(unsigned int i=0;i<widgets.size();i++) {
    if(widgets[i] == widget) {
      cerr << "AddWidget: widget " << widget->GetName() << " is already added to the page " << GetName() << endl;
      return;
    }
  }

  // Add widget to widgets array
  widgets.push_back(widget);
  widget->SetParent(this);

  if(realwidget) {
    // Adding at runtime
    GUI_Widget *rwidget = dynamic_cast<ebWidget *>(widget)->Instantiate();
    if(rwidget)
      ((GUI_Panel *)realwidget)->AddWidget(rwidget);
    else
      cerr << "AddWidget: widget " << widget->GetName() << " could not be instantiated" << endl;
  }
}

void ebPage::RemoveWidget(WidgetBase *widget) {
  for(unsigned int i=0;i<widgets.size();i++) {
    if(widgets[i] == widget) {
      if(realwidget) {
        // Removing at runtime
        ebWidget *sgwidget = dynamic_cast<ebWidget *>(widget);
        
        ((GUI_Panel *)realwidget)->RemoveWidget(sgwidget->GetInstance());
        widget->Uninstantiate();
      }

      vector<WidgetBase *>::iterator p = widgets.begin() + i;
      widgets.erase(p);

      realwidget->MarkChanged();
      break;
    }
  }
}

void ebPage::InvalidateRect(SDL_Rect a) {
  // Mark all the widgets in the specified area as changed so that they will be redrawn
  // FIXME: This is not much use right now, because it can get into an endless loop and
  //   stuff isn't done in the correct order. It requires substantial changes to the way
  //   SDL_gui works in order to fix it, though.
  if(realwidget) {
    for(unsigned int i=0; i<widgets.size(); i++) {
      ebWidget *sgwidget = dynamic_cast<ebWidget *>(widgets[i]);
      if(rectsintersect(a, sgwidget->GetArea())) {
        // Widget is inside specified area, invalidate it
        sgwidget->Invalidate();
      }
    }
  }
}

void ebPage::SetVisible(bool visible) {
  // Do nothing!
}

bool ebPage::IsInstantiated(void) {
  return (realwidget != NULL);
}

bool ebPage::IsGUIFocused(void) {
  return ebWidget::IsGUIFocused();
};

bool ebPage::IsEnabled(void) {
  return ebWidget::IsEnabled();
}
