/***************************************************************************
                          scrollbarwidget.cpp  -  eboxy scrollbar widget
                             -------------------
    begin                : Sun Dec 1 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>

#include "gui_extensions.h"
#include "../../basewidgets/utils.h"

#include "scrollbarwidget.h"

using namespace std;

ScrollBarWidget::ScrollBarWidget(const char *name, int x, int y, int w, int h)
: ScrollBarWidgetBase(name, x, y, w, h), WidgetBase(x, y), Object(name) {

}

GUI_Widget *ScrollBarWidget::Instantiate(void) {
  if(w<0) w=0;
  if(h<0) h=0;

  GUI_ExtScrollBar *scrollbar = new GUI_ExtScrollBar(name, x, y, w, h);

  if(this->background)
    scrollbar->SetBackgroundImage((GUI_Surface *)(background->GetHandle()));
  scrollbar->SetBackgroundStyle(backgroundstyle);
  if(this->knob)
    scrollbar->SetKnobImage((GUI_Surface *)(knob->GetHandle()));
  if(this->knobfocus)
    scrollbar->SetKnobFocusImage((GUI_Surface *)(knobfocus->GetHandle()));
  scrollbar->SetMaximum(maximum);
  scrollbar->SetMinimum(minimum);
  scrollbar->SetValue(value);
  scrollbar->SetPageStep(pagestep);
  scrollbar->SetReversed(reversed);

  GUI_EventHandler<ScrollBarWidget> *cb = new GUI_EventHandler<ScrollBarWidget>(this, &ScrollBarWidget::OnChange);
  scrollbar->SetMovedCallback(cb);
	cb->DecRef();

  realwidget = scrollbar;
  return scrollbar;
}

void ScrollBarWidget::SetBackgroundImage(ImageResource *image) {
  this->background = image;
}

void ScrollBarWidget::SetBackgroundStyle(int style) {
  this->backgroundstyle = style;
}

void ScrollBarWidget::SetKnobImage(ImageResource *image) {
  this->knob = image;
}

void ScrollBarWidget::SetKnobFocusImage(ImageResource *image) {
  this->knobfocus = image;
}

void ScrollBarWidget::SetValue(int value) {
  int oldvalue = this->value;

  if(value >= minimum && value <= maximum) {
    if(realwidget) {
      ((GUI_ExtScrollBar *)realwidget)->SetValue(value);
    }
    this->value = value;
    if(value != oldvalue)
      FireEvent("OnChange");
  }
}

void ScrollBarWidget::SetMaximum(int maximum) {
  if(maximum > minimum) {
    if(maximum < value) {
      SetValue(maximum);
    }
    this->maximum = maximum;
    if(realwidget)
      ((GUI_ExtScrollBar *)realwidget)->SetMaximum(maximum);
  }
}

void ScrollBarWidget::SetMinimum(int minimum) {
  if(minimum < maximum) {
    if(minimum > value) {
      SetValue(minimum);
    }
    this->minimum = minimum;
    if(realwidget)
      ((GUI_ExtScrollBar *)realwidget)->SetMinimum(minimum);
  }
}

void ScrollBarWidget::SetPageStep(int pagestep) {
  ScrollBarWidgetBase::SetPageStep(pagestep);
  if(realwidget)
    ((GUI_ExtScrollBar *)realwidget)->SetPageStep(this->pagestep);
}

void ScrollBarWidget::SetReversed(bool reversed) {
  ScrollBarWidgetBase::SetReversed(reversed);
  if(realwidget)
    ((GUI_ExtScrollBar *)realwidget)->SetReversed(this->reversed);
}

void ScrollBarWidget::GotFocus(void) {
  if(!focused) {
    this->FireEvent("OnGotFocus");
    ebWidget::GotFocus();
    ((GUI_ExtScrollBar *)realwidget)->MarkChanged();
  }
}

void ScrollBarWidget::LostFocus(void) {
  if(focused) {
    this->FireEvent("OnLostFocus");
    ebWidget::LostFocus();
    ((GUI_ExtScrollBar *)realwidget)->MarkChanged();
  }
}

void ScrollBarWidget::OnChange(GUI_Object *sender) {
  int currvalue = ((GUI_ExtScrollBar *)realwidget)->GetValue();

  if(currvalue != value) {
    value = currvalue;
    FireEvent("OnChange");
  }
}

bool ScrollBarWidget::IsInstantiated(void) {
  return (realwidget != NULL);
}

bool ScrollBarWidget::IsGUIFocused(void) {
  return ebWidget::IsGUIFocused();
}

bool ScrollBarWidget::IsEnabled(void) {
  return ebWidget::IsEnabled();
}
