/***************************************************************************
                          textfieldwidget.cpp  -  eboxy textfield widget
                             -------------------
    begin                : Sun Dec 8 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>

#include "gui_extensions.h"
#include "../../basewidgets/utils.h"
#include "../../basewidgets/variant.h"

#include "textfieldwidget.h"

using namespace std;

TextFieldWidget::TextFieldWidget(const char *name, int x, int y, int w, int h, FontResource *font, eboxy_color fontcolor, int maxlength)
: TextFieldWidgetBase(name, x, y, w, h, font, fontcolor, maxlength), WidgetBase(x, y), Object(name) {
}

TextFieldWidget::~TextFieldWidget(void) {

}

GUI_Widget *TextFieldWidget::Instantiate(void) {
  if(w<0) w=0;
  if(h<0) h=0;

  GUI_TextField *textentry = new GUI_TextField(name, x, y, w, h, (GUI_Font *)(font->GetHandle()), maxlength);

  if(background)
    textentry->SetNormalImage((GUI_Surface *)(background->GetHandle()));
  if(backgroundfocus)
    textentry->SetFocusImage((GUI_Surface *)(backgroundfocus->GetHandle()));
  textentry->SetBackgroundStyle(backgroundstyle);
  textentry->SetBorderX(borderx);
  textentry->SetBorderY(bordery);
  textentry->SetTextColor(fontcolor.r, fontcolor.g, fontcolor.b);
  textentry->SetText(text.c_str());
  textentry->SetValidChars(validchars.c_str());

  GUI_EventHandler<TextFieldWidget> *cb = new GUI_EventHandler<TextFieldWidget>(this, &TextFieldWidget::OnChange);
  textentry->SetChangedCallback(cb);
	cb->DecRef();

  inpdigit = -1;

  realwidget = textentry;
  return textentry;
}

void TextFieldWidget::Uninstantiate(void) {
  ebWidget::Uninstantiate();
}

void TextFieldWidget::SetBackgroundImage(ImageResource *image) {
  this->background = image;
}

void TextFieldWidget::SetBackgroundFocusImage(ImageResource *image) {
  this->backgroundfocus = image;
}

void TextFieldWidget::SetBackgroundStyle(int style) {
  this->backgroundstyle = style;
}

void TextFieldWidget::SetFont(FontResource *font) {
  this->font = font;
  if(realwidget) {
    ((GUI_TextField *)realwidget)->SetFont((GUI_Font *)(this->font->GetHandle()));
    realwidget->MarkChanged();
  }
}

void TextFieldWidget::SetFontColor(eboxy_color c) {
  this->fontcolor = c;
  if(realwidget) {
    ((GUI_TextField *)realwidget)->SetTextColor(c.r, c.g, c.b);
    realwidget->MarkChanged();
  }
}

void TextFieldWidget::SetBorderX(int pixels) {
  this->borderx = pixels;
  if(realwidget)
    ((GUI_TextField *)realwidget)->SetBorderX(pixels);
}

void TextFieldWidget::SetBorderY(int pixels) {
  this->bordery = pixels;
  if(realwidget)
    ((GUI_TextField *)realwidget)->SetBorderY(pixels);
}

void TextFieldWidget::SendEvent(SDL_Event e) {
  if(realwidget)
    ((GUI_TextField *)realwidget)->Event(&e, 0, 0);
}

bool TextFieldWidget::SendChar(char c, TextFieldInsertMode insmode) {
  if(realwidget)
    return ((GUI_TextField *)realwidget)->SendChar(c, insmode);
  else
    return false;
}

void TextFieldWidget::Backspace(void) {
  if(realwidget)
    ((GUI_TextField *)realwidget)->Backspace();
}

void TextFieldWidget::DeleteCurrChar(void) {
  if(realwidget)
    ((GUI_TextField *)realwidget)->DeleteCurrChar();
}

void TextFieldWidget::SetText(std::string str) {
  TextFieldWidgetBase::SetText(str);
  if(realwidget) {
    ((GUI_TextField *)realwidget)->SetText(str.c_str());
    SetCursorPos(str.length());
  }
}

int TextFieldWidget::GetCursorPos(void) {
  if(realwidget) {
    return ((GUI_TextField *)realwidget)->GetCursorPos();
  }
  return 0;
}

void TextFieldWidget::SetCursorPos(int pos) {
  if(realwidget) {
    ((GUI_TextField *)realwidget)->SetCursorPos(pos);
  }
}

void TextFieldWidget::SetValidChars(std::string validchars) {
  this->validchars = validchars;
  if(realwidget)
    ((GUI_TextField *)realwidget)->SetValidChars(this->validchars.c_str());
}

void TextFieldWidget::GotFocus(void) {
  if(!focused) {
    this->FireEvent("OnGotFocus");
    ebWidget::GotFocus();
    ((GUI_TextField *)realwidget)->MarkChanged();
  }
}

void TextFieldWidget::LostFocus(void) {
  if(focused) {
    this->FireEvent("OnLostFocus");
    ebWidget::LostFocus();
    ((GUI_TextField *)realwidget)->MarkChanged();
  }
}

void TextFieldWidget::OnChange(GUI_Object *sender) {
  text = ((GUI_TextField *)realwidget)->GetText();
  FireEvent("OnChange");
}

bool TextFieldWidget::IsInstantiated(void) {
  return (realwidget != NULL);
}

bool TextFieldWidget::IsGUIFocused(void) {
  return ebWidget::IsGUIFocused();
}

bool TextFieldWidget::IsEnabled(void) {
  return ebWidget::IsEnabled();
}
