/***************************************************************************
                       togglebuttonwidget.cpp  -  toggle button widget
                             -------------------
    begin                : Thu Feb 6 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>

#include "togglebuttonwidget.h"
#include "../../basewidgets/actionmanager.h"
#include "../../errors.h"
#include "page.h"

using namespace std;

extern ActionManager *actionmanager;

// FIXME: this code is largely duplicated from ButtonWidget. This needs to be refactored.

ToggleButtonWidget::ToggleButtonWidget(const char *name, int x, int y, int w, int h)
: ToggleButtonWidgetBase(name, x, y, w, h), WidgetBase(x, y), Object(name) {
  //
}

ToggleButtonWidget::~ToggleButtonWidget(void) {
  //
}

GUI_Widget *ToggleButtonWidget::Instantiate(void) {
  GUI_ToggleButton *button = new GUI_ToggleButton(name, x, y, w, h);

  if(normalimage)
    button->SetOffNormalImage((GUI_Surface *)(normalimage->GetHandle()));
  if(normalfocusimage)
    button->SetOffHighlightImage((GUI_Surface *)(normalfocusimage->GetHandle()));
  if(checkedimage)
    button->SetOnNormalImage((GUI_Surface *)(checkedimage->GetHandle()));
  if(checkedfocusimage)
    button->SetOnHighlightImage((GUI_Surface *)(checkedfocusimage->GetHandle()));

  if(!enabled)
    button->SetFlags(WIDGET_DISABLED);
  else
    button->ClearFlags(WIDGET_DISABLED);

  if(checked)
    button->SetFlags(WIDGET_TURNED_ON);

  GUI_EventHandler<ToggleButtonWidget> *cb = new GUI_EventHandler<ToggleButtonWidget>(this, &ToggleButtonWidget::OnClick);
  button->SetClick(cb);
	cb->DecRef();

  if(action.length() > 0) {
    int rc = actionmanager->RegisterLocalAction(action, this);
    if(rc == EBERR_ACTION_INVALID) {
      cerr << "Warning: action \"" << action << "\" is not a valid user action" << endl;
    }
    else if(rc == EBERR_ACTION_ASSIGNED) {
      cerr << "Warning: action \"" << action << "\" is assigned to more than one widget in this context" << endl;
    }
  }

  realwidget = button;
  return button;
}

void ToggleButtonWidget::Uninstantiate(void) {
  if(realwidget) {
    focused = false;

    if(action.length() > 0)
      actionmanager->UnregisterLocalAction(action, this);
  }
  ebWidget::Uninstantiate();
}

void ToggleButtonWidget::SetWidth(int value) {
  if(value == -1 && normalimage != NULL)
    value = ((GUI_Surface *)(normalimage->GetHandle()))->GetWidth();
  WidgetBase::SetWidth(value);
}

void ToggleButtonWidget::SetHeight(int value) {
  if(value == -1 && normalimage != NULL)
    value = ((GUI_Surface *)(normalimage->GetHandle()))->GetHeight();
  WidgetBase::SetHeight(value);
}

void ToggleButtonWidget::SetNormalImage(ImageResource *image) {
  if(image) {
    this->normalimage = image;
    if(realwidget)
      ((GUI_ToggleButton *)realwidget)->SetOffNormalImage((GUI_Surface *)(image->GetHandle()));
    if(this->w==-1)
      SetWidth(-1);
    if(this->h==-1)
      SetHeight(-1);
  }
}

void ToggleButtonWidget::SetNormalFocusImage(ImageResource *image) {
  if(image) {
    this->normalfocusimage = image;
    if(realwidget)
      ((GUI_ToggleButton *)realwidget)->SetOffHighlightImage((GUI_Surface *)(image->GetHandle()));
  }
}

void ToggleButtonWidget::SetCheckedImage(ImageResource *image) {
  if(image) {
    this->checkedimage = image;
    if(realwidget)
      ((GUI_ToggleButton *)realwidget)->SetOnNormalImage((GUI_Surface *)(image->GetHandle()));
  }
}

void ToggleButtonWidget::SetCheckedFocusImage(ImageResource *image) {
  if(image) {
    this->checkedfocusimage = image;
    if(realwidget)
      ((GUI_ToggleButton *)realwidget)->SetOnHighlightImage((GUI_Surface *)(image->GetHandle()));
  }
}

void ToggleButtonWidget::SetEnabled(bool enabled) {
  ToggleButtonWidgetBase::SetEnabled(enabled);
  if(realwidget) {
    if(this->enabled)
      ((GUI_ToggleButton *)realwidget)->ClearFlags(WIDGET_DISABLED);
    else {
      ((GUI_ToggleButton *)realwidget)->SetFlags(WIDGET_DISABLED);
      // If the widget is disabled and has the focus, we want to give the focus to some other widget
      PageBase *parentpage = FindParentPage();
      if(parentpage)
        parentpage->UpdateFocus();
    }
  }
}

void ToggleButtonWidget::SetChecked(bool checked) {
  ToggleButtonWidgetBase::SetChecked(checked);
  if(realwidget) {
    if(this->checked)
      realwidget->SetFlags(WIDGET_TURNED_ON);
    else
      realwidget->ClearFlags(WIDGET_TURNED_ON);
  }
}

void ToggleButtonWidget::OnClick(GUI_Object *sender) {
  if(realwidget->GetFlags() & WIDGET_TURNED_ON)
    this->checked = true;
  else
    this->checked = false;

  FireEvent("OnClick");
}

void ToggleButtonWidget::GotFocus(void) {
  if(!focused) {
    this->FireEvent("OnGotFocus");
    if(normalfocusimage != NULL && checkedfocusimage != NULL) {
      ((GUI_ToggleButton *)realwidget)->SetOnNormalImage((GUI_Surface *)(checkedfocusimage->GetHandle()));
      ((GUI_ToggleButton *)realwidget)->SetOffNormalImage((GUI_Surface *)(normalfocusimage->GetHandle()));
    }
    ebWidget::GotFocus();
  }
}

void ToggleButtonWidget::LostFocus(void) {
  if(focused) {
    this->FireEvent("OnLostFocus");
    if(normalimage != NULL && checkedimage != NULL) {
      ((GUI_ToggleButton *)realwidget)->SetOnNormalImage((GUI_Surface *)(checkedimage->GetHandle()));
      ((GUI_ToggleButton *)realwidget)->SetOffNormalImage((GUI_Surface *)(normalimage->GetHandle()));
    }
    ebWidget::LostFocus();
  }
}

bool ToggleButtonWidget::IsInstantiated(void) {
  return (realwidget != NULL);
}

bool ToggleButtonWidget::IsGUIFocused(void) {
  return ebWidget::IsGUIFocused();
}

bool ToggleButtonWidget::IsEnabled(void) {
  return ebWidget::IsEnabled();
}
