/***************************************************************************
                    widget.cpp  -  eboxy widget parent class
                             -------------------
    begin                : Sat Aug 3 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include "SDL_gui.h"

#include "widget.h"
#include "page.h"

using namespace std;

ebWidget::ebWidget(void) {
  realwidget = NULL;
}

ebWidget::~ebWidget(void) {
  if(realwidget)
    Uninstantiate();
}

void ebWidget::GotFocus(void) {
  if(realwidget) {
    realwidget->SetFlags(WIDGET_INSIDE);
    GUI_GetScreen()->SetFocusWidget(realwidget);
  }
  WidgetBase::GotFocus();
}

void ebWidget::LostFocus(void) {
  if(realwidget)
    realwidget->ClearFlags(WIDGET_INSIDE);
  WidgetBase::LostFocus();
}

bool ebWidget::IsGUIFocused(void) {
  if(realwidget) {
    if(realwidget->GetFlags() & WIDGET_INSIDE)
      return true;
    else
      return false;
  }
  return false;
}

bool ebWidget::IsEnabled(void) {
  if(realwidget) {
    if(!(realwidget->GetFlags() & WIDGET_DISABLED))
      return true;
  }
  return false;
}

void ebWidget::SetPosition(int x, int y) {
  WidgetBase::SetPosition(x, y);
  if(realwidget) {
    SDL_Rect a;
    a = GetArea();
    GUI_Lock();
    realwidget->GetParent()->Erase(&a);
//X    FindParentPage()->InvalidateRect(a);
    realwidget->SetPosition(x, y);
    this->Invalidate();
    GUI_Unlock();
  }
}

void ebWidget::Uninstantiate(void) {
  if(realwidget) {
    realwidget->DecRef();
    realwidget = NULL;
  }
}

GUI_Widget *ebWidget::GetInstance(void) {
  return realwidget;
}

int ebWidget::GetX(void) {
  SDL_Rect a = GetArea();
  return a.x;
}

int ebWidget::GetY(void) {
  SDL_Rect a = GetArea();
  return a.y;
}

int ebWidget::GetWidth(void) {
  SDL_Rect a = GetArea();
  return a.w;
}

int ebWidget::GetHeight(void) {
  SDL_Rect a = GetArea();
  return a.h;
}

SDL_Rect ebWidget::GetArea(void) {
  if(realwidget)
    return realwidget->GetArea();
  else {
    SDL_Rect a;
    a.x = x;
    a.y = y;
    a.w = w;
    a.h = h;
    return a;
  }
}

void ebWidget::Invalidate(void) {
  if(realwidget)
    realwidget->MarkChanged();
}

void ebWidget::InvalidateArea(void) {
  if(realwidget) {
//X    FindParentPage()->InvalidateRect(GetArea());
    realwidget->MarkChanged();
  }
}

void ebWidget::SetVisible(bool visible) {
  PageBase *parentpage;
  GUI_Widget *parentinst;

  if(visible != this->visible) {
    parentpage = FindParentPage();
    if(parentpage) {
      parentinst = (dynamic_cast<ebPage *>(parentpage))->GetInstance();
      if(parentinst) {
        if(visible) {
          ((GUI_Panel *)parentinst)->AddWidget(Instantiate());
        }
        else {
          ((GUI_Panel *)parentinst)->RemoveWidget(realwidget);
          Uninstantiate();
          parentinst->MarkChanged();
        }
      }
    }
    this->visible = visible;
  }
}
