/***************************************************************************
                interfacemanager.cpp  -  interface plugin manager
                             -------------------
    begin                : Sun Jul 13 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>

#include "dlfcn.h"
#include "main.h"

#include "interfacemanager.h"
#include "errors.h"
 
using namespace std;

InterfaceManager::InterfaceManager(void) {
  
}

InterfaceManager::~InterfaceManager(void) {
  
}

int InterfaceManager::LoadInterfacePlugin(const char *name) {
  char libfile[PATH_MAX];
  void *libhandle = NULL;

  // Build path to library
  snprintf(libfile, PATH_MAX, "%s/interface/lib%s%s", PLUGIN_LIB_DIR, name, PLUGIN_LIB_SUFFIX);

  // Make sure library exists
  if(!checkFileExists(libfile))
    return EBERR_FILE_ERROR;
  
  // Load the library
  libhandle = dlopen(libfile, RTLD_LAZY);
  if(libhandle == 0) {
    // Load failed
    char *errorstr = dlerror();
    cerr << "InterfaceManager: Couldn't load interface plugin " << libfile << endl;
    cerr << errorstr << endl;
    return EBERR_DLOPEN_FAILED;  // Failed to load library
  }

	// load the symbols
	ifplugin.createfn = (create_t *) dlsym(libhandle, "create");
	ifplugin.destroyfn = (destroy_t *) dlsym(libhandle, "destroy");
	if (!ifplugin.createfn || !ifplugin.destroyfn) {
		cerr << "InterfaceManager: Cannot load symbols: " << dlerror() << endl;
    dlclose(libhandle);
    ifplugin.createfn = NULL;
    ifplugin.destroyfn = NULL;
		return EBERR_DLOPEN_FAILED;
	}

  ifplugin.handle = libhandle;
  
	// create an instance of the class
	ifplugin.plugin = ifplugin.createfn();

  return EBERR_SUCCESS;
}

void InterfaceManager::UnloadInterfacePlugins(void) {
	// destroy the object
	ifplugin.destroyfn(ifplugin.plugin);

	// unload the library
	if(dlclose(ifplugin.handle))
    cerr << "Failed to unload interface plugin shared library" << endl;
}

InterfacePlugin *InterfaceManager::GetPlugin(void) {
  return ifplugin.plugin;
}

PageBase *InterfaceManager::GetLoadedPage(void) {
  return ifplugin.plugin->GetLoadedPage();
}

PageBase *InterfaceManager::FindPage(const char *name) {
  return ifplugin.plugin->FindPage(name);
}

int InterfaceManager::GetPageCount(void) {
  return ifplugin.plugin->GetPageCount();
}

PageBase *InterfaceManager::GetPage(int index) {
  return ifplugin.plugin->GetPage(index);
}

DynamicObject *InterfaceManager::FindExtPageObject(const char *name) {
  return ifplugin.plugin->FindExtPageObject(name);
}
