/***************************************************************************
                          main.cpp  -  eboxy main source file
                             -------------------
    begin                : Mon Jun 10 23:07:59 NZST 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <iostream>
#include <cstring>
#include <cstdlib>
#include <queue>
#include <map>
#include <string>

#include <ctype.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "getopt.h"

#ifdef HAVE_LIRC
#include <lirc/lirc_client.h>
#endif

#include "basewidgets/widgetbase.h"
#include "basewidgets/textfieldwidgetbase.h"
#include "basewidgets/pagebase.h"
#include "basewidgets/resourcemanagerbase.h"
#include "resolver.h"
#include "systemobject.h"
#include "main.h"

#include "basewidgets/utils.h"
#include "constants.h"
#include "errors.h"
#include "pluginmanager.h"
#include "pluginconstants.h"
#include "sdluserevents.h"
#include "skinparser.h"
#include "conf.h"
#include "basewidgets/actionmanager.h"
#include "processobject.h"
#include "interfacemanager.h"
#include "basewidgets/actionhandler.h"

using namespace std;

extern ResourceManagerBase *resourcemanager;
extern ResolverBase *resolver;
extern ActionManager *actionmanager;

// Global variables
PluginManager plugin_manager;
InterfaceManager interface_manager;
ActionManager action_manager;
vector<keybinding> keybindings;
queue<eboxyevent> eventqueue;
map<string, string> confmap;
vector<ActionHandler *> globalactionhandlers;

#ifdef HAVE_LIRC
pthread_t lirc_thread;
bool disablelirc = false;
#endif
#ifdef JOYSTICK
bool disablejoystick = false;
#endif
bool nokeypassthrough = false;
bool nokeyrepeat = false;
bool showkeys = false;
pthread_mutex_t processmutex;
pthread_mutex_t eventmutex;
pthread_mutex_t switchmutex;
//X pthread_cond_t switchcond;
vector<PageBase *> dynpages;    // runtime-created pages (note these are part of the pages vector also)
vector<WidgetBase *> tempwidgets; // temp area for runtime-created widgets 
vector<Process *> processes;
string xmlFile = "";
PageBase *currentPage = NULL;
SystemObject *systemObject;
bool signalhide = false;
bool hidecursor = false;
Process *hideprocess = NULL;
int counter;
bool fullscreen = false;

// Function prototypes

#ifdef HAVE_LIRC
void *lirc_function(void *arg);
#endif
void *exec_function(void *arg);
void unloadObjects(bool unloadplugins);
void removeFinishedProcesses(void);
void SendTextInput(char c);
void nextItem(void);
void previousItem(void);
void nextItemPage(void);
void previousItemPage(void);
void firstItem(void);
void lastItem(void);
int switchPage(const char *pagename);
int switchFile(std::string filename, std::string pagename);
eboxyevent popEvent(void);
void processEvents(void);
bool handleInterfaceEvent(InterfaceEvent &e);
void readConfig(void);

// Function implementations

int switchPage(const char *pagename) {
  int rc;
  PageBase *page;
  
  InterfacePlugin *ifplugin = interface_manager.GetPlugin();

  if(pagename) {
    // Find requested page
    page = interface_manager.FindPage(pagename);
    if(!page) {
      cerr << "Unable to find page \'" << pagename << "\'" << endl;
      return EBERR_OBJECT_NOT_FOUND;
    }  
  }
  else
    page = NULL;

  // Tell loaded plugins we're about to change the page
  if(currentPage)
    plugin_manager.BroadcastMessage(PLMSG_BEFOREPAGECHANGE, (void *)pagename, NULL);
  else
    plugin_manager.BroadcastMessage(PLMSG_BEFOREFIRSTPAGELOAD, (void *)pagename, NULL);

  // Ask interface plugin to change the page
  rc = ifplugin->SwitchPage(page);
  if(rc)
    return rc;  // error occurred

  if(page)
    currentPage = page;
  else
    currentPage = ifplugin->GetCurrentPage();

  // Tell loaded plugins we're done changing the page
  plugin_manager.BroadcastMessage(PLMSG_AFTERPAGECHANGE, (void *)pagename, NULL);

  // Fire the OnShow event of the page
  currentPage->DoOnShow();

  return EBERR_SUCCESS;
}

int switchFile(std::string filename, std::string pagename) {
  int rc = 0;

  if(!checkFileExists(filename.c_str())) // FIXME
    return EBERR_FILE_ERROR;

  // Tell loaded plugins we're about to load a new file
  plugin_manager.BroadcastMessage(PLMSG_BEFOREFILELOAD, (void *)filename.c_str(), NULL);

  // Clear out current stuff, if any
  unloadObjects(false);
  clearTemplates(false);
  InterfacePlugin *ifplugin = interface_manager.GetPlugin();
  ifplugin->ClearPages(false);

  // Load in new page
  string fullpath = expandPath(filename);
  parseSkin(fullpath, false);

  // Initialise system-level plugins from file
  systemObject->InitialisePlugins();

  if(pagename != "") {
    PageBase *page = interface_manager.FindPage(pagename.c_str());
    rc = ifplugin->SwitchPage(page);
  }
  
  if(pagename == "" || rc != EBERR_SUCCESS)
    rc = ifplugin->SwitchPage(NULL);

  if(currentPage)
    delete currentPage;

  currentPage = ifplugin->GetCurrentPage();

  xmlFile = fullpath;

  // Tell loaded plugins we're done loading a new file
  plugin_manager.BroadcastMessage(PLMSG_AFTERFILELOAD, (void *)filename.c_str(), NULL);

  return EBERR_SUCCESS;
}

// Unload all objects
void unloadObjects(bool unloadsystem) {
  plugin_manager.UnloadAllPlugins(unloadsystem);
  plugin_manager.UnloadAllCustomObjects(unloadsystem);
  systemObject->ClearEventHandlers();
  systemObject->ClearObjects();
  for(int i=globalactionhandlers.size()-1;i>=0;i--) {
    ActionHandler *handler = (ActionHandler *)globalactionhandlers[i];
    if(unloadsystem || !(handler->IsSystem())) {
      actionmanager->UnregisterGlobalAction(handler->GetAction(), handler);
      globalactionhandlers.erase(globalactionhandlers.begin() + i);
      delete handler;
    }
  }
}

void SendTextInput(char c) {
  WidgetBase *widget = currentPage->GetFocusedWidget();

  if(widget != NULL) {
    if(strcmp(widget->GetType(), TYPESTR_TEXTFIELD) == 0) {
      TextFieldWidgetBase *textfield = dynamic_cast<TextFieldWidgetBase *>(widget);
      textfield->SendChar(c, INSMODE_INSERT);
    }
  }
}

void nextWidget(void) {
  currentPage->FocusNextWidget();
}

void prevWidget(void) {
  currentPage->FocusPreviousWidget();
}

// FIXME do we really need all these now we have standard actions?
void acceptSelection(void) {
  WidgetBase *widget = currentPage->GetFocusedWidget();

  if(widget != NULL)
    widget->StandardAction(STDACTION_ACCEPT);
}

void nextItem(void) {
  WidgetBase *widget = currentPage->GetFocusedWidget();

  if(widget != NULL)
    widget->StandardAction(STDACTION_ITEMNEXT);
}

void previousItem(void) {
  WidgetBase *widget = currentPage->GetFocusedWidget();

  if(widget != NULL)
    widget->StandardAction(STDACTION_ITEMPREV);
}

void nextItemPage(void) {
  WidgetBase *widget = currentPage->GetFocusedWidget();

  if(widget != NULL)
    widget->StandardAction(STDACTION_ITEMPAGENEXT);
}

void previousItemPage(void) {
  WidgetBase *widget = currentPage->GetFocusedWidget();

  if(widget != NULL)
    widget->StandardAction(STDACTION_ITEMPAGEPREV);
}

void firstItem(void) {
  WidgetBase *widget = currentPage->GetFocusedWidget();

  if(widget != NULL)
    widget->StandardAction(STDACTION_ITEMFIRST);
}

void lastItem(void) {
  WidgetBase *widget = currentPage->GetFocusedWidget();

  if(widget != NULL)
    widget->StandardAction(STDACTION_ITEMLAST);
}

// These are the thread-safe versions of the above

void nextWidgetEvent(void) {
  pushEvent(EBSE_FOCUSNEXT, NULL, NULL);
}

void prevWidgetEvent(void) {
  pushEvent(EBSE_FOCUSPREVIOUS, NULL, NULL);
}

void acceptSelectionEvent(void) {
  pushEvent(EBSE_SELECTFOCUSED, NULL, NULL);
}

int switchFileEvent(const char *filename, const char *pagename) {
  char *pagename1 = NULL;
  if(pagename)
    pagename1 = strdup(pagename);
  pushEvent(EBSE_LOADFILE, strdup(filename), pagename1);

  // FIXME: this doesn't work
//X  pthread_cond_wait(&switchcond, &switchmutex);
  
  return 0; // FIXME: need to make this synchronous
}

int switchPageEvent(const char *pagename) {
  pushEvent(EBSE_SWITCHPAGE, strdup(pagename), NULL);

  // FIXME: this doesn't work
//X  pthread_cond_wait(&switchcond, &switchmutex);

  return 0; // FIXME: need to make this synchronous
}

int main(int argc, char **argv) {
  // Long-only command-line options - these must be outside the range of char (ie, >255)
  //   so that they don't collide with the short option characters
  enum {
    CLOPT_BPP = 256,
    CLOPT_VERSION,
    CLOPT_HELP,
    CLOPT_NOLIRC,
    CLOPT_NOKEYPT,
    CLOPT_NOKEYREPEAT,
    CLOPT_SHOWKEYS,
    CLOPT_NOJOYSTICK
  };

  string lastpagename;

  readConfig();

  int w = strtoint(confmap["width"]);
  int h = strtoint(confmap["height"]);
  int d = strtoint(confmap["depth"]);
  string ifpluginname = confmap["interfaceplugin"];

  // Parse command line using getopt_long
  while(true) {
    static struct option long_options[] = {
      {"window",     no_argument,       0, 'w'},
      {"fullscreen", no_argument,       0, 'f'},
      {"hidecursor", no_argument,       0, 'h'},
      {"width",      required_argument, 0, 'x'},
      {"height",     required_argument, 0, 'y'},
      {"interfaceplugin",     required_argument, 0, 'i'},
      {"bpp",        required_argument, 0, CLOPT_BPP},
      {"showkeys",   no_argument,       0, CLOPT_SHOWKEYS},
      {"version",    no_argument,       0, CLOPT_VERSION},
      {"nokeypassthrough",    no_argument,       0, CLOPT_NOKEYPT},
      {"nokeyrepeat",no_argument,       0, CLOPT_NOKEYREPEAT},
#ifdef JOYSTICK
      {"nojoystick", no_argument,       0, CLOPT_NOJOYSTICK},
#endif
#ifdef HAVE_LIRC
      {"nolirc",     no_argument,       0, CLOPT_NOLIRC},
#endif
      {"help",       no_argument,       0, CLOPT_HELP},
      {0, 0, 0, 0}
    };

    // getopt_long stores the option index here
    int option_index = 0;
    opterr = 1;

    int c = getopt_long(argc, argv, "wfx:y:hi:", long_options, &option_index);

    // Detect the end of the options
    if (c == -1)
      break;

    switch (c) {
      case 0:
        // Flags (unused currently, although they probably should be)
        break;

      case 'w':
        fullscreen = false;
        break;

      case 'f':
        fullscreen = true;
        break;

      case 'x':
        w = atoi(optarg);
        break;

      case 'y':
        h = atoi(optarg);
        break;

      case 'i':
        ifpluginname = optarg;
        break;

      case 'h':
        hidecursor = true;
        break;

      case CLOPT_HELP:
        cout << "eboxy " << APP_MAJOR_VERSION << "." << APP_MINOR_VERSION << "." << APP_SUB_VERSION << " by Paul Eggleton (bluelightning@bluelightning.org)" << endl;
        cout << endl;
        cout << "  Syntax: eboxy [xmlfile] [options]" << endl;
        cout << "  Options:" << endl;
        cout << "    -w  --window            Use windowed display (default)" << endl;
        cout << "    -f  --fullscreen        Use full-screen display" << endl;
        cout << "    -x  --width             Set width of display" << endl;
        cout << "    -y  --height            Set height of display" << endl;
        cout << "    -h  --hidecursor        Hide the mouse cursor" << endl;
        cout << "    -i  --interfaceplugin   Specify the interface plugin to use" << endl;
        cout << "        --showkeys          Show keyboard key info on key presses" << endl;
        cout << "        --nokeyrepeat       Disable keyboard key repeat" << endl;
        cout << "        --nokeypassthrough  Do not send keys directly to textfields" << endl;
#ifdef JOYSTICK
        cout << "        --nojoystick        Disable joystick support" << endl;
#endif
#ifdef HAVE_LIRC
        cout << "        --nolirc            Disable LIRC support" << endl;
#endif
        cout << endl;
        cout << "  If you do not specify xmlfile, eboxy will look for \'" << DEFAULT_XML_FILE << "\' by default." << endl;
        cout << "  See the eboxy documentation for more information." << endl;
        exit(0);
        break;

      case CLOPT_BPP:
  			d = atoi(optarg);
        break;

      case CLOPT_VERSION:
        cout << "eboxy " << APP_MAJOR_VERSION << "." << APP_MINOR_VERSION << "." << APP_SUB_VERSION << " by Paul Eggleton (bluelightning@bluelightning.org)" << endl;
        cout << "Released under the GNU General Public Licence version 2 or later" << endl;
        exit(0);
        break;

      case CLOPT_NOKEYPT:
        nokeypassthrough = true;
        break;

      case CLOPT_NOKEYREPEAT:
        nokeyrepeat = true;
        break;

      case CLOPT_SHOWKEYS:
        showkeys = true;
        break;

#ifdef JOYSTICK
      case CLOPT_NOJOYSTICK:
        disablejoystick = true;
        break;
#endif

#ifdef HAVE_LIRC
      case CLOPT_NOLIRC:
        disablelirc = true;
        break;
#endif

      case '?':
        break;

      default:
        abort();

    }
  }

  if(argc - optind > 1) {
    cerr << "You can only specify one XML file" << endl;
    cerr << "Run eboxy --help for more information" << endl;
    exit(1);
  }
  else if((argc - optind) == 1) {
    xmlFile = argv[optind];
  }
  else {
    // Check that the default XML file exists
    if(!checkFileExists((char *)DEFAULT_XML_FILE)) {
      cerr << "Run eboxy --help for more information" << endl;
      exit(1);
    }
  }

  // Read key bindings
  if(readKeyConf() != 0) {
    exit(2);
  }

  // FIXME: should search for the first available plugin if none specified
  if(ifpluginname == "")
    ifpluginname = "sdl_gui";
  if(interface_manager.LoadInterfacePlugin(ifpluginname.c_str()))
    exit(3);
  InterfacePlugin *ifplugin = interface_manager.GetPlugin();
  
  actionmanager = &action_manager;
  ifplugin->SetAttributeValue("width", anytovariant(w));
  ifplugin->SetAttributeValue("height", anytovariant(h));
  ifplugin->SetAttributeValue("depth", anytovariant(d));
  ifplugin->SetAttributeValue("nokeypassthrough", anytovariant(nokeypassthrough));
  ifplugin->SetAttributeValue("nokeyrepeat", anytovariant(nokeyrepeat));
  ifplugin->SetAttributeValue("showkeys", anytovariant(showkeys));
  ifplugin->SetAttributeValue("hidecursor", anytovariant(hidecursor));
  ifplugin->SetAttributeValue("fullscreen", anytovariant(fullscreen));
#ifdef JOYSTICK
  ifplugin->SetAttributeValue("nojoystick", anytovariant(disablejoystick));
#endif
  ifplugin->Initialise();

  resolver = new Resolver;
  resourcemanager = ifplugin->GetResourceManager();
  resourcemanager->SetImageSearchPath(confmap["imagepath"]);
  resourcemanager->SetFontSearchPath(confmap["fontpath"]);

  if(xmlFile == "")
    xmlFile = DEFAULT_XML_FILE;

  // Create "system" object
  systemObject = new SystemObject();

  // Initialise mutex for controlling access to processes vector
  pthread_mutex_init(&processmutex, NULL);
  // Initialise mutex for waiting on event processing
  pthread_mutex_init(&eventmutex, NULL);
  // Initialise mutex & condition variable for switching page/file
//X  pthread_mutex_init(&switchmutex, NULL);
//X  pthread_cond_init(&switchcond, NULL);

  // Parse system XML file if it exists
  string systemxmlfile = expandPath("~/.eboxy/system.xml");
  int rc = statfile(systemxmlfile.c_str());
  if(rc==0) 
    parseSkin(systemxmlfile, true);
  else if(rc == EACCES)
    cerr << "File " << systemxmlfile << " could not be read - access denied" << endl;

  if(parseSkin(xmlFile, false) == EBERR_SUCCESS) {
#ifdef HAVE_LIRC
    if(!disablelirc) {
      // Initialise thread for LIRC events
      if(lirc_init((char *) LIRC_APP_NAME,1)==-1) {
        cerr << "Couldn't initialise LIRC (is lircd running?)" << endl;
        disablelirc = true;
      }
      else {
        if(pthread_create(&lirc_thread, NULL, lirc_function, NULL)) {
          lirc_deinit();
          cerr << "Failed to create LIRC thread" << endl;
          disablelirc = true;
        }
      }
    }
#endif

    systemObject->InitialisePlugins();
    systemObject->FireSystemEvent("OnLoad");
    systemObject->AutoStartTimers();

    // Switch to first page
    switchPage(NULL);
  
    while(true) {
      systemObject->FireSystemEvent("OnShow");

      // Transfer to plugin event loop - returns when plugin exits
      ifplugin->BeginEventLoop(&processEvents, &handleInterfaceEvent);

      if(signalhide)
        systemObject->FireSystemEvent("OnHide");
      else
        systemObject->FireSystemEvent("OnQuit");

      if(currentPage) {
        lastpagename = currentPage->GetName();
        currentPage = NULL; // required
      }
      else
        lastpagename = "";

      if(signalhide)
        unloadObjects(false);
      else
        unloadObjects(true);

      ifplugin->Uninitialise();

      if(signalhide) {
        // Hiding
        while(signalhide) {
          // Wait
          if(hideprocess != NULL && hideprocess->IsDone()) {
            signalhide = false;
            break;
          }

          // Don't hog the CPU
          delayms(500);
        }
        if(hideprocess) {
          delete hideprocess;
          hideprocess = NULL;
        }
        // Come back to life
        if(ifplugin->Initialise()) {
          // Something bad happened
          return 1;  
        }
        switchFile(xmlFile, lastpagename);
      }
      else {
        // We're not hiding, we're quitting
        break;
      }
    }

#ifdef HAVE_LIRC
    if(!disablelirc)
      lirc_deinit();
#endif

    // Clear out key bindings
    while(keybindings.size() > 0) {
      free(keybindings.back().action);
      keybindings.pop_back();
    }

    clearTemplates(true);

    delete systemObject;
    
    interface_manager.UnloadInterfacePlugins();
  }
  else {
    // XML parsing failed
    delete resolver;
    resolver = NULL;
    ifplugin->Uninitialise();
    return 1;
  }

  delete resolver;
  resolver = NULL;

  return 0;
}

#ifdef HAVE_LIRC
void *lirc_function(void *arg) {
  struct lirc_config *config;

  if(lirc_readconfig(NULL, &config, NULL) == 0) {
    char *code;
    char *c;
    int ret;

    while(lirc_nextcode(&code)==0) {
      if(code==NULL)
        continue;
      if(signalhide || !(interface_manager.GetPlugin()->HasFocus()))
        continue;  // eat events if the app doesn't have input focus
      while((ret = lirc_code2char(config, code, &c))==0 && c != NULL) {
        performAction(c);
      }

      free(code);
      if(ret == -1) break;
    }
    lirc_freeconfig(config);
  }

  pthread_mutex_destroy(&processmutex);
  pthread_mutex_destroy(&eventmutex);

  return 0;
}
#endif

int performAction(const char *action) {
  // We may be in another thread here so we must consider thread-safety
  if(!strcasecmp(action, "prev")) {
    prevWidgetEvent();
  }
  else if(!strcasecmp(action, "next")) {
    nextWidgetEvent();
  }
  else if(!strcasecmp(action, "accept")) {
    acceptSelectionEvent();
  }
  else if(!strcasecmp(action, "quit")) {
    SignalExit();
  }
  else if(!strcasecmp(action, "item_next")) {
    pushEvent(EBSE_LISTNEXT, NULL, NULL);
  }
  else if(!strcasecmp(action, "item_prev")) {
    pushEvent(EBSE_LISTPREVIOUS, NULL, NULL);
  }
  else if(!strcasecmp(action, "item_nextpage")) {
    pushEvent(EBSE_LISTNEXTPAGE, NULL, NULL);
  }
  else if(!strcasecmp(action, "item_prevpage")) {
    pushEvent(EBSE_LISTPREVIOUSPAGE, NULL, NULL);
  }
  else if(!strcasecmp(action, "item_first")) {
    pushEvent(EBSE_LISTFIRST, NULL, NULL);
  }
  else if(!strcasecmp(action, "item_last")) {
    pushEvent(EBSE_LISTLAST, NULL, NULL);
  }
  else if(!strncasecmp(action, "textinput_", 10)) {
    WidgetBase *widget = currentPage->GetFocusedWidget();
    if(widget != NULL) {
      if(strcmp(widget->GetType(), TYPESTR_TEXTFIELD) == 0) {
        TextFieldWidgetBase *textfield = dynamic_cast<TextFieldWidgetBase *>(widget);

        if(isalnum(action[10]) && strlen(action) == 11) {
          // Single alphanumeric character
          textfield->SendChar(action[10], INSMODE_INSERT);
          // Reset digit input
          textfield->InputDigit(-1);
        }
        else if(!strcasecmp(action+10, "backspace")) {
          textfield->Backspace();
          // Reset digit input
          textfield->InputDigit(-1);
        }
        else if(!strcasecmp(action+10, "delete")) {
          textfield->DeleteCurrChar();
          // Reset digit input
          textfield->InputDigit(-1);
        }
        else if(!strncasecmp(action+10, "digit", 5) && strlen(action+15) == 1 && isdigit(action[15])) {
          // Digits
          textfield->InputDigit(atoi(action+15));
        }
        else {
          // Symbol
          char c = textInputNameToChar(action + 10);
          if(c != '\0') {
            textfield->SendChar(c, INSMODE_INSERT);
            // Reset digit input
            textfield->InputDigit(-1);
          }
          else {
            cerr << "Unrecognised action \"" << action << "\"" << endl;
            return 1;
          }
        }
      }
    }
  }
  else {
    if(GetUserAction(action) != "") {  // FIXME: this is a bit inefficient
      action_manager.ExecuteAction(action);
    }
    else {
      cerr << "Unrecognised action \"" << action << "\"" << endl;
      return 1;
    }
  }
  return 0;
}

void SignalExit(void) {
  interface_manager.GetPlugin()->Quit();
}

void SignalHide(void) {
  signalhide = true;
  interface_manager.GetPlugin()->Quit();
}

// Check to see if a name is a valid identifier
bool checkName(const char *name) {
  if(name == NULL || strlen(name) == 0)
    return false;

  if(name[0] == '_')  // names may not start with an underscore (reserved)
    return false;

  for(unsigned int i=0; i < strlen(name); i++) {
    if(!(isalnum(name[i]) || name[i] == '_'))
      return false;
  }

  // Check for reserved words
  char *chkname = newstring(strlen(name)+3);
  strcpy(chkname, "|");
  strcat(chkname, name);
  strcat(chkname, "|");
  if(strstr("|echo|exec|exechide|execwait|delay|quit|load|loadplugin|page|property|method|event|new|", chkname)) {
    free(chkname);
    return false;
  }
  free(chkname);

  return true;
}

Variant execute(const char *command, bool threaded, bool capture) {
  Process *process = new Process(command);
  process->SetCapture(capture);
  if(threaded) {
    // Save the process object
    pthread_mutex_lock(&processmutex);
    processes.push_back(process);
    pthread_mutex_unlock(&processmutex);
    // Run the process, waiting on it in a separate thread
    process->ExecuteThreaded();
    return VARNULL;
  }
  else {
    // Run the process and wait while it executes
    int retcode = process->Execute();
    std::string output = process->GetOutput();
    delete process;
    if(capture)
      return anytovariant(output);
    else
      return anytovariant(retcode);
  }
}

void executeHide(std::string command) {
  SignalHide();
  hideprocess = new Process(command);
  hideprocess->ExecuteThreaded();
}

void removeFinishedProcesses(void) {
  unsigned int i=0;
  vector<Process *>::iterator p;

  // Go through the processes vector and remove any that are finished

  pthread_mutex_lock(&processmutex);
  while(i<processes.size()) {
    if(processes[i]->IsDone()) {
      // Process is finished, remove it from the vector
      delete processes[i];
      p = processes.begin() + i;
      processes.erase(p);
    }
    else
      i++;
  }
  pthread_mutex_unlock(&processmutex);
}

DynamicObject *findPageObject(const char *name, const char *type) {
  // Find the specified widget/page object
  PageBase *loadedpage = getLoadedPage();
  
  if(!loadedpage)
    return NULL;

  return loadedpage->FindObject(name, type);
}

int checkVersion(eboxy_version_info verinfo, bool is_plugin) {
  // Check a plugin client library version identifier
  if(verinfo.major > APP_MAJOR_VERSION
      || (verinfo.major == APP_MAJOR_VERSION && verinfo.minor > APP_MINOR_VERSION)
      || (verinfo.major == APP_MAJOR_VERSION && verinfo.minor == APP_MINOR_VERSION && verinfo.subminor > APP_SUB_VERSION)) {
    // Too new
    return 1;
  }
  else if((is_plugin) && (verinfo.major < PLUGIN_MIN_MAJOR_VERSION
      || (verinfo.major >= PLUGIN_MIN_MAJOR_VERSION && verinfo.minor < PLUGIN_MIN_MINOR_VERSION)
      || (verinfo.major >= PLUGIN_MIN_MAJOR_VERSION && verinfo.minor >= PLUGIN_MIN_MINOR_VERSION && verinfo.subminor < PLUGIN_MIN_SUB_VERSION))) {
    // Too old
    return -1;
  }
  return 0;
}

const char *getAppVersion(void) {
  return APP_VERSTRING;
}

PageBase *getLoadedPage(void) {
  InterfacePlugin *ifplugin = interface_manager.GetPlugin();
  return ifplugin->GetLoadedPage();
}

void pushEvent(int type, void *data1, void *data2) {
  pthread_mutex_lock(&eventmutex);
  eboxyevent ev;
  ev.type = type;
  ev.data1 = data1;
  ev.data2 = data2;
  eventqueue.push(ev);
  pthread_mutex_unlock(&eventmutex);
}

eboxyevent popEvent(void) {
  pthread_mutex_lock(&eventmutex);
  eboxyevent ev;
  if(eventqueue.size() > 0) {
    ev = eventqueue.front();
    eventqueue.pop();
  }
  else
    ev.type = EBSE_NOEVENT;
  pthread_mutex_unlock(&eventmutex);
  return ev;
}

void processEvents(void) {
  while(true) {
    // Handle "posted" events from other threads
    eboxyevent event = popEvent();
    switch(event.type) {
      case EBSE_NOEVENT:
        break;
      case EBSE_SWITCHPAGE:
        switchPage((const char *)event.data1);
//X              pthread_cond_signal(&switchcond);
        break;
      case EBSE_LOADFILE:
        {
          std::string pagename;
          if(event.data2)
            pagename = (const char *)event.data2;
          else
            pagename = "";
          switchFile((const char *)event.data1, pagename);
//X                pthread_cond_signal(&switchcond);
          break;
        }
      case EBSE_FOCUSNEXT:
        nextWidget();
        break;
      case EBSE_FOCUSPREVIOUS:
        prevWidget();
        break;
      case EBSE_SELECTFOCUSED:
        acceptSelection();
        break;
      case EBSE_LISTNEXT:
        nextItem();
        break;
      case EBSE_LISTPREVIOUS:
        previousItem();
        break;
      case EBSE_LISTNEXTPAGE:
        nextItemPage();
        break;
      case EBSE_LISTPREVIOUSPAGE:
        previousItemPage();
        break;
      case EBSE_LISTFIRST:
        firstItem();
        break;
      case EBSE_LISTLAST:
        lastItem();
        break;
      default:
        cerr << "processEvents: unknown event " << event.type << endl;
        break;
    }
    if(event.type == EBSE_NOEVENT)
      break;
  }

  // Check if any plugins have asked to be unloaded
  plugin_manager.UnloadRequestingPlugins();

  counter++;
  if(counter > 500) {
    // Housekeeping
    removeFinishedProcesses();
    counter = 0;
  }
}

bool handleInterfaceEvent(InterfaceEvent &e) {
  if(e.type == EIT_KEYDOWN
#ifdef JOYSTICK
      || ( !disablejoystick && (e.type == EIT_JOYBUTTONDOWN || e.type == EIT_JOYAXISMOTION || e.type == EIT_JOYHATMOTION))
#endif
      ) {
    if(showkeys)
      cout << "eboxyname=\"" << keyToName(e.key) << "\"" << endl;

    for(unsigned int i=0; i<keybindings.size(); i++) {
      if(e.key == keybindings[i].key && checkmodkeys(e.keymod, keybindings[i].mod)) {
        performAction(keybindings[i].action);
        break;
      }
    }
  }
  return true;
}

void readConfig(void) {
  string conffile = expandPath("~/.eboxy/eboxy.conf");
  
  // Set up recognised config options and their default values
  confmap["width"] = "640";
  confmap["height"] = "480";
  confmap["depth"] = "16";
  confmap["fontpath"] = "";
  confmap["imagepath"] = "";
  confmap["interfaceplugin"] = "";

  // Read the config file
  readConfFile(conffile, confmap, true);
}
