/***************************************************************************
                          processobject.cpp - process wrapper
                             -------------------
    begin                : Sun Oct 6 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <cstring>
#include <cstdio>
#include <cstdlib>
#include <pthread.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

#include "processobject.h"

using namespace std;

// Execute thread function
void *exec_thread_function(void *arg);

Process::Process(const string &cmd) {
  this->cmd = cmd;
  this->done = false;
  this->capture = false;
}

Process::~Process(void) {
  //
}

int Process::Execute(void) {
  done = false;
  output = "";
  pid_t childpid;

  if(capture) {
    // create the pipes
    int fds_poci[2], fds_pico[2];
    if (0 != pipe(fds_poci) || 0 != pipe(fds_pico)) return 1;

    // create the child
    childpid = fork();
    if (childpid == 0) {
      // now executing in the child process
      dup2(fds_poci[0], 0);  // copy reading end to stdin
      dup2(fds_pico[1], 1);  // copy writing end to stdout
      // close the pipes as they're now redundant
      close(fds_poci[0]);
      close(fds_poci[1]);
      close(fds_pico[0]);
      close(fds_pico[1]);
      // execute the command using 'sh -c'
      execlp("sh", "sh", "-c", cmd.c_str(), NULL);
      // if we get here at all then there was an error
      if(errno) {
        done = true;
        perror("execute");
        return -1;
      }
    }

    // in the parent process
    close(fds_poci[0]);  // close reading end
    close(fds_pico[1]);  // close writing end

    // read from child and write into string
    int n;
    char buffer[EXEC_PIPE_BUFFERSIZE];
    while (0 < (n = read(fds_pico[0], buffer, EXEC_PIPE_BUFFERSIZE)))
      output.append(buffer, n);

    close(fds_poci[1]);
    close(fds_pico[0]);

    if(output.length() > 0) {
      // Trim output string
      int startpos = output.find_first_not_of(" \t\n\r", 0);
      int endpos = output.find_last_not_of(" \t\n\r", output.length());
      output = output.substr(startpos, endpos+1 - startpos);
    }
    
    // Obtain return code
    int status;
    wait(&status);
    returncode = WEXITSTATUS(status);
  }
  else
    returncode = system(cmd.c_str());
    
  done = true;
  return returncode;
}

int Process::ExecuteThreaded(void) {
  if(pthread_create(&exec_thread, NULL, exec_thread_function, (void *)this)) {
    cerr << "Failed to create exec thread" << endl;
    return -1;
  }
  return 0;
}

void Process::SetCapture(bool capture) {
  this->capture = capture;
}

bool Process::IsDone(void) {
  return done;
}

int Process::GetReturnCode(void) {
  if(done)
    return returncode;
  else
    return -2;
}

std::string Process::GetOutput(void) {
  return output;
}

void *exec_thread_function(void *arg) {
  return (void *)((Process *)arg)->Execute();
}
