/***************************************************************************
                          script.h  -  header for eboxy scripting system
                             -------------------
    begin                : Sat Sep 7 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INCLUDED__SCRIPT_H
#define INCLUDED__SCRIPT_H
 
#define MAX_STR_CONST        4096
#define SCRIPTFLAGS_TRUSTED     1

#include <iostream>
#include <sstream>
#include <string>

#include "basewidgets/variant.h"

#ifndef E_INCLUDED__FLEXLEXER_H
#define E_INCLUDED__FLEXLEXER_H
#include <FlexLexer.h> // Unfortunately this means you have to have Flex installed to compile this... :(
#endif

#include "script_bison.h"

// Function defined in the Bison input file (script_bison.ypp)
int yylex(YYSTYPE *lvalp, void* parm);

// Lexical analyser (lexer) object
class ScriptLexer: public yyFlexLexer {
  public:
    ScriptLexer(char *string, int flags);
    virtual ~ScriptLexer(void);
    void embedded_yylex(int& result, YYSTYPE *lvalp);
    virtual int yylex(void);
    void SetLineOffset(int line);
    int GetLineNumber(void);
    void SetFileName(const char *filename);
    const char *GetFileName(void);
    bool IsTrusted(void);
  private:
    YYSTYPE semantic_value;
    int lineoffset;
    char *filename;
    std::istream *mem_input;
    int flags;
};

// This object wraps the "execution context" of the parser, which
// make it possible to use a Flex C++ lexer with a reentrant Bison parser
class ParserExecutionContext {
  private:
    ScriptLexer *lexer_;
  public:
    ParserExecutionContext(ScriptLexer *lexer) {
      lexer_ = lexer;
    }
    ScriptLexer *GetLexer(void) {
      return lexer_;
    }
};

void printScriptError(ScriptLexer *lexer, std::string message); // Defined in script_flex.l

#endif
