/***************************************************************************
                     skinparser.h - XML skin file parser header
                             -------------------
    begin                : Thu Nov 7 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INCLUDED__SKINPARSER_H
#define INCLUDED__SKINPARSER_H
 
#include "libxml/xmlmemory.h"
#include "libxml/parser.h"

#include "basewidgets/widgetbase.h"
#include "pluginobject.h"

// Structs
enum TEMPLATE_TYPES {
  TT_BUTTON,
  TT_LABEL,
  TT_PICTURE,
  TT_LISTBOX,
  TT_SCROLLBAR,
  TT_TEXTFIELD,
  TT_TOGGLEBUTTON
};

struct GenericTemplate {
  char *name;
  int type;
  int origin;
  StringList attributes;
};

// Functions

int parseSkin(std::string &filename, bool system);
int parsePages(xmlDocPtr doc, xmlNodePtr node, bool system);
int parseSystem(xmlDocPtr doc, xmlNodePtr node, bool system);
int parseTemplates(xmlDocPtr doc, xmlNodePtr node, bool system);
PageBase *parsePage(xmlDocPtr doc, xmlNodePtr node, bool system);

WidgetBase *parseWidget(xmlDocPtr doc, xmlNodePtr node);

void parseEvent(xmlDocPtr doc, xmlNodePtr node, DynamicObject *parent);
EventTemplate *parseEvent(xmlDocPtr doc, xmlNodePtr node);
TimerObjectBase *parseTimer(xmlDocPtr doc, xmlNodePtr node);
Plugin *parsePlugin(xmlDocPtr doc, xmlNodePtr node, bool system);
ActionHandler *parseActionHandler(xmlDocPtr doc, xmlNodePtr node, bool system);

void clearTemplates(bool clearall);

GenericTemplate *parseTemplate(xmlDocPtr doc, xmlNodePtr node);
GenericTemplate *findTemplate(const char *templatename);
int applyTemplate(WidgetBase *widget, const char *templatename);

WidgetBase *CreateWidget(const char *name, const char *type);

#endif
