/***************************************************************************
                          systemobject.cpp  -  global system object
                             -------------------
    begin                : Thu Sep 26 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <cstring>
#include <cstdlib>

#include "basewidgets/utils.h"
#include "basewidgets/timerbase.h"
#include "basewidgets/memberfunctionproperty.h"
#include "basewidgets/membermethodhandler.h"
#include "interfacemanager.h"
#include "main.h"
#include "pluginconstants.h"

#include "systemobject.h"
#include "pluginmanager.h"

using namespace std;

extern string xmlFile;
extern PluginManager plugin_manager;
extern InterfaceManager interface_manager;
extern vector<PageBase *> pages;
extern const char *TYPESTR_TIMER;
extern const char *TYPESTR_PLUGIN;

SystemObject::SystemObject(void)
: Object("system") {
  this->type = "SystemObject";

  MemberFunctionProperty<SystemObject> *mp;
  mp = new MemberFunctionProperty<SystemObject>("screenwidth", this, &SystemObject::pget_ScreenWidth, NULL, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<SystemObject>("screenheight", this, &SystemObject::pget_ScreenHeight, NULL, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<SystemObject>("currentfile", this, &SystemObject::pget_CurrentFile, NULL, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<SystemObject>("version", this, &SystemObject::pget_Version, NULL, false);
  AddProperty(mp);
  mp = new MemberFunctionProperty<SystemObject>("pagecount", this, &SystemObject::pget_PageCount, NULL, false);
  AddProperty(mp);

  MemberMethodHandler<SystemObject> *mh;
  mh = new MemberMethodHandler<SystemObject>("quit", this, 0, &SystemObject::m_Quit);
  AddMethod(mh);
  mh = new MemberMethodHandler<SystemObject>("focusnext", this, 0, &SystemObject::m_FocusNext);
  AddMethod(mh);
  mh = new MemberMethodHandler<SystemObject>("focusprevious", this, 0, &SystemObject::m_FocusPrevious);
  AddMethod(mh);
  mh = new MemberMethodHandler<SystemObject>("selectfocused", this, 0, &SystemObject::m_SelectFocused);
  AddMethod(mh);
  mh = new MemberMethodHandler<SystemObject>("printpluginlist", this, 0, &SystemObject::m_PrintPluginList);
  AddMethod(mh);
  mh = new MemberMethodHandler<SystemObject>("performaction", this, 1, &SystemObject::m_PerformAction);
  AddMethod(mh);
  mh = new MemberMethodHandler<SystemObject>("getpage", this, 1, &SystemObject::m_GetPage);
  AddMethod(mh);
  mh = new MemberMethodHandler<SystemObject>("gotopage", this, 1, &SystemObject::m_GotoPage);
  AddMethod(mh);
  
  AddEvent("OnLoad");
  AddEvent("OnShow");
  AddEvent("OnHide");
  AddEvent("OnUnload");
}

SystemObject::~SystemObject(void) {
  ClearEventHandlers();
  ClearObjects();
}

int SystemObject::FireSystemEvent(const char *event) {
  // All system events are non-threaded
  return FireEventWait(event);
}

void SystemObject::AutoStartTimers(void) {
  // Start any auto-starting timers
  for(unsigned int i=0;i<containedobjects.size();i++) {
    DynamicObject *object = containedobjects[i];
    if(!strcmp(object->GetType(), TYPESTR_TIMER)) {
      // Start the timer
      (dynamic_cast<TimerObjectBase *>(object))->AutoStart();
    }
  }
}

void SystemObject::InitialisePlugins(void) {
  // Initialise plugins
  for(unsigned int i=0;i<containedobjects.size();i++) {
    DynamicObject *object = containedobjects[i];
    if(!strcmp(object->GetType(), TYPESTR_PLUGIN)) {
      // Initialise the plugin
      if(((Plugin *)object)->GetState() == PLSTATE_DORMANT)
        plugin_manager.InitialisePlugin((Plugin *)object);
    }
  }
}

/* Property get/set functions */

// -- screenwidth --

Variant SystemObject::pget_ScreenWidth(void) {
  return anytovariant(interface_manager.GetPlugin()->GetScreenWidth());
}

// -- screenheight --

Variant SystemObject::pget_ScreenHeight(void) {
  return anytovariant(interface_manager.GetPlugin()->GetScreenHeight());
}

// -- currentfile --

Variant SystemObject::pget_CurrentFile(void) {
  return anytovariant(xmlFile);
}

// -- version --

Variant SystemObject::pget_Version(void) {
  return anytovariant(getAppVersion());
}

// -- pagecount --

Variant SystemObject::pget_PageCount(void) {
  return anytovariant(interface_manager.GetPageCount());
}

/* Method functions */

Variant SystemObject::m_Quit(int numargs, Variant args[]) {
  SignalExit();
  return VARNULL;
}

Variant SystemObject::m_FocusNext(int numargs, Variant args[]) {
  nextWidgetEvent();
  return VARNULL;
}

Variant SystemObject::m_FocusPrevious(int numargs, Variant args[]) {
  prevWidgetEvent();
  return VARNULL;
}

Variant SystemObject::m_SelectFocused(int numargs, Variant args[]) {
  acceptSelectionEvent();
  return VARNULL;
}

Variant SystemObject::m_PrintPluginList(int numargs, Variant args[]) {
  plugin_manager.PrintPluginList();
  return VARNULL;
}

Variant SystemObject::m_PerformAction(int numargs, Variant args[]) {
  string str = args[0];
  performAction(str.c_str());
  return VARNULL;
}

Variant SystemObject::m_GetPage(int numargs, Variant args[]) {
  int index = args[0];
  PageBase *page = interface_manager.GetPage(index);
  if(page)
    return anytovariant(page->GetName());
  else {
    cerr << "system.getpage: index " << index << " is out of bounds" << endl;
    return VARNULL;
  }
}

Variant SystemObject::m_GotoPage(int numargs, Variant args[]) {
  string pagename = args[0];
  switchPageEvent(pagename.c_str());
  return VARNULL;
}
